@@ -59,6 +59,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include <X11/cursorfont.h>
+#include <fixx11h.h>
 
 extern "C" {
 
@@ -330,6 +331,8 @@ main(int argc ATTR_UNUSED, char **argv)
     setenv("KDE_FULL_SESSION", "true", 1);
     setenv("KDE_SESSION_VERSION", "4", 1);
     setenv("DESKTOP_SESSION", "kde", 1); // for qt 4.6 only
+    // fool d-bus, so we get no kbuildsycoca, etc.
+    setenv("DBUS_SESSION_BUS_ADDRESS", "fake", 1);
 
     static char *fakeArgv[] = { (char *)"kdmgreet", 0 };
     static int fakeArgc = as(fakeArgv) - 1;
@@ -343,7 +346,7 @@ main(int argc ATTR_UNUSED, char **argv)
         QCoreApplication::addLibraryPath(dir);
     GreeterApp app(fakeArgc, fakeArgv);
     initQAppConfig();
-    KGlobalSettings::self()->activate();
+    KGlobalSettings::self()->activate(KGlobalSettings::ApplySettings);
 
     Display *dpy = QX11Info::display();
     QDesktopWidget *dw = app.desktop();
