@@ -53,7 +53,9 @@ public:
 signals:
     void changed( QClipboard::Mode m );
 protected:
+#ifdef Q_WS_X11
     virtual bool x11Event( XEvent* e );
+#endif
 private:
     int m_xfixes_event_base;
     unsigned int m_xa_clipboard;
@@ -97,6 +99,9 @@ public Q_SLOTS:
     void slotHistoryTopChanged();
     void slotConfigure();
     void slotEditData();
+#ifdef HAVE_DMTX
+    void slotShowBarcode();
+#endif
     void slotCycleNext();
     void slotCyclePrev();
 
@@ -196,12 +201,14 @@ private:
     QAction* m_clearHistoryAction;
     QAction* m_repeatAction;
     QAction* m_editAction;
+#ifdef HAVE_DMTX
+    QAction* m_showBarcodeAction;
+#endif
     QAction* m_configureAction;
     QAction* m_quitAction;
     KAction* m_cycleNextAction;
     KAction* m_cyclePrevAction;
     QPixmap m_pixmap;
-    bool m_bPopupAtMouse :1;
     bool m_bKeepContents :1;
     bool m_bURLGrabber   :1;
     bool m_bReplayActionInHistory :1;
