@@ -34,6 +34,8 @@
 #include <NETRootInfo>
 #endif
 
+#include <QtCore/QStringBuilder> // % operator for QString
+
 #include "kworkspace/kdisplaymanager.h"
 
 #include <Plasma/AbstractRunner>
@@ -65,17 +67,17 @@ KRunnerDialog::KRunnerDialog(Plasma::RunnerManager *runnerManager, QWidget *pare
     setMouseTracking(true);
     //setButtons(0);
     setWindowTitle(i18n("Run Command"));
-    setWindowIcon(KIcon("system-run"));
+    setWindowIcon(KIcon(QLatin1String( "system-run" )));
 
     QPalette pal = palette();
     pal.setColor(backgroundRole(), Qt::transparent);
     setPalette(pal);
 
     m_iconSvg = new Plasma::Svg(this);
-    m_iconSvg->setImagePath("widgets/configuration-icons");
+    m_iconSvg->setImagePath(QLatin1String( "widgets/configuration-icons" ));
 
     m_background = new Plasma::FrameSvg(this);
-    m_background->setImagePath("dialogs/krunner");
+    m_background->setImagePath(QLatin1String( "dialogs/krunner" ));
     setFreeFloating(KRunnerSettings::freeFloating());
 
     connect(Kephal::Screens::self(), SIGNAL(screenRemoved(int)),
@@ -99,7 +101,7 @@ KRunnerDialog::~KRunnerDialog()
         while (it.hasNext()) {
             it.next();
             //kDebug() << "saving" << "Screen" + QString::number(it.key()) << it.value();
-            cg.writeEntry("Screen" + QString::number(it.key()), it.value());
+            cg.writeEntry(QLatin1String( "Screen" ) % QString::number(it.key()), it.value());
         }
     }
 }
@@ -197,6 +199,8 @@ void KRunnerDialog::positionOnScreen()
 
     if (m_floating) {
         KWindowSystem::setOnDesktop(winId(), KWindowSystem::currentDesktop());
+        //Turn the sliding effect off
+        Plasma::WindowEffects::slideWindow(this, Plasma::Floating);
     } else {
         KWindowSystem::setOnAllDesktops(winId(), true);
         Plasma::WindowEffects::slideWindow(this, Plasma::TopEdge);
@@ -230,7 +234,7 @@ void KRunnerDialog::setFreeFloating(bool floating)
         const int numScreens = Kephal::ScreenUtils::numScreens();
         KConfigGroup cg(KGlobal::config(), "EdgePositions");
         for (int i = 0; i < numScreens; ++i) {
-            QPoint p = cg.readEntry("Screen" + QString::number(i), QPoint());
+            QPoint p = cg.readEntry(QLatin1String( "Screen" ) % QString::number(i), QPoint());
             if (!p.isNull()) {
                 QRect r = Kephal::ScreenUtils::screenGeometry(i);
                 m_screenPos.insert(i, QPoint(p.x(), r.top()));
@@ -264,36 +268,6 @@ void KRunnerDialog::setStaticQueryMode(bool staticQuery)
     Q_UNUSED(staticQuery)
 }
 
-void KRunnerDialog::switchUser()
-{
-    const KService::Ptr service = KService::serviceByStorageId("plasma-runner-sessions.desktop");
-    KPluginInfo info(service);
-
-    if (info.isValid()) {
-        SessList sessions;
-        KDisplayManager dm;
-        dm.localSessions(sessions);
-
-        if (sessions.isEmpty()) {
-            // no sessions to switch between, let's just start up another session directly
-            Plasma::AbstractRunner *sessionRunner = m_runnerManager->runner(info.pluginName());
-            if (sessionRunner) {
-                Plasma::QueryMatch switcher(sessionRunner);
-                sessionRunner->run(*m_runnerManager->searchContext(), switcher);
-            }
-        } else {
-            display(QString());
-            //TODO: create a "single runner" mode
-            //m_header->setText(i18n("Switch users"));
-            //m_header->setPixmap("system-switch-user");
-
-            //TODO: ugh, magic strings. See sessions/sessionrunner.cpp
-            setStaticQueryMode(true);
-            m_runnerManager->launchQuery("SESSIONS", info.pluginName());
-        }
-    }
-}
-
 void KRunnerDialog::toggleConfigDialog()
 {
     if (m_configWidget) {
