@@ -63,6 +63,8 @@
 #include <QDBusInterface>
 #include <QDBusServiceWatcher>
 
+#include <QtCore/QStringBuilder> // % operator for QString
+
 #include <QDateTime>
 
 #include <stdlib.h>
@@ -126,7 +128,7 @@ int XSelectInput( Display* dpy, Window w, long e )
 }
 #endif
 
-static QString s_overlayServiceName("org.kde.plasma-overlay");
+static QLatin1String s_overlayServiceName("org.kde.plasma-overlay");
 
 //===========================================================================
 //
@@ -135,6 +137,7 @@ static QString s_overlayServiceName("org.kde.plasma-overlay");
 //
 LockProcess::LockProcess(bool child, bool useBlankOnly)
     : QWidget(0L, Qt::X11BypassWindowManagerHint),
+      mInitialLock(false),
       mLocked(false),
       mBusy(false),
       mPlasmaDBus(0),
@@ -146,16 +149,17 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
       mUseBlankOnly(useBlankOnly),
       mSuspended(false),
       mVisibility(false),
+      mEventRecursed(false),
       mRestoreXF86Lock(false),
       mForbidden(false),
       mAutoLogoutTimerId(0)
 {
-    setObjectName("save window");
+    setObjectName(QLatin1String( "save window" ));
     setupSignals();
 
     new LockProcessAdaptor(this);
-    QDBusConnection::sessionBus().registerService("org.kde.screenlocker");
-    QDBusConnection::sessionBus().registerObject("/LockProcess", this);
+    QDBusConnection::sessionBus().registerService(QLatin1String( "org.kde.screenlocker" ));
+    QDBusConnection::sessionBus().registerObject(QLatin1String( "/LockProcess" ), this);
 
     kapp->installX11EventFilter(this);
 
@@ -183,9 +187,9 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
     connect(&mSuspendTimer, SIGNAL(timeout()), SLOT(suspend()));
 
     const QStringList dmopt =
-        QString::fromLatin1( ::getenv( "XDM_MANAGED" )).split(QChar(','), QString::SkipEmptyParts);
+        QString::fromLatin1( ::getenv( "XDM_MANAGED" )).split(QLatin1Char(','), QString::SkipEmptyParts);
     for (QStringList::ConstIterator it = dmopt.constBegin(); it != dmopt.constEnd(); ++it)
-        if ((*it).startsWith("method="))
+        if ((*it).startsWith(QLatin1String( "method=" )))
             mMethod = (*it).mid(7);
 
     configure();
@@ -257,6 +261,12 @@ static void sighup_handler(int)
     ::write( signal_pipe[1], &tmp, 1);
 }
 
+static void sigusr1_handler(int)
+{
+    char tmp = '1';
+    ::write(signal_pipe[1], &tmp, 1);
+}
+
 void LockProcess::timerEvent(QTimerEvent *ev)
 {
     if (ev->timerId() == mAutoLogoutTimerId)
@@ -283,6 +293,9 @@ void LockProcess::setupSignals()
     // SIGHUP forces lock
     act.sa_handler= sighup_handler;
     sigaction(SIGHUP, &act, 0L);
+    // SIGUSR1 simulates user activity
+    act.sa_handler= sigusr1_handler;
+    sigaction(SIGUSR1, &act, 0L);
 
     pipe(signal_pipe);
     QSocketNotifier* notif = new QSocketNotifier(signal_pipe[0], QSocketNotifier::Read, this);
@@ -294,16 +307,22 @@ void LockProcess::signalPipeSignal()
 {
     char tmp;
     ::read( signal_pipe[0], &tmp, 1);
-    if( tmp == 'T' )
+    if (tmp == 'T') {
         quitSaver();
-    else if( tmp == 'H' ) {
+    } else if (tmp == '1') {
+        if (!mBusy && mDialogs.isEmpty()) {
+            mBusy = true;
+            quit();
+            mBusy = false;
+        }
+    } else if (tmp == 'H') {
         if( !mLocked )
             startLock();
     }
 }
 
 //---------------------------------------------------------------------------
-bool LockProcess::lock()
+bool LockProcess::lock(bool initial)
 {
     if (startSaver()) {
         // In case of a forced lock we don't react to events during
@@ -312,6 +331,7 @@ bool LockProcess::lock()
         // the screensaver kicks in because the user moved the mouse after
         // selecting "lock screen", that looks really untidy.
         mBusy = true;
+        mInitialLock = initial;
         if (startLock())
         {
             QTimer::singleShot(1000, this, SLOT(slotDeadTimePassed()));
@@ -325,6 +345,8 @@ bool LockProcess::lock()
 //---------------------------------------------------------------------------
 void LockProcess::slotDeadTimePassed()
 {
+    if (mInitialLock)
+        quit();
     mBusy = false;
 }
 
@@ -392,7 +414,7 @@ void LockProcess::configure()
 
     mSaver = KScreenSaverSettings::saver();
     if (mSaver.isEmpty() || mUseBlankOnly) {
-        mSaver = "kblank.desktop";
+        mSaver = QLatin1String( "kblank.desktop" );
     }
 
     readSaver();
@@ -404,7 +426,7 @@ void LockProcess::configure()
 
     mPlugins = KScreenSaverSettings::pluginsUnlock();
     if (mPlugins.isEmpty()) {
-        mPlugins << "classic" << "generic";
+        mPlugins << QLatin1String( "classic" ) << QLatin1String( "generic" );
     }
     mPluginOptions = KScreenSaverSettings::pluginOptions();
 }
@@ -418,10 +440,10 @@ void LockProcess::readSaver()
     if (!mSaver.isEmpty())
     {
         QString entryName = mSaver;
-        if( entryName.endsWith( ".desktop" ))
+        if( entryName.endsWith( QLatin1String( ".desktop" ) ))
             entryName = entryName.left( entryName.length() - 8 ); // strip it
-        const KService::List offers = KServiceTypeTrader::self()->query( "ScreenSaver",
-            "DesktopEntryName == '" + entryName.toLower() + '\'' );
+        const KService::List offers = KServiceTypeTrader::self()->query( QLatin1String( "ScreenSaver" ),
+            QLatin1String( "DesktopEntryName == '" ) + entryName.toLower() + QLatin1Char( '\'' ) );
         if( offers.isEmpty() )
         {
             kDebug(1204) << "Cannot find screesaver: " << mSaver;
@@ -429,39 +451,30 @@ void LockProcess::readSaver()
         }
         const QString file = KStandardDirs::locate("services", offers.first()->entryPath());
 
-        const bool opengl = KAuthorized::authorizeKAction("opengl_screensavers");
-        const bool manipulatescreen = KAuthorized::authorizeKAction("manipulatescreen_screensavers");
+        const bool opengl = KAuthorized::authorizeKAction(QLatin1String( "opengl_screensavers" ));
+        const bool manipulatescreen = KAuthorized::authorizeKAction(QLatin1String( "manipulatescreen_screensavers" ));
         KDesktopFile config( file );
         KConfigGroup desktopGroup = config.desktopGroup();
-        if (!desktopGroup.readEntry("X-KDE-Type").toUtf8().isEmpty())
-        {
-            const QString saverType = desktopGroup.readEntry("X-KDE-Type").toUtf8();
-            const QStringList saverTypes = saverType.split( ";");
-            const int nbSaverTypes = saverTypes.count();
-            for (int i = 0; i < nbSaverTypes ; ++i)
-            {
-                if ((saverTypes[i] == "ManipulateScreen") && !manipulatescreen)
-                {
+        foreach (const QString &type, desktopGroup.readEntry("X-KDE-Type").split(QLatin1Char(';'))) {
+            if (type == QLatin1String("ManipulateScreen")) {
+                if (!manipulatescreen) {
                     kDebug(1204) << "Screensaver is type ManipulateScreen and ManipulateScreen is forbidden";
                     mForbidden = true;
                 }
-                if ((saverTypes[i] == "OpenGL") && !opengl)
-                {
+            } else if (type == QLatin1String("OpenGL")) {
+                mOpenGLVisual = true;
+                if (!opengl) {
                     kDebug(1204) << "Screensaver is type OpenGL and OpenGL is forbidden";
                     mForbidden = true;
                 }
-                if (saverTypes[i] == "OpenGL")
-                {
-                    mOpenGLVisual = true;
-                }
             }
         }
 
         kDebug(1204) << "mForbidden: " << (mForbidden ? "true" : "false");
 
-        if (config.hasActionGroup("Root"))
+        if (config.hasActionGroup(QLatin1String( "Root" )))
         {
-            mSaverExec = config.actionGroup("Root").readPathEntry("Exec", QString());
+            mSaverExec = config.actionGroup(QLatin1String( "Root" )).readPathEntry("Exec", QString());
         }
     }
 }
@@ -770,7 +783,7 @@ bool LockProcess::startSaver()
     setVRoot( winId(), winId() );
     startHack();
     startPlasma();
-    KNotification::event( "savingstarted" );
+    KNotification::event( QLatin1String( "savingstarted" ) );
     return true;
 }
 
@@ -790,21 +803,21 @@ void LockProcess::stopSaver()
         if (mLocked) {
             KDisplayManager().setLock( false );
             mLocked = false;
-            KNotification::event( "unlocked" );
+            KNotification::event( QLatin1String( "unlocked" ) );
         }
         ungrabInput();
         const char *out = "GOAWAY!";
         for (QList<int>::ConstIterator it = child_sockets.constBegin(); it != child_sockets.constEnd(); ++it)
             write(*it, out, sizeof(out));
     }
-    KNotification::event( "savingstopped" );
+    KNotification::event( QLatin1String( "savingstopped" ) );
 }
 
 // private static
 QVariant LockProcess::getConf(void *ctx, const char *key, const QVariant &dflt)
 {
     LockProcess *that = (LockProcess *)ctx;
-    QString fkey = QLatin1String( key ) + '=';
+    QString fkey = QLatin1String( key ) % QLatin1Char( '=' );
     for (QStringList::ConstIterator it = that->mPluginOptions.constBegin();
          it != that->mPluginOptions.constEnd(); ++it)
         if ((*it).startsWith( fkey ))
@@ -832,7 +845,7 @@ bool LockProcess::startLock()
         mLocked = true;
         KDisplayManager().setLock(true);
         lockPlasma();
-        KNotification::event( "locked" );
+        KNotification::event( QLatin1String( "locked" ) );
         return true;
     }
     return false;
@@ -847,7 +860,7 @@ bool LockProcess::loadGreetPlugin()
     }
     for (QStringList::ConstIterator it = mPlugins.constBegin(); it != mPlugins.constEnd(); ++it) {
         GreeterPluginHandle plugin;
-        KLibrary *lib = new KLibrary( (*it)[0] == '/' ? *it : "kgreet_" + *it );
+        KLibrary *lib = new KLibrary( (*it)[0] == QLatin1Char( '/' ) ? *it : QLatin1String( "kgreet_" ) + *it );
         if (lib->fileName().isEmpty()) {
             kWarning(1204) << "GreeterPlugin " << *it << " does not exist" ;
             delete lib;
@@ -866,7 +879,7 @@ bool LockProcess::loadGreetPlugin()
             delete lib;
             continue;
         }
-        if (plugin.info->method && !mMethod.isEmpty() && mMethod != plugin.info->method) {
+        if (plugin.info->method && !mMethod.isEmpty() && mMethod != QLatin1String(  plugin.info->method )) {
             kDebug(1204) << "GreeterPlugin " << *it << " (" << lib->fileName() << ") serves " << plugin.info->method << ", not " << mMethod;
             lib->unload();
             delete lib;
@@ -901,7 +914,7 @@ bool LockProcess::startHack()
     }
 
     QHash<QChar, QString> keyMap;
-    keyMap.insert('w', QString::number(winId()));
+    keyMap.insert(QLatin1Char( 'w' ), QString::number(winId()));
     mHackProc << KShell::splitArgs(KMacroExpander::expandMacrosShellQuote(mSaverExec, keyMap));
 
     mHackProc.start();
@@ -957,14 +970,14 @@ bool LockProcess::startPlasma()
     if (!mPlasmaDBus) {
         //try to get it, in case it's already running somehow
         //mPlasmaDBus = new QDBusInterface(s_overlayServiceName, "/MainApplication", QString(),
-        mPlasmaDBus = new org::kde::plasmaoverlay::App(s_overlayServiceName, "/App",
+        mPlasmaDBus = new org::kde::plasmaoverlay::App(s_overlayServiceName, QLatin1String( "/App" ),
                                                        QDBusConnection::sessionBus(), this);
         //FIXME this might-already-be-running stuff seems really really Wrong.
     }
 
     if (mPlasmaDBus->isValid()) {
         kDebug() << "weird, plasma-overlay is already running";
-        mPlasmaDBus->call(QDBus::NoBlock, "setup", mSetupMode);
+        mPlasmaDBus->call(QDBus::NoBlock, QLatin1String( "setup" ), mSetupMode);
         return true;
     }
 
@@ -980,9 +993,9 @@ bool LockProcess::startPlasma()
     }
 
     KProcess *plasmaProc = new KProcess;
-    plasmaProc->setProgram("plasma-overlay");
+    plasmaProc->setProgram(QLatin1String( "plasma-overlay" ));
     if (mSetupMode) {
-        *plasmaProc << "--setup";
+        *plasmaProc << QLatin1String( "--setup" );
     }
 
     //make sure it goes away when it's done (and not before)
@@ -1044,7 +1057,7 @@ void LockProcess::disablePlasma()
 void LockProcess::stopPlasma()
 {
     if (mPlasmaDBus && mPlasmaDBus->isValid()) {
-        mPlasmaDBus->call(QDBus::NoBlock, "quit");
+        mPlasmaDBus->call(QDBus::NoBlock, QLatin1String( "quit" ));
     } else {
         kDebug() << "cannot stop plasma-overlay";
     }
@@ -1066,21 +1079,21 @@ void LockProcess::newService(QString name, QString oldOwner, QString newOwner)
     }
 
     kDebug() << "plasma! yaay!";
-    mPlasmaDBus = new org::kde::plasmaoverlay::App(s_overlayServiceName, "/App", QDBusConnection::sessionBus(), this);
+    mPlasmaDBus = new org::kde::plasmaoverlay::App(s_overlayServiceName, QLatin1String( "/App" ), QDBusConnection::sessionBus(), this);
 
     //XXX this isn't actually used any more iirc
     connect(mPlasmaDBus, SIGNAL(hidden()), SLOT(unSuppressUnlock()));
 
     if (!mDialogs.isEmpty()) {
         //whoops, activation probably failed earlier
-        mPlasmaDBus->call(QDBus::NoBlock, "setActive", true);
+        mPlasmaDBus->call(QDBus::NoBlock, QLatin1String( "setActive" ), true);
     }
 }
 
 void LockProcess::deactivatePlasma()
 {
     if (isPlasmaValid()) {
-        mPlasmaDBus->call(QDBus::NoBlock, "setActive", false);
+        mPlasmaDBus->call(QDBus::NoBlock, QLatin1String( "setActive" ), false);
     }
     if (!mLocked && mLockGrace >=0) {
         QTimer::singleShot(mLockGrace, this, SLOT(startLock())); //this is only ok because any activity will quit
@@ -1090,7 +1103,7 @@ void LockProcess::deactivatePlasma()
 void LockProcess::lockPlasma()
 {
     if (isPlasmaValid()) {
-        mPlasmaDBus->call(QDBus::NoBlock, "lock");
+        mPlasmaDBus->call(QDBus::NoBlock, QLatin1String( "lock" ));
     }
 }
 
@@ -1154,7 +1167,7 @@ void LockProcess::resume( bool force )
 bool LockProcess::checkPass()
 {
     if (isPlasmaValid()) {
-        mPlasmaDBus->call(QDBus::NoBlock, "setActive", true);
+        mPlasmaDBus->call(QDBus::NoBlock, QLatin1String( "setActive" ), true);
     }
 
     PasswordDlg passDlg( this, &greetPlugin);
@@ -1164,7 +1177,7 @@ bool LockProcess::checkPass()
         if (ret == QDialog::Rejected) {
             mSuppressUnlock.start(mSuppressUnlockTimeout);
         } else if (ret == TIMEOUT_CODE) {
-            mPlasmaDBus->call(QDBus::NoBlock, "setActive", false);
+            mPlasmaDBus->call(QDBus::NoBlock, QLatin1String( "setActive" ), false);
         }
     }
 
@@ -1198,7 +1211,7 @@ bool LockProcess::checkPass(const QString &reason)
         //so that the user doesn't have to type their password twice
         mLocked = false;
         KDisplayManager().setLock(false);
-        KNotification::event( "unlocked" );
+        KNotification::event( QLatin1String( "unlocked" ) );
         //FIXME while suppressUnlock *should* always be running, if it isn't
         //(say if someone's doing things they shouldn't with dbus) then it won't get started by this
         //which means that a successful unlock will never re-lock
@@ -1307,6 +1320,9 @@ void LockProcess::updateFocus()
 //
 bool LockProcess::x11Event(XEvent *event)
 {
+    if (mEventRecursed)
+        return false;
+
     bool ret = false;
     switch (event->type)
     {
@@ -1340,10 +1356,35 @@ bool LockProcess::x11Event(XEvent *event)
                     killTimer(mAutoLogoutTimerId);
                     mAutoLogoutTimerId = startTimer(mAutoLogoutTimeout * 1000);
                 }
-            } else if (!mLocked || checkPass()) {
+            } else if (!mLocked) {
                 quitSaver();
                 mBusy = false;
                 return true; //it's better not to forward any input while quitting, right?
+            } else {
+                if (event->type == KeyPress) {
+                    // Bounce the keypress to the dialog
+                    QByteArray chars;
+                    chars.resize(513);
+                    KeySym keysym;
+                    XLookupString(&event->xkey, chars.data(), chars.size(), &keysym, 0);
+                    switch (keysym) {
+                    // These would cause immediate failure
+                    case XK_Escape:
+                    case XK_Return:
+                    case XK_KP_Enter:
+                    // These just make no sense
+                    case XK_Tab:
+                    case XK_space:
+                        break;
+                    default:
+                        mEventQueue.enqueue(*event);
+                    }
+                }
+                if (checkPass()) {
+                    quitSaver();
+                    mBusy = false;
+                    return true; //it's better not to forward any input while quitting, right?
+                }
             }
             mBusy = false;
             ret = true;
@@ -1399,6 +1440,8 @@ bool LockProcess::x11Event(XEvent *event)
         case MapNotify: // from SubstructureNotifyMask on the root window
             if( event->xmap.event == QX11Info::appRootWindow()) {
                 kDebug(1204) << "MapNotify:" << event->xmap.window;
+                if (!mDialogs.isEmpty() && mDialogs.first()->winId() == event->xmap.window)
+                    mVisibleDialogs.append(event->xmap.window);
                 int index = findWindowInfo( event->xmap.window );
                 if( index >= 0 )
                     windowInfo[ index ].viewable = true;
@@ -1436,6 +1479,7 @@ bool LockProcess::x11Event(XEvent *event)
                     windowInfo[ index ].viewable = false;
                 else
                     kDebug(1204) << "Unknown toplevel for MapNotify";
+                mVisibleDialogs.removeAll(event->xunmap.window);
                 mForeignWindows.removeAll(event->xunmap.window);
                 if (mForeignInputWindows.removeAll(event->xunmap.window)) {
                     updateFocus();
@@ -1504,23 +1548,29 @@ bool LockProcess::x11Event(XEvent *event)
     // so let's simply dupe it with correct destination window,
     // and ignore the original one.
     if (!mDialogs.isEmpty()) {
-        if ((event->type == KeyPress || event->type == KeyRelease) &&
-                event->xkey.window != mDialogs.first()->winId()) {
-            //kDebug() << "forward to dialog";
-            XEvent ev2 = *event;
-            ev2.xkey.window = ev2.xkey.subwindow = mDialogs.first()->winId();
-            qApp->x11ProcessEvent( &ev2 );
+        if (event->type == KeyPress || event->type == KeyRelease) {
+            mEventQueue.enqueue(*event);
             ret = true;
         }
-    } else if (!mForeignInputWindows.isEmpty()) {
-        //when there are no dialogs, forward some events to plasma
-        switch (event->type) {
-        case KeyPress:
-        case KeyRelease:
-        case ButtonPress:
-        case ButtonRelease:
-        case MotionNotify:
-            {
+        if (!mVisibleDialogs.isEmpty())
+            while (!mEventQueue.isEmpty()) {
+                //kDebug() << "forward to dialog";
+                XEvent ev2 = mEventQueue.dequeue();
+                ev2.xkey.window = ev2.xkey.subwindow = mVisibleDialogs.last();
+                mEventRecursed = true;
+                qApp->x11ProcessEvent( &ev2 );
+                mEventRecursed = false;
+            }
+    } else {
+        mEventQueue.clear();
+        if (!mForeignInputWindows.isEmpty()) {
+            //when there are no dialogs, forward some events to plasma
+            switch (event->type) {
+            case KeyPress:
+            case KeyRelease:
+            case ButtonPress:
+            case ButtonRelease:
+            case MotionNotify: {
                 //kDebug() << "forward to plasma";
                 XEvent ev2 = *event;
                 Window root_return;
@@ -1551,9 +1601,10 @@ bool LockProcess::x11Event(XEvent *event)
                 }
                 XSendEvent(QX11Info::display(), targetWindow, False, NoEventMask, &ev2);
                 ret = true;
+                break; }
+            default:
+                break;
             }
-        default:
-            break;
         }
     }
 
