@@ -114,7 +114,12 @@ namespace Oxygen
 
         //! addLine opacity
         virtual void setAddLineOpacity( qreal value )
-        { addLineData_.opacity_ = digitize( value ); }
+        {
+            value = digitize( value );
+            if( addLineData_.opacity_ == value ) return;
+            addLineData_.opacity_ = value;
+            setDirty();
+        }
 
         //! addLine opacity
         virtual qreal addLineOpacity( void ) const
@@ -122,7 +127,12 @@ namespace Oxygen
 
         //! subLine opacity
         virtual void setSubLineOpacity( qreal value )
-        { subLineData_.opacity_ = digitize( value ); }
+        {
+            value = digitize( value );
+            if( subLineData_.opacity_ == value ) return;
+            subLineData_.opacity_ = value;
+            setDirty();
+        }
 
         //! subLine opacity
         virtual qreal subLineOpacity( void ) const
