@@ -23,7 +23,6 @@
 #ifndef KSG_MODULES_H
 #define KSG_MODULES_H
 
-#include <config-ksysguardd.h>
 #include "Command.h"
 #include "conf.h"
 #include "ksysguardd.h"
@@ -49,16 +48,19 @@
 
 #if defined(OSTYPE_FreeBSD) || defined(OSTYPE_DragonFly)
 #include <grp.h>
+#include "acpi.h"
 #ifdef __i386__
  #include "apm.h"
 #endif
-#include "CPU.h"
+#include "cpuinfo.h"
 #include "diskstat.h"
 #include "loadavg.h"
 #include "logfile.h"
 #include "Memory.h"
 #include "netdev.h"
 #include "ProcessList.h"
+#include "stat.h"
+#include "uptime.h"
 #endif /* OSTYPE_FreeBSD */
 
 #ifdef OSTYPE_Irix
@@ -111,6 +113,14 @@ typedef int (*IVFunc)( void );
 #define NULLIVFUNC ((IVFunc) 0)
 #define NULLTIME ((unsigned long long) 0)
 
+/* Here we state all the available "modules" for a system.
+ * 1. configName    - Just a name that it associated with it
+ * 2. initCommand   - The function that will be called once when ksysguardd is started
+ * 3. extiCommand   - The function that will be called once when ksysguardd is closed, if it's closed nicely
+ * 4. updateCommand - The function that will be called when any of the functions for that module are called
+ * 5. checkCommand  - The function that will be called periodically after 5 seconds, when any next command is issued
+ * 6. available     - Used internally - set to 0 here
+ * 7. timeCentiSeconds - Used internally - set to NULLTIME here */
 struct SensorModul SensorModulList[] = {
 #ifdef OSTYPE_Linux
   { "Acpi", initAcpi, exitAcpi, updateAcpi, NULLVVFUNC, 0, NULLTIME },
@@ -125,7 +135,7 @@ struct SensorModul SensorModulList[] = {
   { "LoadAvg", initLoadAvg, exitLoadAvg, updateLoadAvg, NULLVVFUNC, 0, NULLTIME },
   { "LogFile", initLogFile, exitLogFile, NULLIVFUNC, NULLVVFUNC, 0, NULLTIME },
   { "Memory", initMemory, exitMemory, updateMemory, NULLVVFUNC, 0, NULLTIME },
-  { "NetDev", initNetDev, exitNetDev, updateNetDev, NULLVVFUNC, 0, NULLTIME },
+  { "NetDev", initNetDev, exitNetDev, updateNetDev, checkNetDev, 0, NULLTIME },
   { "NetStat", initNetStat, exitNetStat, NULLIVFUNC, NULLVVFUNC, 0, NULLTIME },
   { "ProcessList", initProcessList, exitProcessList, NULLIVFUNC, NULLVVFUNC, 0, NULLTIME },
   { "Stat", initStat, exitStat, updateStat, NULLVVFUNC, 0, NULLTIME },
@@ -134,6 +144,7 @@ struct SensorModul SensorModulList[] = {
 #endif /* OSTYPE_Linux */
 
 #if defined OSTYPE_FreeBSD || defined OSTYPE_DragonFly
+  { "Acpi", initACPI, exitACPI, updateACPI, NULLVVFUNC, 0, NULLTIME },
   #ifdef __i386__
     { "Apm", initApm, exitApm, updateApm, NULLVVFUNC, 0, NULLTIME },
   #endif
@@ -144,6 +155,8 @@ struct SensorModul SensorModulList[] = {
   { "Memory", initMemory, exitMemory, updateMemory, NULLVVFUNC, 0, NULLTIME },
   { "NetDev", initNetDev, exitNetDev, updateNetDev, checkNetDev, 0, NULLTIME },
   { "ProcessList", initProcessList, exitProcessList, updateProcessList, NULLVVFUNC, 0, NULLTIME },
+  { "Stat", initStat, exitStat, updateStat, NULLVVFUNC, 0, NULLTIME },
+  { "Uptime", initUptime, exitUptime, NULLIVFUNC, NULLVVFUNC, 0, NULLTIME },
 #endif /* OSTYPE_FreeBSD */
 
 #ifdef OSTYPE_Irix
