@@ -47,31 +47,6 @@
 namespace Oxygen
 {
 
-    //_________________________________________________________
-    QColor reduceContrast(const QColor &c0, const QColor &c1, double t)
-    {
-        double s = KColorUtils::contrastRatio(c0, c1);
-        if (s < t)
-            return c1;
-
-        double l = 0.0, h = 1.0;
-        double x = s, a;
-        QColor r = c1;
-        for (int maxiter = 16; maxiter; --maxiter)
-        {
-
-            a = 0.5 * (l + h);
-            r = KColorUtils::mix(c0, c1, a);
-            x = KColorUtils::contrastRatio(c0, r);
-
-            if (fabs(x - t) < 0.01) break;
-            if (x > t) h = a;
-            else l = a;
-        }
-
-        return r;
-    }
-
     //___________________________________________
     Client::Client(KDecorationBridge *b, Factory *f):
         KCommonDecorationUnstable(b, f),
@@ -111,20 +86,17 @@ namespace Oxygen
         widget()->setAcceptDrops( true );
 
         // setup glow animation
-        glowAnimation().data()->setStartValue( glowBias() );
-        glowAnimation().data()->setEndValue( 1.0 );
-        glowAnimation().data()->setTargetObject( this );
-        glowAnimation().data()->setPropertyName( "glowIntensity" );
-        glowAnimation().data()->setEasingCurve( QEasingCurve::InOutQuad );
-
-        connect( glowAnimation().data(), SIGNAL( valueChanged( const QVariant& ) ), widget(), SLOT( update( void ) ) );
-        connect( glowAnimation().data(), SIGNAL( finished( void ) ), widget(), SLOT( update( void ) ) );
-        connect( glowAnimation().data(), SIGNAL( finished() ), this, SLOT( clearForceActive() ) );
+        glowAnimation_->setStartValue( glowBias() );
+        glowAnimation_->setEndValue( 1.0 );
+        glowAnimation_->setTargetObject( this );
+        glowAnimation_->setPropertyName( "glowIntensity" );
+        glowAnimation_->setEasingCurve( QEasingCurve::InOutQuad );
+        connect( glowAnimation_, SIGNAL( finished( void ) ), this, SLOT( clearForceActive( void ) ) );
 
 
         // title animation data
-        titleAnimationData_.data()->initialize();
-        connect( titleAnimationData_.data(), SIGNAL( pixmapsChanged() ), widget(), SLOT( update( void ) ) );
+        titleAnimationData_->initialize();
+        connect( titleAnimationData_, SIGNAL( pixmapsChanged() ), SLOT( updateTitleRect() ) );
 
         // lists
         connect( itemData_.animation().data(), SIGNAL( finished() ), this, SLOT( clearTargetItem() ) );
@@ -166,13 +138,13 @@ namespace Oxygen
         configuration_ = factory_->configuration( *this );
 
         // animations duration
-        glowAnimation().data()->setDuration( configuration_.animationsDuration() );
-        titleAnimationData_.data()->setDuration( configuration_.animationsDuration() );
+        glowAnimation_->setDuration( configuration_.animationsDuration() );
+        titleAnimationData_->setDuration( configuration_.animationsDuration() );
         itemData_.animation().data()->setDuration( configuration_.animationsDuration() );
         itemData_.setAnimationsEnabled( useAnimations() );
 
         // reset title transitions
-        titleAnimationData_.data()->reset();
+        titleAnimationData_->reset();
 
         // should also update animations for buttons
         resetButtons();
@@ -260,7 +232,7 @@ namespace Oxygen
     {
 
         if( isMaximized() ) { return widget()->rect(); }
-        QRect frame( widget()->rect().adjusted(
+        const QRect frame( widget()->rect().adjusted(
             layoutMetric( LM_OuterPaddingLeft ), layoutMetric( LM_OuterPaddingTop ),
             -layoutMetric( LM_OuterPaddingRight ), -layoutMetric( LM_OuterPaddingBottom ) ) );
 
@@ -268,7 +240,7 @@ namespace Oxygen
         if( configuration().frameBorder() == Configuration::BorderNone && !isShade() )
         {
 
-            if( configuration().hideTitleBar() ) mask = QRegion();
+            if( hideTitleBar() ) mask = QRegion();
             else if( compositingActive() ) mask = (configuration().shadowMode() == Configuration::OxygenShadows ) ? QRegion():helper().decoRoundedMask( frame, 1, 1, 1, 0 );
             else mask = helper().roundedMask( frame, 1, 1, 1, 0 );
 
@@ -287,10 +259,10 @@ namespace Oxygen
     int Client::layoutMetric(LayoutMetric lm, bool respectWindowState, const KCommonDecorationButton *btn) const
     {
 
-        bool maximized( isMaximized() );
-        bool narrowSpacing( configuration().useNarrowButtonSpacing() );
-        int frameBorder( configuration().frameBorder() );
-        int buttonSize( configuration().hideTitleBar() ? 0 : configuration().buttonSize() );
+        const bool maximized( isMaximized() );
+        const bool narrowSpacing( configuration().useNarrowButtonSpacing() );
+        const int frameBorder( configuration().frameBorder() );
+        const int buttonSize( hideTitleBar() ? 0 : configuration().buttonSize() );
 
         switch (lm)
         {
@@ -304,7 +276,7 @@ namespace Oxygen
 
                     border = 0;
 
-                }  else if( lm == LM_BorderBottom && frameBorder >= Configuration::BorderNoSide ) {
+                } else if( lm == LM_BorderBottom && frameBorder >= Configuration::BorderNoSide ) {
 
                     // for tiny border, the convention is to have a larger bottom area in order to
                     // make resizing easier
@@ -330,7 +302,7 @@ namespace Oxygen
             case LM_TitleEdgeTop:
             {
                 int border = 0;
-                if( frameBorder == Configuration::BorderNone && configuration().hideTitleBar() )
+                if( frameBorder == Configuration::BorderNone && hideTitleBar() )
                 {
 
                     border = 0;
@@ -402,13 +374,12 @@ namespace Oxygen
     //_________________________________________________________
     QRect Client::defaultTitleRect( bool active ) const
     {
-        QRect titleRect( this->titleRect() );
-        titleRect.adjust( 0, -layoutMetric( LM_TitleEdgeTop ), 0, 0 );
 
+        QRect titleRect( this->titleRect().adjusted( 0, -layoutMetric( LM_TitleEdgeTop ), 0, 0 ) );
         if( active && configuration().drawTitleOutline() && isActive() )
         {
 
-            QRect textRect( titleBoundingRect( options()->font( true, false),  titleRect, caption() ) );
+            const QRect textRect( titleBoundingRect( options()->font( true, false),  titleRect, caption() ) );
             titleRect.setLeft( textRect.left() - layoutMetric( LM_TitleBorderLeft ) );
             titleRect.setRight( textRect.right() + layoutMetric( LM_TitleBorderRight ) );
 
@@ -428,7 +399,7 @@ namespace Oxygen
     {
 
         // get title bounding rect
-        QRect boundingRect = QFontMetrics( font ).boundingRect( rect, configuration().titleAlignment() | Qt::AlignVCenter, caption );
+        QRect boundingRect( QFontMetrics( font ).boundingRect( rect, configuration().titleAlignment() | Qt::AlignVCenter, caption ) );
 
         // adjust to make sure bounding rect
         // 1/ has same vertical alignment as original titleRect
@@ -461,10 +432,10 @@ namespace Oxygen
         itemData_.animate( AnimationNone );
 
         // maximum available space
-        QRect titleRect( this->titleRect() );
+        const QRect titleRect( this->titleRect() );
 
         // get tabs
-        int items( clientGroupItems().count() );
+        const int items( clientGroupItems().count() );
 
         // make sure item data have the correct number of items
         while( itemData_.count() < items ) itemData_.push_back( ClientGroupItemData() );
@@ -490,7 +461,7 @@ namespace Oxygen
         } else {
 
             int left( titleRect.left() );
-            int width( titleRect.width()/items );
+            const int width( titleRect.width()/items );
             for( int index = 0; index < itemData_.count(); index++ )
             {
 
@@ -538,7 +509,7 @@ namespace Oxygen
     }
 
     //_________________________________________________________
-    QColor Client::titlebarTextColor(const QPalette &palette)
+    QColor Client::titlebarTextColor(const QPalette &palette) const
     {
         if( glowIsAnimated() ) return KColorUtils::mix(
             titlebarTextColor( palette, false ),
@@ -547,27 +518,6 @@ namespace Oxygen
         else return titlebarTextColor( palette, isActive() );
     }
 
-    //_________________________________________________________
-    QColor Client::titlebarTextColor(const QPalette &palette, bool active)
-    {
-
-        if( active ){
-
-            return palette.color(QPalette::Active, QPalette::WindowText);
-
-        } else {
-
-            // todo: reimplement cache
-            QColor ab = palette.color(QPalette::Active, QPalette::Window);
-            QColor af = palette.color(QPalette::Active, QPalette::WindowText);
-            QColor nb = palette.color(QPalette::Inactive, QPalette::Window);
-            QColor nf = palette.color(QPalette::Inactive, QPalette::WindowText);
-            return reduceContrast(nb, nf, qMax(qreal(2.5), KColorUtils::contrastRatio(ab, KColorUtils::mix(ab, af, 0.4))));
-
-        }
-
-    }
-
     //_________________________________________________________
     void Client::renderWindowBackground( QPainter* painter, const QRect& rect, const QWidget* widget, const QPalette& palette ) const
     {
@@ -575,13 +525,15 @@ namespace Oxygen
         if( configuration().blendColor() == Configuration::NoBlending )
         {
 
-            painter->fillRect( rect, palette.color( widget->window()->backgroundRole() ) );
+            painter->fillRect( rect, palette.color( QPalette::Window ) );
 
         } else {
 
-            int offset = layoutMetric( LM_OuterPaddingTop );
+            const int offset = layoutMetric( LM_OuterPaddingTop );
+
             int height = 64 - Configuration::ButtonDefault;
-            if( !configuration().hideTitleBar() ) height += configuration().buttonSize();
+            if( !hideTitleBar() ) height += configuration().buttonSize();
+
             const QWidget* window( isPreview() ? this->widget() : widget->window() );
             helper().renderWindowBackground(painter, rect, widget, window, palette, offset, height );
 
@@ -593,9 +545,6 @@ namespace Oxygen
     void Client::renderWindowBorder( QPainter* painter, const QRect& clipRect, const QWidget* widget, const QPalette& palette ) const
     {
 
-        // check if outline is needed
-        if( clientGroupItems().count() < 2 && !itemData_.isAnimated() && !( isActive() && configuration().drawTitleOutline() ) ) return;
-
         // get coordinates relative to the client area
         // this is annoying. One could use mapTo if this was taking const QWidget* and not
         // const QWidget* as argument.
@@ -616,7 +565,8 @@ namespace Oxygen
         }
 
         QRect r = (isPreview()) ? this->widget()->rect():window->rect();
-        qreal shadowSize( shadowCache().shadowSize() );
+
+        const qreal shadowSize( shadowCache().shadowSize() );
         r.adjust( shadowSize, shadowSize, -shadowSize, -shadowSize );
         r.adjust(0,0, 1, 1);
 
@@ -624,14 +574,14 @@ namespace Oxygen
         QColor color( palette.window().color() );
 
         // title height
-        int titleHeight( layoutMetric( LM_TitleEdgeTop ) + layoutMetric( LM_TitleEdgeBottom ) + layoutMetric( LM_TitleHeight ) );
+        const int titleHeight( layoutMetric( LM_TitleEdgeTop ) + layoutMetric( LM_TitleEdgeBottom ) + layoutMetric( LM_TitleHeight ) );
 
         // make titlebar background darker for tabbed, non-outline window
         if( ( clientGroupItems().count() >= 2 || itemData_.isAnimated() ) && !(configuration().drawTitleOutline() && isActive() ) )
         {
 
-            QPoint topLeft( r.topLeft()-position );
-            QRect rect( topLeft, QSize( r.width(), titleHeight ) );
+            const QPoint topLeft( r.topLeft()-position );
+            const QRect rect( topLeft, QSize( r.width(), titleHeight ) );
 
             QLinearGradient lg( rect.topLeft(), rect.bottomLeft() );
             lg.setColorAt( 0, helper().alphaColor( Qt::black, 0.05 ) );
@@ -644,10 +594,10 @@ namespace Oxygen
 
         // horizontal line
         {
-            int shadowSize = 7;
-            int height = shadowSize-3;
+            const int shadowSize = 7;
+            const int height = shadowSize-3;
 
-            QPoint topLeft( r.topLeft()+QPoint(0,titleHeight-height)-position );
+            const QPoint topLeft( r.topLeft()+QPoint(0,titleHeight-height)-position );
             QRect rect( topLeft, QSize( r.width(), height ) );
 
             // adjustements to cope with shadow size and outline border.
@@ -668,32 +618,32 @@ namespace Oxygen
             QRect frame;
 
             // bottom line
-            int leftOffset = qMin( layoutMetric( LM_BorderLeft ), int(HFRAMESIZE) );
-            int rightOffset = qMin( layoutMetric( LM_BorderRight ), int(HFRAMESIZE) );
+            const int leftOffset = qMin( layoutMetric( LM_BorderLeft ), int(HFRAMESIZE) );
+            const int rightOffset = qMin( layoutMetric( LM_BorderRight ), int(HFRAMESIZE) );
             if( configuration().frameBorder() > Configuration::BorderNone )
             {
 
-                int height = qMax( 0, layoutMetric( LM_BorderBottom ) - HFRAMESIZE );
-                int width = r.width() - leftOffset - rightOffset - 1;
+                const int height = qMax( 0, layoutMetric( LM_BorderBottom ) - HFRAMESIZE );
+                const int width = r.width() - leftOffset - rightOffset - 1;
 
-                QRect rect( r.bottomLeft()-position + QPoint( leftOffset, -layoutMetric( LM_BorderBottom ) ), QSize( width, height ) );
+                const QRect rect( r.bottomLeft()-position + QPoint( leftOffset, -layoutMetric( LM_BorderBottom ) ), QSize( width, height ) );
                 if( height > 0 ) { mask += rect; frame |= rect; }
 
-                QColor shadow( helper().calcDarkColor( color ) );
+                const QColor shadow( helper().calcDarkColor( color ) );
                 painter->setPen( shadow );
                 painter->drawLine( rect.bottomLeft()+QPoint(0,1), rect.bottomRight()+QPoint(0,1) );
 
             }
 
             // left and right
-            int topOffset = titleHeight;
-            int bottomOffset = qMin( layoutMetric( LM_BorderBottom ), int(HFRAMESIZE) );
-            int height = r.height() - topOffset - bottomOffset - 1;
+            const int topOffset = titleHeight;
+            const int bottomOffset = qMin( layoutMetric( LM_BorderBottom ), int(HFRAMESIZE) );
+            const int height = r.height() - topOffset - bottomOffset - 1;
 
             if( configuration().frameBorder() >= Configuration::BorderTiny )
             {
 
-                QColor shadow( helper().calcLightColor( color ) );
+                const QColor shadow( helper().calcLightColor( color ) );
                 painter->setPen( shadow );
 
                 // left
@@ -714,7 +664,7 @@ namespace Oxygen
             // in preview mode also adds center square
             if( isPreview() )
             {
-                QRect rect( r.topLeft()-position + QPoint( layoutMetric( LM_BorderLeft ), topOffset ), QSize(r.width()-layoutMetric( LM_BorderLeft )-layoutMetric( LM_BorderRight ),height) );
+                const QRect rect( r.topLeft()-position + QPoint( layoutMetric( LM_BorderLeft ), topOffset ), QSize(r.width()-layoutMetric( LM_BorderLeft )-layoutMetric( LM_BorderRight ),height) );
                 mask += rect; frame |= rect;
             }
 
@@ -781,9 +731,7 @@ namespace Oxygen
         // center (for active windows only)
         {
             painter->save();
-            int offset = 1;
-            int voffset = 1;
-            QRect adjustedRect( rect.adjusted( offset, voffset, -offset, 1 ) );
+            QRect adjustedRect( rect.adjusted( 1, 1, -1, 1 ) );
 
             // prepare painter mask
             QRegion mask( adjustedRect.adjusted( 1, 0, -1, 0 ) );
@@ -796,10 +744,10 @@ namespace Oxygen
         }
 
         // shadow
-        int shadowSize = 7;
-        int offset = -3;
-        int voffset = 5-shadowSize;
-        QRect adjustedRect( rect.adjusted(offset, voffset, -offset, shadowSize) );
+        const int shadowSize( 7 );
+        const int offset( -3 );
+        const int voffset( 5-shadowSize );
+        const QRect adjustedRect( rect.adjusted(offset, voffset, -offset, shadowSize) );
         helper().slab( palette.color( widget()->backgroundRole() ), 0, shadowSize )->render( adjustedRect, painter, TileSet::Tiles(TileSet::Top|TileSet::Left|TileSet::Right) );
 
     }
@@ -808,35 +756,35 @@ namespace Oxygen
     void Client::renderTitleText( QPainter* painter, const QRect& rect, const QColor& color, const QColor& contrast ) const
     {
 
-        if( !titleAnimationData_.data()->isValid() )
+        if( !titleAnimationData_->isValid() )
         {
             // contrast pixmap
-            titleAnimationData_.data()->reset(
+            titleAnimationData_->reset(
                 rect,
                 renderTitleText( rect, caption(), color ),
                 renderTitleText( rect, caption(), contrast ) );
         }
 
-        if( titleAnimationData_.data()->isDirty() )
+        if( titleAnimationData_->isDirty() )
         {
 
             // clear dirty flags
-            titleAnimationData_.data()->setDirty( false );
+            titleAnimationData_->setDirty( false );
 
             // finish current animation if running
-            if( titleAnimationData_.data()->isAnimated() )
-            { titleAnimationData_.data()->finishAnimation(); }
+            if( titleAnimationData_->isAnimated() )
+            { titleAnimationData_->finishAnimation(); }
 
-            if( !titleAnimationData_.data()->isLocked() )
+            if( !titleAnimationData_->isLocked() )
             {
 
                 // set pixmaps
-                titleAnimationData_.data()->setPixmaps(
+                titleAnimationData_->setPixmaps(
                     rect,
                     renderTitleText( rect, caption(), color ),
                     renderTitleText( rect, caption(), contrast ) );
 
-                titleAnimationData_.data()->startAnimation();
+                titleAnimationData_->startAnimation();
                 renderTitleText( painter, rect, color, contrast );
 
             } else if( !caption().isEmpty() ) {
@@ -848,19 +796,19 @@ namespace Oxygen
             // lock animations (this must be done whether or not
             // animation was actually started, in order to extend locking
             // every time title get changed too rapidly
-            titleAnimationData_.data()->lockAnimations();
+            titleAnimationData_->lockAnimations();
 
-        } else if( titleAnimationData_.data()->isAnimated() ) {
+        } else if( titleAnimationData_->isAnimated() ) {
 
             if( isMaximized() ) painter->translate( 0, 2 );
-            if( !titleAnimationData_.data()->contrastPixmap().isNull() )
+            if( !titleAnimationData_->contrastPixmap().isNull() )
             {
                 painter->translate( 0, 1 );
-                painter->drawPixmap( rect.topLeft(), titleAnimationData_.data()->contrastPixmap() );
+                painter->drawPixmap( rect.topLeft(), titleAnimationData_->contrastPixmap() );
                 painter->translate( 0, -1 );
             }
 
-            painter->drawPixmap( rect.topLeft(), titleAnimationData_.data()->pixmap() );
+            painter->drawPixmap( rect.topLeft(), titleAnimationData_->pixmap() );
 
             if( isMaximized() ) painter->translate( 0, -2 );
 
@@ -876,8 +824,8 @@ namespace Oxygen
     void Client::renderTitleText( QPainter* painter, const QRect& rect, const QString& caption, const QColor& color, const QColor& contrast, bool elide ) const
     {
 
-        Qt::Alignment alignment( configuration().titleAlignment() | Qt::AlignVCenter );
-        QString local( elide ? QFontMetrics( painter->font() ).elidedText( caption, Qt::ElideRight, rect.width() ):caption );
+        const Qt::Alignment alignment( configuration().titleAlignment() | Qt::AlignVCenter );
+        const QString local( elide ? QFontMetrics( painter->font() ).elidedText( caption, Qt::ElideRight, rect.width() ):caption );
 
         // translate title down in case of maximized window
         if( isMaximized() ) painter->translate( 0, 2 );
@@ -908,8 +856,8 @@ namespace Oxygen
 
         QPainter painter( &out );
         painter.setFont( options()->font(isActive(), false) );
-        Qt::Alignment alignment( configuration().titleAlignment() | Qt::AlignVCenter );
-        QString local( elide ? QFontMetrics( painter.font() ).elidedText( caption, Qt::ElideRight, rect.width() ):caption );
+        const Qt::Alignment alignment( configuration().titleAlignment() | Qt::AlignVCenter );
+        const QString local( elide ? QFontMetrics( painter.font() ).elidedText( caption, Qt::ElideRight, rect.width() ):caption );
 
         painter.setPen( color );
         painter.drawText( out.rect(), alignment, local );
@@ -925,7 +873,7 @@ namespace Oxygen
         const ClientGroupItemData& item( itemData_[index] );
 
         // see if tag is active
-        int itemCount( itemData_.count() );
+        const int itemCount( itemData_.count() );
 
         //
         if( !item.boundingRect_.isValid() ) return;
@@ -942,11 +890,11 @@ namespace Oxygen
         { textRect.adjust( 0, 0, - configuration().buttonSize() - layoutMetric(LM_TitleEdgeRight), 0 ); }
 
         // check if current item is active
-        bool active( index == visibleClientGroupItem() );
+        const bool active( index == visibleClientGroupItem() );
 
         // get current item caption and update text rect
         const QList< ClientGroupItem >& items( clientGroupItems() );
-        QString caption( itemCount == 1 ? this->caption() : items[index].title() );
+        const QString caption( itemCount == 1 ? this->caption() : items[index].title() );
 
         // always make sure that titleRect never conflicts with window buttons
         QRect titleRect( this->titleRect() );
@@ -1022,12 +970,12 @@ namespace Oxygen
 
             // separators
             // draw Left separator
-            QColor color( backgroundPalette( widget(), palette ).window().color() );
-            bool isFirstItem(  index == 0 || (index == 1 && !itemData_[0].boundingRect_.isValid() ) );
+            const QColor color( backgroundPalette( widget(), palette ).window().color() );
+            const bool isFirstItem(  index == 0 || (index == 1 && !itemData_[0].boundingRect_.isValid() ) );
             if( !active && ( ( isFirstItem && buttonsLeftWidth() > 0 ) || itemData_.isTarget( index ) ) )
             {
 
-                QRect local( item.boundingRect_.topLeft()+QPoint(0,2), QSize( 2, item.boundingRect_.height()-3 ) );
+                const QRect local( item.boundingRect_.topLeft()+QPoint(0,2), QSize( 2, item.boundingRect_.height()-3 ) );
                 helper().drawSeparator( painter, local, color, Qt::Vertical);
 
             }
@@ -1039,7 +987,7 @@ namespace Oxygen
                 !( index+1 == visibleClientGroupItem() && itemData_[index+1].boundingRect_.isValid() ) ) ) )
             {
 
-                QRect local( item.boundingRect_.topRight()+QPoint(0,2), QSize( 2, item.boundingRect_.height()-3 ) );
+                const QRect local( item.boundingRect_.topRight()+QPoint(0,2), QSize( 2, item.boundingRect_.height()-3 ) );
                 helper().drawSeparator( painter, local, color, Qt::Vertical);
 
             }
@@ -1054,7 +1002,7 @@ namespace Oxygen
         if( itemData_.targetRect().isNull() || itemData_.isAnimationRunning() ) return;
 
         p->save();
-        QColor color = palette.color(QPalette::Highlight);
+        const QColor color = palette.color(QPalette::Highlight);
         p->setPen(KColorUtils::mix(color, palette.color(QPalette::Active, QPalette::WindowText)));
         p->setBrush( helper().alphaColor( color, 0.5 ) );
         p->drawRect( itemData_.targetRect().adjusted( 4, 2, -4, -2 ) );
@@ -1082,7 +1030,7 @@ namespace Oxygen
             } else {
 
                 // for small borders, use a frame that matches the titlebar only
-                QRect local( frame.topLeft(), QSize( frame.width(), layoutMetric(LM_TitleHeight) + layoutMetric(LM_TitleEdgeTop) ) );
+                const QRect local( frame.topLeft(), QSize( frame.width(), layoutMetric(LM_TitleHeight) + layoutMetric(LM_TitleEdgeTop) ) );
                 helper().drawFloatFrame(
                     painter, local, backgroundColor( widget(), palette ),
                     false, isActive() && configuration().useOxygenShadows(),
@@ -1119,14 +1067,12 @@ namespace Oxygen
             {
 
                 // Draw right side 3-dots resize handles
-                //qreal cenY = h / 2 + y + 0.5;
-                //qreal posX = w + x - 2.5;
-                qreal cenY = h / 2 + y ;
-                qreal posX = w + x - 3;
+                const int cenY = (h / 2 + y) ;
+                const int posX = (w + x - 3);
 
-                helper().renderDot( painter, QPointF(posX, cenY - 3), color);
-                helper().renderDot( painter, QPointF(posX, cenY), color);
-                helper().renderDot( painter, QPointF(posX, cenY + 3), color);
+                helper().renderDot( painter, QPoint(posX, cenY - 3), color);
+                helper().renderDot( painter, QPoint(posX, cenY), color);
+                helper().renderDot( painter, QPoint(posX, cenY + 3), color);
 
             }
 
@@ -1136,9 +1082,9 @@ namespace Oxygen
 
                 painter->save();
                 painter->translate(x + w-9, y + h-9);
-                helper().renderDot( painter, QPointF(2, 6), color);
-                helper().renderDot( painter, QPointF(5, 5), color);
-                helper().renderDot( painter, QPointF(6, 2), color);
+                helper().renderDot( painter, QPoint(2, 6), color);
+                helper().renderDot( painter, QPoint(5, 5), color);
+                helper().renderDot( painter, QPoint(6, 2), color);
                 painter->restore();
             }
 
@@ -1156,8 +1102,8 @@ namespace Oxygen
         // reset animation
         if( animateActiveChange() )
         {
-            glowAnimation().data()->setDirection( isActive() ? Animation::Forward : Animation::Backward );
-            if(!glowIsAnimated()) { glowAnimation().data()->start(); }
+            glowAnimation_->setDirection( isActive() ? Animation::Forward : Animation::Backward );
+            if(!glowIsAnimated()) { glowAnimation_->start(); }
         }
 
         // update size grip so that it gets the right color
@@ -1192,7 +1138,7 @@ namespace Oxygen
         KCommonDecorationUnstable::captionChange();
         itemData_.setDirty( true );
         if( animateTitleChange() )
-        { titleAnimationData_.data()->setDirty( true ); }
+        { titleAnimationData_->setDirty( true ); }
 
     }
 
@@ -1205,16 +1151,16 @@ namespace Oxygen
             if( glowIsAnimated() && !isForcedActive() )
             {
 
-                QColor inactiveColor( backgroundColor( widget, palette, false ) );
-                QColor activeColor( backgroundColor( widget, palette, true ) );
-                QColor mixed( KColorUtils::mix( inactiveColor, activeColor, glowIntensity() ) );
-                palette.setColor( widget->window()->backgroundRole(), mixed );
+                const QColor inactiveColor( backgroundColor( widget, palette, false ) );
+                const QColor activeColor( backgroundColor( widget, palette, true ) );
+                const QColor mixed( KColorUtils::mix( inactiveColor, activeColor, glowIntensity() ) );
+                palette.setColor( QPalette::Window, mixed );
                 palette.setColor( QPalette::Button, mixed );
 
             } else if( isActive() || isForcedActive() ) {
 
-                QColor color =  options()->color( KDecorationDefines::ColorTitleBar, true );
-                palette.setColor( widget->window()->backgroundRole(), color );
+                const QColor color =  options()->color( KDecorationDefines::ColorTitleBar, true );
+                palette.setColor( QPalette::Window, color );
                 palette.setColor( QPalette::Button, color );
 
             }
@@ -1226,12 +1172,12 @@ namespace Oxygen
     }
 
     //_________________________________________________________
-    QColor Client::backgroundColor( const QWidget* widget, QPalette palette, bool active ) const
+    QColor Client::backgroundColor( const QWidget*, QPalette palette, bool active ) const
     {
 
         return ( configuration().drawTitleOutline() && active ) ?
             options()->color( KDecorationDefines::ColorTitleBar, true ):
-            palette.color( widget->window()->backgroundRole() );
+            palette.color( QPalette::Window );
 
     }
 
@@ -1384,14 +1330,14 @@ namespace Oxygen
             } else {
 
                 // multipliers
-                int left = 1;
-                int right = 1;
-                int top = 1;
+                const int left = 1;
+                const int right = 1;
+                const int top = 1;
                 int bottom = 1;
 
                 // disable bottom corners when border frame is too small and window is not shaded
                 if( configuration().frameBorder() == Configuration::BorderNone && !isShade() ) bottom = 0;
-                QRegion mask( helper().roundedRegion( frame, left, right, top, bottom ) );
+                QRegion mask( helper().roundedMask( frame, left, right, top, bottom ) );
 
                 // in no-border configuration, an extra pixel is added to the mask
                 // in order to get the corners color right in case of title highlighting.
@@ -1419,7 +1365,9 @@ namespace Oxygen
 
         // window background
         renderWindowBackground( &painter, frame, widget(), backgroundPalette( widget(), palette ) );
-        renderWindowBorder( &painter, frame, widget(), palette );
+
+        // window border (for title outline)
+        if( hasTitleOutline() ) renderWindowBorder( &painter, frame, widget(), palette );
 
         // clipping
         if( compositingActive() )
@@ -1441,14 +1389,14 @@ namespace Oxygen
             painter.restore();
         }
 
-        if( !configuration().hideTitleBar() )
+        if( !hideTitleBar() )
         {
 
             // title bounding rect
             painter.setFont( options()->font(isActive(), false) );
 
             // draw ClientGroupItems
-            int itemCount( itemData_.count() );
+            const int itemCount( itemData_.count() );
             for( int i = 0; i < itemCount; i++ ) renderItem( &painter, i, palette );
 
             // draw target rect
@@ -1466,7 +1414,7 @@ namespace Oxygen
     bool Client::mousePressEvent( QMouseEvent* event )
     {
 
-        QPoint point = event->pos();
+        const QPoint point = event->pos();
         if( itemClicked( point ) < 0 ) return false;
         dragPoint_ = point;
 
@@ -1498,10 +1446,10 @@ namespace Oxygen
             buttonToWindowOperation( mouseButton_ ) != OperationsOp )
         {
 
-            QPoint point = event->pos();
+            const QPoint point = event->pos();
 
-            int visibleItem = visibleClientGroupItem();
-            int itemClicked( this->itemClicked( point ) );
+            const int visibleItem = visibleClientGroupItem();
+            const int itemClicked( this->itemClicked( point ) );
             if( itemClicked >= 0 && visibleItem != itemClicked )
             {
                 setVisibleClientGroupItem( itemClicked );
@@ -1521,18 +1469,18 @@ namespace Oxygen
     {
 
         // check button and distance to drag point
-        if( configuration().hideTitleBar() || mouseButton_ == Qt::NoButton  || ( event->pos() - dragPoint_ ).manhattanLength() <= QApplication::startDragDistance() )
+        if( hideTitleBar() || mouseButton_ == Qt::NoButton  || ( event->pos() - dragPoint_ ).manhattanLength() <= QApplication::startDragDistance() )
         { return false; }
 
         bool accepted( false );
         if( buttonToWindowOperation( mouseButton_ ) == ClientGroupDragOp )
         {
 
-            QPoint point = event->pos();
-            int itemClicked( this->itemClicked( point ) );
+            const QPoint point = event->pos();
+            const int itemClicked( this->itemClicked( point ) );
             if( itemClicked < 0 ) return false;
 
-            titleAnimationData_.data()->reset();
+            titleAnimationData_->reset();
 
             QDrag *drag = new QDrag( widget() );
             QMimeData *groupData = new QMimeData();
@@ -1541,8 +1489,7 @@ namespace Oxygen
             sourceItem_ = this->itemClicked( dragPoint_ );
 
             // get tab geometry
-            QRect geometry;
-            geometry = itemData_[itemClicked].boundingRect_;
+            QRect geometry( itemData_[itemClicked].boundingRect_ );
 
             // remove space used for buttons
             if( itemData_.count() > 1  )
@@ -1605,20 +1552,20 @@ namespace Oxygen
     {
 
         // check if drag enter is allowed
-        if( !event->mimeData()->hasFormat( clientGroupItemDragMimeType() ) || configuration().hideTitleBar() ) return false;
+        if( !event->mimeData()->hasFormat( clientGroupItemDragMimeType() ) || hideTitleBar() ) return false;
 
         //
         event->acceptProposedAction();
         if( event->source() != widget() )
         {
 
-            QPoint position( event->pos() );
+            const QPoint position( event->pos() );
             itemData_.animate( AnimationEnter, itemClicked( position, true ) );
 
         } else if( itemData_.count() > 1 )  {
 
-            QPoint position( event->pos() );
-            int itemClicked( this->itemClicked( position, false ) );
+            const QPoint position( event->pos() );
+            const int itemClicked( this->itemClicked( position, false ) );
             itemData_.animate( AnimationEnter|AnimationSameTarget, itemClicked );
 
         }
@@ -1639,7 +1586,6 @@ namespace Oxygen
 
         } else if( itemData_.isAnimated() ) {
 
-
             itemData_.animate( AnimationLeave );
 
         }
@@ -1658,15 +1604,15 @@ namespace Oxygen
         if( event->source() != widget() )
         {
 
-            QPoint position( event->pos() );
+            const QPoint position( event->pos() );
             itemData_.animate( AnimationMove, itemClicked( position, true ) );
 
         } else if( itemData_.count() > 1 )  {
 
             if( dragStartTimer_.isActive() ) dragStartTimer_.stop();
 
-            QPoint position( event->pos() );
-            int itemClicked( this->itemClicked( position, false ) );
+            const QPoint position( event->pos() );
+            const int itemClicked( this->itemClicked( position, false ) );
             itemData_.animate( AnimationMove|AnimationSameTarget, itemClicked );
 
         }
@@ -1679,7 +1625,7 @@ namespace Oxygen
     bool Client::dropEvent( QDropEvent* event )
     {
 
-        QPoint point = event->pos();
+        const QPoint point = event->pos();
         itemData_.animate( AnimationNone );
 
         const QMimeData *groupData = event->mimeData();
@@ -1688,7 +1634,7 @@ namespace Oxygen
         if( widget() == event->source() )
         {
 
-            int from = this->itemClicked( dragPoint_ );
+            const int from = this->itemClicked( dragPoint_ );
             int itemClicked( this->itemClicked( point, false ) );
 
             if( itemClicked > from )
@@ -1700,19 +1646,19 @@ namespace Oxygen
 
             itemData_.setDirty( true );
             moveItemInClientGroup( from, itemClicked );
-            widget()->update();
+            updateTitleRect();
 
         } else {
 
             setForceActive( true );
-            int itemClicked( this->itemClicked( point, true ) );
-            long source = QString( groupData->data( clientGroupItemDragMimeType() ) ).toLong();
+            const int itemClicked( this->itemClicked( point, true ) );
+            const long source = QString( groupData->data( clientGroupItemDragMimeType() ) ).toLong();
             itemData_.setDirty( true );
             moveItemToClientGroup( source, itemClicked );
 
         }
 
-        titleAnimationData_.data()->reset();
+        titleAnimationData_->reset();
         return true;
 
     }
@@ -1755,7 +1701,7 @@ namespace Oxygen
     //________________________________________________________________
     QPixmap Client::itemDragPixmap( int index, const QRect& geometry )
     {
-        bool itemValid( index >= 0 && index < clientGroupItems().count() );
+        const bool itemValid( index >= 0 && index < clientGroupItems().count() );
 
         QPixmap pixmap( geometry.size() );
         QPainter painter( &pixmap );
@@ -1767,7 +1713,7 @@ namespace Oxygen
         renderWindowBackground( &painter, geometry, widget(), widget()->palette() );
 
         // darken background if item is inactive
-        bool itemActive = !( itemValid && index != visibleClientGroupItem() );
+        const bool itemActive = !( itemValid && index != visibleClientGroupItem() );
         if( !itemActive )
         {
 
@@ -1787,7 +1733,7 @@ namespace Oxygen
         if( itemValid )
         { textRect.adjust( layoutMetric( LM_TitleBorderLeft ), 0, -layoutMetric(LM_TitleBorderRight), 0 ); }
 
-        QString caption( itemValid ? clientGroupItems()[index].title() : this->caption() );
+        const QString caption( itemValid ? clientGroupItems()[index].title() : this->caption() );
         renderTitleText(
             &painter, textRect, caption,
             titlebarTextColor( widget()->palette(), isActive() && itemActive ),
