@@ -99,6 +99,7 @@ EffectsHandlerImpl::EffectsHandlerImpl(CompositingType type)
     , fullscreen_effect( 0 )
     , next_window_quad_type( EFFECT_QUAD_TYPE_START )
     , mouse_poll_ref_count( 0 )
+    , current_paint_effectframe( 0 )
     {
     reconfigure();
     }
@@ -203,6 +204,20 @@ void EffectsHandlerImpl::paintWindow( EffectWindow* w, int mask, QRegion region,
         scene->finalPaintWindow( static_cast<EffectWindowImpl*>( w ), mask, region, data );
     }
 
+void EffectsHandlerImpl::paintEffectFrame( EffectFrame* frame, QRegion region, double opacity, double frameOpacity )
+    {
+    if( current_paint_effectframe < loaded_effects.size())
+        {
+        loaded_effects[current_paint_effectframe++].second->paintEffectFrame( frame, region, opacity, frameOpacity );
+        --current_paint_effectframe;
+        }
+    else
+        {
+        const EffectFrameImpl* frameImpl = static_cast<const EffectFrameImpl*>( frame );
+        frameImpl->finalRender( region, opacity, frameOpacity );
+        }
+    }
+
 void EffectsHandlerImpl::postPaintWindow( EffectWindow* w )
     {
     if( current_paint_window < loaded_effects.size())
@@ -213,10 +228,10 @@ void EffectsHandlerImpl::postPaintWindow( EffectWindow* w )
     // no special final code
     }
 
-bool EffectsHandlerImpl::providesResizeEffect()
+bool EffectsHandlerImpl::provides( Effect::Feature ef )
     {
     for( int i = 0; i < loaded_effects.size(); ++i )
-        if( loaded_effects.at(i).second->isResizeEffect() )
+        if( loaded_effects.at(i).second->provides( ef ) )
             return true;
     return false;
     }
@@ -251,6 +266,11 @@ bool EffectsHandlerImpl::decorationsHaveAlpha() const
     return Workspace::self()->decorationHasAlpha();
     }
 
+bool EffectsHandlerImpl::decorationSupportsBlurBehind() const
+    {
+    return Workspace::self()->decorationSupportsBlurBehind();
+    }
+
 // start another painting pass
 void EffectsHandlerImpl::startPaint()
     {
@@ -769,6 +789,15 @@ GLRenderTarget* EffectsHandlerImpl::popRenderTarget()
 #endif
 }
 
+bool EffectsHandlerImpl::isRenderTargetBound()
+    {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+    return !render_targets.isEmpty();
+#else
+    return false;
+#endif
+    }
+
 void EffectsHandlerImpl::addRepaintFull()
     {
     Workspace::self()->addRepaintFull();
@@ -1164,6 +1193,10 @@ void EffectsHandlerImpl::effectsChanged()
         }
     }
 
+EffectFrame* EffectsHandlerImpl::effectFrame( EffectFrameStyle style, bool staticSize, const QPoint& position, Qt::Alignment alignment ) const
+    {
+    return new EffectFrameImpl( style, staticSize, position, alignment );
+    }
 
 //****************************************
 // EffectWindowImpl
@@ -1177,6 +1210,12 @@ EffectWindowImpl::EffectWindowImpl() : EffectWindow()
 
 EffectWindowImpl::~EffectWindowImpl()
     {
+    QVariant cachedTextureVariant = data( LanczosCacheRole );
+    if( cachedTextureVariant.isValid() )
+        {
+        GLTexture *cachedTexture = static_cast< GLTexture*>(cachedTextureVariant.value<void*>());
+        delete cachedTexture;
+        }
     }
 
 bool EffectWindowImpl::isPaintingEnabled()
@@ -1632,4 +1671,266 @@ EffectWindowList EffectWindowGroupImpl::members() const
     return ret;
     }
 
+//****************************************
+// EffectFrameImpl
+//****************************************
+
+EffectFrameImpl::EffectFrameImpl( EffectFrameStyle style, bool staticSize, QPoint position, Qt::Alignment alignment )
+    : QObject ( 0 )
+    , EffectFrame()
+    , m_style( style )
+    , m_static( staticSize )
+    , m_point( position )
+    , m_alignment( alignment )
+    , m_shader( NULL )
+    {
+    if( m_style == EffectFrameStyled )
+        {
+        m_frame.setImagePath( "widgets/background" );
+        m_frame.setCacheAllRenderedFrames( true );
+        m_selection.setImagePath( "widgets/viewitem" );
+        m_selection.setElementPrefix( "hover" );
+        m_selection.setCacheAllRenderedFrames( true );
+        m_selection.setEnabledBorders( Plasma::FrameSvg::AllBorders );
+        connect( Plasma::Theme::defaultTheme(), SIGNAL( themeChanged() ), this, SLOT( plasmaThemeChanged() ));
+        }
+
+    if( effects->compositingType() == OpenGLCompositing )
+        {
+#ifdef KWIN_HAVE_OPENGL_COMPOSITING
+        m_sceneFrame = new SceneOpenGL::EffectFrame( this );
+#endif
+        }
+    else if( effects->compositingType() == XRenderCompositing )
+        {
+        m_sceneFrame = new SceneXrender::EffectFrame( this );
+        }
+    else
+        {
+        // that should not happen and will definitely crash!
+        m_sceneFrame = NULL;
+        }
+    }
+
+EffectFrameImpl::~EffectFrameImpl()
+    {
+    delete m_sceneFrame;
+    }
+
+const QFont& EffectFrameImpl::font() const
+    {
+    return m_font;
+    }
+
+void EffectFrameImpl::setFont( const QFont& font )
+    {
+    if (m_font == font)
+        {
+        return;
+        }
+    m_font = font;
+    QRect oldGeom = m_geometry;
+    if( !m_text.isEmpty() )
+        {
+        autoResize();
+        }
+    if( oldGeom == m_geometry )
+        { // Wasn't updated in autoResize()
+        m_sceneFrame->freeTextFrame();
+        }
+    }
+
+void EffectFrameImpl::free()
+    {
+    m_sceneFrame->free();
+    }
+
+const QRect& EffectFrameImpl::geometry() const
+    {
+    return m_geometry;
+    }
+
+void EffectFrameImpl::setGeometry( const QRect& geometry, bool force )
+    {
+    QRect oldGeom = m_geometry;
+    m_geometry = geometry;
+    if( m_geometry == oldGeom && !force )
+        {
+        return;
+        }
+    effects->addRepaint( oldGeom );
+    effects->addRepaint( m_geometry );
+    if( m_geometry.size() == oldGeom.size() && !force )
+        {
+        return;
+        }
+
+    if( m_style == EffectFrameStyled )
+        {
+        qreal left, top, right, bottom;
+        m_frame.getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
+        m_frame.resizeFrame( m_geometry.adjusted( -left, -top, right, bottom ).size() );
+        }
+
+    free();
+    }
+
+const QPixmap& EffectFrameImpl::icon() const
+    {
+    return m_icon;
+    }
+
+void EffectFrameImpl::setIcon( const QPixmap& icon )
+    {
+    m_icon = icon;
+    if( isCrossFade() )
+        {
+        m_sceneFrame->crossFadeIcon();
+        }
+    if( m_iconSize.isEmpty() ) // Set a size if we don't already have one
+        {
+        setIconSize( m_icon.size() );
+        }
+    m_sceneFrame->freeIconFrame();
+    }
+
+const QSize& EffectFrameImpl::iconSize() const
+    {
+    return m_iconSize;
+    }
+
+void EffectFrameImpl::setIconSize( const QSize& size )
+    {
+    if( m_iconSize == size )
+        {
+        return;
+        }
+    m_iconSize = size;
+    autoResize();
+    m_sceneFrame->freeIconFrame();
+    }
+
+void EffectFrameImpl::plasmaThemeChanged()
+    {
+    free();
+    }
+
+void EffectFrameImpl::render( QRegion region, double opacity, double frameOpacity )
+    {
+    if( m_geometry.isEmpty() )
+        {
+        return; // Nothing to display
+        }
+    m_shader = NULL;
+    effects->paintEffectFrame( this, region, opacity, frameOpacity );
+    }
+
+void EffectFrameImpl::finalRender( QRegion region, double opacity, double frameOpacity ) const
+    {
+    region = infiniteRegion(); // TODO: Old region doesn't seem to work with OpenGL
+
+    m_sceneFrame->render( region, opacity, frameOpacity );
+    }
+
+Qt::Alignment EffectFrameImpl::alignment() const
+    {
+    return m_alignment;
+    }
+
+
+void
+EffectFrameImpl::align( QRect &geometry )
+{
+    if( m_alignment & Qt::AlignLeft )
+        geometry.moveLeft( m_point.x() );
+    else if( m_alignment & Qt::AlignRight )
+        geometry.moveLeft( m_point.x() - geometry.width() );
+    else
+        geometry.moveLeft( m_point.x() - geometry.width() / 2 );
+    if( m_alignment & Qt::AlignTop )
+        geometry.moveTop( m_point.y() );
+    else if( m_alignment & Qt::AlignBottom )
+        geometry.moveTop( m_point.y() - geometry.height() );
+    else
+        geometry.moveTop( m_point.y() - geometry.height() / 2 );
+}
+
+
+void EffectFrameImpl::setAlignment( Qt::Alignment alignment )
+    {
+    m_alignment = alignment;
+    align( m_geometry );
+    setGeometry( m_geometry );
+    }
+
+void EffectFrameImpl::setPosition( const QPoint& point )
+    {
+    m_point = point;
+    QRect geometry = m_geometry; // this is important, setGeometry need call repaint for old & new geometry
+    align( geometry );
+    setGeometry( geometry );
+    }
+
+const QString& EffectFrameImpl::text() const
+    {
+    return m_text;
+    }
+
+void EffectFrameImpl::setText( const QString& text )
+    {
+    if( m_text == text )
+        {
+        return;
+        }
+    if( isCrossFade() )
+        {
+        m_sceneFrame->crossFadeText();
+        }
+    m_text = text;
+    QRect oldGeom = m_geometry;
+    autoResize();
+    if( oldGeom == m_geometry )
+        { // Wasn't updated in autoResize()
+        m_sceneFrame->freeTextFrame();
+        }
+    }
+
+void EffectFrameImpl::setSelection( const QRect& selection )
+    {
+    if( selection == m_selectionGeometry )
+        {
+        return;
+        }
+    m_selectionGeometry = selection;
+    if( m_selectionGeometry.size() != m_selection.frameSize().toSize() )
+        {
+        m_selection.resizeFrame( m_selectionGeometry.size() );
+        }
+    // TODO; optimize to only recreate when resizing
+    m_sceneFrame->freeSelection();
+    }
+
+void EffectFrameImpl::autoResize()
+    {
+    if( m_static )
+        return; // Not automatically resizing
+
+    QRect geometry;
+    // Set size
+    if( !m_text.isEmpty() )
+        {
+        QFontMetrics metrics( m_font );
+        geometry.setSize( metrics.size( 0, m_text ));
+        }
+    if( !m_icon.isNull() && !m_iconSize.isEmpty() )
+        {
+        geometry.setLeft( -m_iconSize.width() );
+        if( m_iconSize.height() > geometry.height() )
+            geometry.setHeight( m_iconSize.height() );
+        }
+
+    align( geometry );
+    setGeometry( geometry );
+    }
+
 } // namespace
