@@ -194,7 +194,7 @@ void DesktopGridEffect::paintScreen( int mask, QRegion region, ScreenPaintData&
         d.yScale *= (float)geo.height()/(float)windowMove->height();
         d.xTranslate += qRound( geo.left() - windowMove->x() );
         d.yTranslate += qRound( geo.top() - windowMove->y() );
-        effects->drawWindow( windowMove, PAINT_WINDOW_TRANSFORMED, infiniteRegion(), d );
+        effects->drawWindow( windowMove, PAINT_WINDOW_TRANSFORMED | PAINT_WINDOW_LANCZOS, infiniteRegion(), d );
         }
 
     if( desktopNameAlignment )
@@ -326,6 +326,8 @@ void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region,
                         screenQuads.append( quad );
                     transformedGeo = manager.transformedGeometry( w );
                     quadsAdded = true;
+                    if( !manager.areWindowsMoving() )
+                        mask |= PAINT_WINDOW_LANCZOS;
                     }
                 else if( w->screen() != screen )
                     quadsAdded = true; // we don't want parts of overlapping windows on the other screen
@@ -381,6 +383,12 @@ void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region,
             else
                 {
                 PaintClipper pc( effects->clientArea( ScreenArea, screen, 0 ));
+                if( w->isDesktop() && timeline.value() == 1.0 )
+                    {
+                    // desktop windows are not in a motion manager and can always be rendered with
+                    // lanczos sampling except for animations
+                    mask |= PAINT_WINDOW_LANCZOS;
+                    }
                 effects->paintWindow( w, mask, region, d );
                 }
             // Assume desktop windows can never be on two screens at once (Plasma makes one window per screen)
@@ -464,6 +472,20 @@ void DesktopGridEffect::windowClosed( EffectWindow* w )
     effects->addRepaintFull();
     }
 
+void DesktopGridEffect::windowDeleted( EffectWindow* w )
+    {
+    for( QHash< DesktopButtonsView*, EffectWindow*>::iterator it = m_desktopButtonsViews.begin();
+         it != m_desktopButtonsViews.end(); ++it )
+        {
+        if( it.value() && it.value() == w )
+            {
+            it.key()->deleteLater();
+            m_desktopButtonsViews.erase( it );
+            break;
+            }
+        }
+    }
+
 void DesktopGridEffect::windowGeometryShapeChanged( EffectWindow* w, const QRect& old )
     {
     Q_UNUSED( old )
@@ -1192,7 +1214,7 @@ void DesktopGridEffect::setup()
         font.setPointSize( 12 );
         for( int i = 0; i < effects->numberOfDesktops(); i++ )
             {
-            EffectFrame* frame = new EffectFrame( EffectFrame::Unstyled, false );
+            EffectFrame* frame = effects->effectFrame( EffectFrameUnstyled, false );
             frame->setFont( font );
             frame->setText( effects->desktopName( i+1 ));
             frame->setAlignment( desktopNameAlignment );
@@ -1325,14 +1347,15 @@ void DesktopGridEffect::finish()
             }
         m_proxy = 0;
         }
-    while( !m_desktopButtonsViews.isEmpty() )
+    
+    QHash< DesktopButtonsView*, EffectWindow* >::iterator i = m_desktopButtonsViews.begin();
+    while (i != m_desktopButtonsViews.end())
         {
-        DesktopButtonsView* view = m_desktopButtonsViews.begin().key();
-        m_desktopButtonsViews[ view ]->unrefWindow();
-        m_desktopButtonsViews[ view ] = 0;
-        m_desktopButtonsViews.remove( view );
-        delete view;
-        view = 0;
+        if( *i && (*i)->isDeleted() )
+            (*i)->unrefWindow();
+        DesktopButtonsView *view = i.key();
+        i = m_desktopButtonsViews.erase(i);
+        view->deleteLater();
         }
     }
 
@@ -1437,7 +1460,7 @@ void DesktopGridEffect::desktopsAdded( int old )
         font.setPointSize( 12 );
         for( int i = old; i < desktop; i++ )
             {
-            EffectFrame* frame = new EffectFrame( EffectFrame::Unstyled, false );
+            EffectFrame* frame = effects->effectFrame( EffectFrameUnstyled, false );
             frame->setFont( font );
             frame->setText( effects->desktopName( i+1 ));
             frame->setAlignment( desktopNameAlignment );
@@ -1562,6 +1585,7 @@ DesktopButtonsView::DesktopButtonsView( QWidget* parent )
     qreal height = form->size().height() + top + bottom;
     m_frame->resizeFrame( QSizeF( width, height ) );
     Plasma::WindowEffects::enableBlurBehind( winId(), true, m_frame->mask() );
+    Plasma::WindowEffects::overrideShadow( winId(), true );
     form->setPos( left, top );
     scene->setSceneRect( QRectF( QPointF( 0, 0 ), QSizeF( width, height ) ) );
     setScene( scene );
