@@ -33,10 +33,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #endif
 
 #include <QMouseEvent>
+#include <QtGui/QPainter>
+#include <QtGui/QGraphicsLinearLayout>
+#include <Plasma/PushButton>
+#include <Plasma/WindowEffects>
 
 #include <math.h>
 #include <assert.h>
 #include <limits.h>
+#include <QTimer>
 
 namespace KWin
 {
@@ -53,7 +58,8 @@ PresentWindowsEffect::PresentWindowsEffect()
     , m_mode( ModeCurrentDesktop )
     , m_managerWindow( NULL )
     , m_highlightedWindow( NULL )
-    , m_filterFrame( EffectFrame::Styled, false )
+    , m_filterFrame( effects->effectFrame( EffectFrameStyled, false ) )
+    , m_closeView( NULL )
     {
     m_atomDesktop = XInternAtom( display(), "_KDE_PRESENT_WINDOWS_DESKTOP", False );
     m_atomWindows = XInternAtom( display(), "_KDE_PRESENT_WINDOWS_GROUP", False );
@@ -68,7 +74,7 @@ PresentWindowsEffect::PresentWindowsEffect()
     QFont font;
     font.setPointSize( font.pointSize() * 2 );
     font.setBold( true );
-    m_filterFrame.setFont( font );
+    m_filterFrame->setFont( font );
 
     KActionCollection* actionCollection = new KActionCollection( this );
     KAction* a = ( KAction* )actionCollection->addAction( "Expose" );
@@ -106,6 +112,8 @@ PresentWindowsEffect::~PresentWindowsEffect()
         {
         effects->unreserveElectricBorder( border );
         }
+    delete m_filterFrame;
+    delete m_closeView;
     }
 
 void PresentWindowsEffect::reconfigure( ReconfigureFlags )
@@ -140,6 +148,7 @@ void PresentWindowsEffect::reconfigure( ReconfigureFlags )
     m_layoutMode = conf.readEntry( "LayoutMode", int( LayoutNatural ));
     m_showCaptions = conf.readEntry( "DrawWindowCaptions", true );
     m_showIcons = conf.readEntry( "DrawWindowIcons", true );
+    m_doNotCloseWindows = !conf.readEntry( "AllowClosingWindows", true );
     m_tabBoxAllowed = conf.readEntry( "TabBox", false );
     m_tabBoxAlternativeAllowed = conf.readEntry( "TabBoxAlternative", false );
     m_ignoreMinimized = conf.readEntry( "IgnoreMinimized", false );
@@ -197,7 +206,7 @@ void PresentWindowsEffect::paintScreen( int mask, QRegion region, ScreenPaintDat
 
     // Display the filter box
     if( !m_windowFilter.isEmpty() )
-        m_filterFrame.render( region );
+        m_filterFrame->render( region );
     }
 
 void PresentWindowsEffect::postPaintScreen()
@@ -276,7 +285,7 @@ void PresentWindowsEffect::prePaintWindow( EffectWindow *w, WindowPrePaintData &
             data.setTranslucent();
 
         // Calculate window's brightness
-        if( w == m_highlightedWindow || !m_activated )
+        if( w == m_highlightedWindow || w == m_closeWindow || !m_activated )
             m_windowData[w].highlight = qMin( 1.0, m_windowData[w].highlight + time / m_fadeDuration );
         else
             m_windowData[w].highlight = qMax( 0.0, m_windowData[w].highlight - time / m_fadeDuration );
@@ -380,12 +389,12 @@ void PresentWindowsEffect::windowAdded( EffectWindow *w )
     m_windowData[w].visible = isVisibleWindow( w );
     m_windowData[w].opacity = 0.0;
     m_windowData[w].highlight = 0.0;
-    m_windowData[w].textFrame = new EffectFrame( EffectFrame::Unstyled, false );
+    m_windowData[w].textFrame = effects->effectFrame( EffectFrameUnstyled, false );
     QFont font;
     font.setBold( true );
     font.setPointSize( 12 );
     m_windowData[w].textFrame->setFont( font );
-    m_windowData[w].iconFrame = new EffectFrame( EffectFrame::Unstyled, false );
+    m_windowData[w].iconFrame = effects->effectFrame( EffectFrameUnstyled, false );
     m_windowData[w].iconFrame->setAlignment( Qt::AlignRight | Qt::AlignBottom );
     m_windowData[w].iconFrame->setIcon( w->icon() );
     if( isSelectableWindow( w ))
@@ -393,6 +402,13 @@ void PresentWindowsEffect::windowAdded( EffectWindow *w )
         m_motionManager.manage( w );
         rearrangeWindows();
         }
+    if( w == effects->findWindow( m_closeView->winId() ) )
+        {
+        m_windowData[w].visible = true;
+        m_windowData[w].highlight = 1.0;
+        m_closeWindow = w;
+        w->setData( WindowForceBlurRole, QVariant( true ) );
+        }
     }
 
 void PresentWindowsEffect::windowClosed( EffectWindow *w )
@@ -406,6 +422,11 @@ void PresentWindowsEffect::windowClosed( EffectWindow *w )
     w->refWindow();
     if( m_highlightedWindow == w )
         setHighlightedWindow( findFirstWindow() );
+    if( m_closeWindow == w )
+        {
+        m_closeWindow = 0;
+        return; // don't rearrange
+        }
     rearrangeWindows();
     }
 
@@ -447,6 +468,22 @@ void PresentWindowsEffect::windowInputMouseEvent( Window w, QEvent *e )
     assert( w == m_input );
     Q_UNUSED( w );
 
+    QMouseEvent* me = static_cast< QMouseEvent* >( e );
+    if( m_closeView->geometry().contains( me->pos() ) )
+        {
+        if( !m_closeView->isVisible() )
+            {
+            updateCloseWindow();
+            }
+        if( m_closeView->isVisible() )
+            {
+            const QPoint widgetPos = m_closeView->mapFromGlobal( me->pos() );
+            const QPointF scenePos = m_closeView->mapToScene( widgetPos );
+            QMouseEvent event( me->type(), widgetPos, me->pos(), me->button(), me->buttons(), me->modifiers() );
+            m_closeView->windowInputMouseEvent( &event );
+            return;
+            }
+        }
     // Which window are we hovering over? Always trigger as we don't always get move events before clicking
     // We cannot use m_motionManager.windowAtPoint() as the window might not be visible
     EffectWindowList windows = m_motionManager.managedWindows();
@@ -463,11 +500,14 @@ void PresentWindowsEffect::windowInputMouseEvent( Window w, QEvent *e )
             break;
             }
         }
+    if( m_highlightedWindow && m_motionManager.transformedGeometry( m_highlightedWindow ).contains( me->pos() ) )
+        updateCloseWindow();
+    else
+        m_closeView->hide();
 
     if( e->type() != QEvent::MouseButtonPress )
         return;
 
-    QMouseEvent* me = static_cast<QMouseEvent*>( e );
     if( me->button() == Qt::LeftButton )
         {
         if( hovering )
@@ -841,6 +881,7 @@ void PresentWindowsEffect::rearrangeWindows()
         return;
 
     effects->addRepaintFull(); // Trigger the first repaint
+    m_closeView->hide();
 
     // Work out which windows are on which screens
     EffectWindowList windowlist;
@@ -1541,6 +1582,9 @@ void PresentWindowsEffect::setActive( bool active, bool closingTab )
         m_highlightedWindow = NULL;
         m_windowFilter.clear();
 
+        m_closeView = new CloseWindowView();
+        connect( m_closeView, SIGNAL(close()), SLOT(closeWindow()) );
+
         // Add every single window to m_windowData (Just calling [w] creates it)
         foreach( EffectWindow *w, effects->stackingOrder() )
             {
@@ -1553,12 +1597,12 @@ void PresentWindowsEffect::setActive( bool active, bool closingTab )
             if( w->isOnCurrentDesktop() && !w->isMinimized() )
                 m_windowData[w].opacity = 1.0;
             m_windowData[w].highlight = 1.0;
-            m_windowData[w].textFrame = new EffectFrame( EffectFrame::Unstyled, false );
+            m_windowData[w].textFrame = effects->effectFrame( EffectFrameUnstyled, false );
             QFont font;
             font.setBold( true );
             font.setPointSize( 12 );
             m_windowData[w].textFrame->setFont( font );
-            m_windowData[w].iconFrame = new EffectFrame( EffectFrame::Unstyled, false );
+            m_windowData[w].iconFrame = effects->effectFrame( EffectFrameUnstyled, false );
             m_windowData[w].iconFrame->setAlignment( Qt::AlignRight | Qt::AlignBottom );
             m_windowData[w].iconFrame->setIcon( w->icon() );
             }
@@ -1642,11 +1686,13 @@ void PresentWindowsEffect::setActive( bool active, bool closingTab )
             m_windowData[w].visible = ( w->isOnDesktop( desktop ) || w->isOnAllDesktops() ) &&
                 !w->isMinimized() && ( w->visibleInClientGroup() || m_windowData[w].visible );
             }
+        delete m_closeView;
+        m_closeView = 0;
 
         // Move all windows back to their original position
         foreach( EffectWindow *w, m_motionManager.managedWindows() )
             m_motionManager.moveWindow( w, w->geometry() );
-        m_filterFrame.free();
+        m_filterFrame->free();
         m_windowFilter.clear();
         m_selectedWindows.clear();
 
@@ -1674,8 +1720,8 @@ void PresentWindowsEffect::setActive( bool active, bool closingTab )
 void PresentWindowsEffect::updateFilterFrame()
     {
     QRect area = effects->clientArea( ScreenArea, effects->activeScreen(), effects->currentDesktop() );
-    m_filterFrame.setPosition( QPoint( area.x() + area.width() / 2, area.y() + area.height() / 2 ));
-    m_filterFrame.setText( i18n( "Filter:\n%1", m_windowFilter ));
+    m_filterFrame->setPosition( QPoint( area.x() + area.width() / 2, area.y() + area.height() / 2 ));
+    m_filterFrame->setText( i18n( "Filter:\n%1", m_windowFilter ));
     }
 
 //-----------------------------------------------------------------------------
@@ -1693,10 +1739,17 @@ bool PresentWindowsEffect::isSelectableWindow( EffectWindow *w )
         return false;
     if( w->isSkipSwitcher() )
         return false;
+    if( w == effects->findWindow( m_closeView->winId() ) )
+        return false;
+    if( m_tabBoxEnabled )
+        return true;
+    if ( m_ignoreMinimized && w->isMinimized() )
+        return false;
     switch( m_mode )
         {
+        default:
         case ModeAllDesktops:
-            break;
+            return true;
         case ModeCurrentDesktop:
             return w->isOnCurrentDesktop();
         case ModeSelectedDesktop:
@@ -1706,9 +1759,6 @@ bool PresentWindowsEffect::isSelectableWindow( EffectWindow *w )
         case ModeWindowClass:
             return m_class == w->windowClass();
         }
-    if( !m_tabBoxEnabled && m_ignoreMinimized && w->isMinimized() )
-        return false;
-    return true;
     }
 
 bool PresentWindowsEffect::isVisibleWindow( EffectWindow *w )
@@ -1723,6 +1773,7 @@ void PresentWindowsEffect::setHighlightedWindow( EffectWindow *w )
     if( w == m_highlightedWindow || ( w != NULL && !m_motionManager.isManaging( w )))
         return;
 
+    m_closeView->hide();
     if( m_highlightedWindow )
         m_highlightedWindow->addRepaintFull(); // Trigger the first repaint
     m_highlightedWindow = w;
@@ -1731,6 +1782,33 @@ void PresentWindowsEffect::setHighlightedWindow( EffectWindow *w )
 
     if( m_tabBoxEnabled && m_highlightedWindow )
         effects->setTabBoxWindow( w );
+    updateCloseWindow();
+    }
+
+void PresentWindowsEffect::updateCloseWindow()
+    {
+    if ( m_doNotCloseWindows )
+        return;
+    if( m_closeView->isVisible() )
+        return;
+    if( !m_highlightedWindow )
+        {
+        m_closeView->hide();
+        return;
+        }
+    const QRectF rect = m_motionManager.targetGeometry( m_highlightedWindow );
+    m_closeView->setGeometry( rect.x() + rect.width() - m_closeView->sceneRect().width(), rect.y(),
+                                m_closeView->sceneRect().width(), m_closeView->sceneRect().height() );
+    if( rect.contains( effects->cursorPos() ) )
+        m_closeView->delayedShow();
+    else
+        m_closeView->hide();
+    }
+
+void PresentWindowsEffect::closeWindow()
+    {
+    if( m_highlightedWindow )
+        m_highlightedWindow->closeWindow();
     }
 
 EffectWindow* PresentWindowsEffect::relativeWindow( EffectWindow *w, int xdiff, int ydiff, bool wrap ) const
@@ -1932,6 +2010,101 @@ void PresentWindowsEffect::globalShortcutChangedClass( const QKeySequence& seq )
     shortcutClass = KShortcut( seq );
     }
 
+/************************************************
+* CloseWindowView
+************************************************/
+CloseWindowView::CloseWindowView( QWidget* parent )
+    : QGraphicsView(parent)
+    , m_delayedShowTimer( new QTimer( this ))
+    {
+    setWindowFlags( Qt::X11BypassWindowManagerHint );
+    setAttribute( Qt::WA_TranslucentBackground );
+    setFrameShape( QFrame::NoFrame );
+    QPalette pal = palette();
+    pal.setColor( backgroundRole(), Qt::transparent );
+    setPalette( pal );
+    setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+    setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+
+    // setup the scene
+    QGraphicsScene* scene = new QGraphicsScene( this );
+    m_closeButton = new Plasma::PushButton();
+    m_closeButton->setIcon( KIcon( "window-close" ) );
+    scene->addItem( m_closeButton );
+    connect( m_closeButton, SIGNAL(clicked()), SIGNAL(close()));
+
+    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout;
+    layout->addItem( m_closeButton );
+
+    QGraphicsWidget *form = new QGraphicsWidget;
+    form->setLayout( layout );
+    form->setGeometry(0, 0, 32, 32);
+    scene->addItem( form );
+
+    m_frame = new Plasma::FrameSvg( this );
+    m_frame->setImagePath( "dialogs/background" );
+    m_frame->setCacheAllRenderedFrames( true );
+    m_frame->setEnabledBorders( Plasma::FrameSvg::AllBorders );
+    qreal left, top, right, bottom;
+    m_frame->getMargins( left, top, right, bottom );
+    qreal width = form->size().width() + left + right;
+    qreal height = form->size().height() + top + bottom;
+    m_frame->resizeFrame( QSizeF( width, height ) );
+    Plasma::WindowEffects::enableBlurBehind( winId(), true, m_frame->mask() );
+    Plasma::WindowEffects::overrideShadow( winId(), true );
+    form->setPos( left, top );
+    scene->setSceneRect( QRectF( QPointF( 0, 0 ), QSizeF( width, height ) ) );
+    setScene( scene );
+
+    // setup the timer
+    m_delayedShowTimer->setSingleShot( true );
+    m_delayedShowTimer->setInterval( 500 );
+    connect( m_delayedShowTimer, SIGNAL(timeout()), SLOT(show()));
+    }
+
+void CloseWindowView::windowInputMouseEvent( QMouseEvent* e )
+    {
+    if( m_delayedShowTimer->isActive() )
+        return;
+    if( e->type() == QEvent::MouseMove )
+        {
+        mouseMoveEvent( e );
+        }
+    else if( e->type() == QEvent::MouseButtonPress )
+        {
+        mousePressEvent( e );
+        }
+    else if( e->type() == QEvent::MouseButtonDblClick )
+        {
+        mouseDoubleClickEvent( e );
+        }
+    else if( e->type() == QEvent::MouseButtonRelease )
+        {
+        mouseReleaseEvent( e );
+        }
+    }
+
+void CloseWindowView::drawBackground( QPainter* painter, const QRectF& rect )
+    {
+    Q_UNUSED( rect )
+    painter->setRenderHint( QPainter::Antialiasing );
+    m_frame->paintFrame( painter );
+    }
+
+void CloseWindowView::hide()
+    {
+    m_delayedShowTimer->stop();
+    QWidget::hide();
+    }
+
+void CloseWindowView::delayedShow()
+    {
+    if( isVisible() || m_delayedShowTimer->isActive() )
+        return;
+    m_delayedShowTimer->start();
+    }
+
+
 } // namespace
 
 #include "presentwindows.moc"
