@@ -56,7 +56,7 @@ ConfirmDialog::ConfirmDialog() :
                      i18n("Confirm Desktop Effects Change"), KTimerDialog::Ok|KTimerDialog::Cancel,
                      KTimerDialog::Cancel)
     {
-    setObjectName( "mainKTimerDialog" );
+    setObjectName( QLatin1String( "mainKTimerDialog" ) );
     setButtonGuiItem( KDialog::Ok, KGuiItem( i18n( "&Accept Configuration" ), "dialog-ok" ));
     setButtonGuiItem( KDialog::Cancel, KGuiItem( i18n( "&Return to Previous Configuration" ), "dialog-cancel" ));
 
@@ -84,6 +84,9 @@ KWinCompositingConfig::KWinCompositingConfig(QWidget *parent, const QVariantList
     ui.tabWidget->setCurrentIndex(0);
     ui.statusTitleWidget->hide();
 
+    // For future use
+    (void) I18N_NOOP( "Use GLSL shaders" );
+
 #define OPENGL_INDEX 0
 #define XRENDER_INDEX 1
 #ifndef KWIN_HAVE_OPENGL_COMPOSITING
@@ -117,13 +120,16 @@ KWinCompositingConfig::KWinCompositingConfig(QWidget *parent, const QVariantList
     connect(ui.animationSpeedCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(changed()));
 
     connect(ui.compositingType, SIGNAL(currentIndexChanged(int)), this, SLOT(changed()));
+    connect(ui.compositingType, SIGNAL(currentIndexChanged(int)), this, SLOT(toogleSmoothScaleUi(int)));
     connect(ui.windowThumbnails, SIGNAL(activated(int)), this, SLOT(changed()));
     connect(ui.disableChecks, SIGNAL(toggled(bool)), this, SLOT(changed()));
+    connect(ui.unredirectFullscreen , SIGNAL(toggled(bool)), this, SLOT(changed()));
+    connect(ui.glScaleFilter, SIGNAL(currentIndexChanged(int)), this, SLOT(changed()));
+    connect(ui.xrScaleFilter, SIGNAL(currentIndexChanged(int)), this, SLOT(changed()));
+    
     connect(ui.glMode, SIGNAL(currentIndexChanged(int)), this, SLOT(changed()));
-    connect(ui.glTextureFilter, SIGNAL(currentIndexChanged(int)), this, SLOT(changed()));
     connect(ui.glDirect, SIGNAL(toggled(bool)), this, SLOT(changed()));
     connect(ui.glVSync, SIGNAL(toggled(bool)), this, SLOT(changed()));
-    connect(ui.xrenderSmoothScale, SIGNAL(toggled(bool)), this, SLOT(changed()));
     connect(ui.compositingStateButton, SIGNAL(clicked(bool)), kwinInterface, SLOT(toggleCompositing()));
     connect(kwinInterface, SIGNAL(compositingToggled(bool)), this, SLOT(setupCompositingState(bool)));
 
@@ -414,6 +420,13 @@ void KWinCompositingConfig::setupCompositingState( bool active, bool enabled )
     ui.compositingStateButton->setEnabled( enabled );
     }
 
+void KWinCompositingConfig::toogleSmoothScaleUi( int compositingType )
+{
+    ui.glScaleFilter->setVisible( compositingType == OPENGL_INDEX );
+    ui.xrScaleFilter->setVisible( compositingType == XRENDER_INDEX );
+    ui.scaleMethodLabel->setBuddy( compositingType == XRENDER_INDEX ? ui.xrScaleFilter : ui.glScaleFilter );
+}
+
 bool KWinCompositingConfig::effectEnabled( const QString& effect, const KConfigGroup& cfg ) const
     {
     KService::List services = KServiceTypeTrader::self()->query(
@@ -443,14 +456,17 @@ void KWinCompositingConfig::loadAdvancedTab()
     else // shown, or default
         ui.windowThumbnails->setCurrentIndex( 1 );
     ui.disableChecks->setChecked( config.readEntry( "DisableChecks", false ));
+    ui.unredirectFullscreen->setChecked( config.readEntry( "UnredirectFullscreen", true ));
 
+    ui.xrScaleFilter->setCurrentIndex((int)config.readEntry("XRenderSmoothScale", false));
+    ui.glScaleFilter->setCurrentIndex(config.readEntry("GLTextureFilter", 2));
+    
     QString glMode = config.readEntry("GLMode", "TFP");
     ui.glMode->setCurrentIndex((glMode == "TFP") ? 0 : ((glMode == "SHM") ? 1 : 2));
-    ui.glTextureFilter->setCurrentIndex(config.readEntry("GLTextureFilter", 1));
     ui.glDirect->setChecked(config.readEntry("GLDirect", mDefaultPrefs.enableDirectRendering()));
     ui.glVSync->setChecked(config.readEntry("GLVSync", mDefaultPrefs.enableVSync()));
 
-    ui.xrenderSmoothScale->setChecked(config.readEntry("XRenderSmoothScale", false));
+    toogleSmoothScaleUi( ui.compositingType->currentIndex() );
     }
 
 void KWinCompositingConfig::load()
@@ -601,19 +617,22 @@ bool KWinCompositingConfig::saveAdvancedTab()
         advancedChanged = true;
         }
     else if( config.readEntry("HiddenPreviews", 5) != hps[ ui.windowThumbnails->currentIndex() ]
-        || config.readEntry("XRenderSmoothScale", false ) != ui.xrenderSmoothScale->isChecked() )
+        || (int)config.readEntry("XRenderSmoothScale", false ) != ui.xrScaleFilter->currentIndex() 
+        || config.readEntry("GLTextureFilter", 2 ) != ui.glScaleFilter->currentIndex())
         advancedChanged = true;
 
     config.writeEntry("Backend", (ui.compositingType->currentIndex() == OPENGL_INDEX) ? "OpenGL" : "XRender");
     config.writeEntry("HiddenPreviews", hps[ ui.windowThumbnails->currentIndex() ] );
     config.writeEntry("DisableChecks", ui.disableChecks->isChecked());
+    config.writeEntry( "UnredirectFullscreen", ui.unredirectFullscreen->isChecked() );
+
+    config.writeEntry("XRenderSmoothScale", ui.xrScaleFilter->currentIndex() == 1);
+    config.writeEntry("GLTextureFilter", ui.glScaleFilter->currentIndex());
 
     config.writeEntry("GLMode", glModes[ui.glMode->currentIndex()]);
-    config.writeEntry("GLTextureFilter", ui.glTextureFilter->currentIndex());
     config.writeEntry("GLDirect", ui.glDirect->isChecked());
     config.writeEntry("GLVSync", ui.glVSync->isChecked());
 
-    config.writeEntry("XRenderSmoothScale", ui.xrenderSmoothScale->isChecked());
 
     return advancedChanged;
     }
@@ -752,11 +771,12 @@ void KWinCompositingConfig::defaults()
     ui.compositingType->setCurrentIndex( 0 );
     ui.windowThumbnails->setCurrentIndex( 1 );
     ui.disableChecks->setChecked( false );
+    ui.unredirectFullscreen->setChecked( true );
+    ui.xrScaleFilter->setCurrentIndex( 0 );
+    ui.glScaleFilter->setCurrentIndex( 2 );
     ui.glMode->setCurrentIndex( 0 );
-    ui.glTextureFilter->setCurrentIndex( 1 );
     ui.glDirect->setChecked( mDefaultPrefs.enableDirectRendering() );
     ui.glVSync->setChecked( mDefaultPrefs.enableVSync() );
-    ui.xrenderSmoothScale->setChecked( false );
     }
 
 QString KWinCompositingConfig::quickHelp() const
