@@ -66,6 +66,7 @@ Sources and other compositing managers:
 */
 
 #include "scene_opengl.h"
+#include "kwinglplatform.h"
 
 #include <kxerrorhandler.h>
 
@@ -91,6 +92,8 @@ Sources and other compositing managers:
 namespace KWin
 {
 
+extern int currentRefreshRate();
+
 //****************************************
 // SceneOpenGL
 //****************************************
@@ -141,6 +144,12 @@ SceneOpenGL::SceneOpenGL( Workspace* ws )
         return; // error
     // Initialize OpenGL
     initGL();
+    if( QString((const char*)glGetString( GL_RENDERER )) == "Software Rasterizer" )
+        {
+        kError( 1212 ) << "OpenGL Software Rasterizer detected. Falling back to XRender.";
+        QTimer::singleShot( 0, Workspace::self(), SLOT( fallbackToXRenderCompositing()));
+        return;
+        }
     if( !hasGLExtension( "GL_ARB_texture_non_power_of_two" )
         && !hasGLExtension( "GL_ARB_texture_rectangle" ))
         {
@@ -158,7 +167,11 @@ SceneOpenGL::SceneOpenGL( Workspace* ws )
             {
             if( glXWaitVideoSync( 1, 0, &sync ) == 0 )
                 has_waitSync = true;
+            else
+                qWarning() << "NO VSYNC! glXWaitVideoSync(1,0,&uint) isn't 0 but" << glXWaitVideoSync( 1, 0, &sync );
             }
+        else
+            qWarning() << "NO VSYNC! glXGetVideoSync(&uint) isn't 0 but" << glXGetVideoSync( &sync );
         }
 
     // OpenGL scene setup
@@ -236,7 +249,7 @@ SceneOpenGL::~SceneOpenGL()
             glXDestroyPixmap( display(), last_pixmap );
         glXDestroyContext( display(), ctxdrawable );
         }
-    EffectFrame::cleanup();
+    SceneOpenGL::EffectFrame::cleanup();
     checkGLError( "Cleanup" );
     }
 
@@ -743,6 +756,7 @@ bool SceneOpenGL::selfCheckFinish()
 // the entry function for painting
 void SceneOpenGL::paint( QRegion damage, ToplevelList toplevels )
     {
+    QTime t = QTime::currentTime();
     foreach( Toplevel* c, toplevels )
         {
         assert( windows.contains( c ));
@@ -771,6 +785,7 @@ void SceneOpenGL::paint( QRegion damage, ToplevelList toplevels )
         damage |= selfCheckRegion();
         }
 #endif
+    lastRenderTime = t.elapsed();
     flushBuffer( mask, damage );
 #if 0
     if( !selfCheckDone )
@@ -790,8 +805,7 @@ void SceneOpenGL::waitSync()
     { // NOTE that vsync has no effect with indirect rendering
     if( waitSyncAvailable())
         {
-        unsigned int sync;
-
+        uint sync;
         glFlush();
         glXGetVideoSync( &sync );
         glXWaitVideoSync( 2, ( sync + 1 ) % 2, &sync );
@@ -1123,17 +1137,22 @@ bool SceneOpenGL::Texture::load( const Pixmap& pix, const QSize& size,
                 GLX_MIPMAP_TEXTURE_EXT, fbcdrawableinfo[ depth ].mipmap,
                 None, None, None
                 };
-            if ( ( fbcdrawableinfo[ depth ].texture_targets & GLX_TEXTURE_2D_BIT_EXT ) &&
-                 ( GLTexture::NPOTTextureSupported() ||
-                   ( isPowerOfTwo(size.width()) && isPowerOfTwo(size.height()) )))
+            // Specifying the texture target explicitly is reported to cause a performance
+            // regression with R300G (see bug #256654).
+            if ( GLPlatform::instance()->driver() != Driver_R300G )
                 {
-                attrs[ 4 ] = GLX_TEXTURE_TARGET_EXT;
-                attrs[ 5 ] = GLX_TEXTURE_2D_EXT;
-                }
-            else if ( fbcdrawableinfo[ depth ].texture_targets & GLX_TEXTURE_RECTANGLE_BIT_EXT )
-                {
-                attrs[ 4 ] = GLX_TEXTURE_TARGET_EXT;
-                attrs[ 5 ] = GLX_TEXTURE_RECTANGLE_EXT;
+                if ( ( fbcdrawableinfo[ depth ].texture_targets & GLX_TEXTURE_2D_BIT_EXT ) &&
+                     ( GLTexture::NPOTTextureSupported() ||
+                       ( isPowerOfTwo(size.width()) && isPowerOfTwo(size.height()) )))
+                    {
+                    attrs[ 4 ] = GLX_TEXTURE_TARGET_EXT;
+                    attrs[ 5 ] = GLX_TEXTURE_2D_EXT;
+                    }
+                else if ( fbcdrawableinfo[ depth ].texture_targets & GLX_TEXTURE_RECTANGLE_BIT_EXT )
+                    {
+                    attrs[ 4 ] = GLX_TEXTURE_TARGET_EXT;
+                    attrs[ 5 ] = GLX_TEXTURE_RECTANGLE_EXT;
+                    }
                 }
             glxpixmap = glXCreatePixmap( display(), fbcdrawableinfo[ depth ].fbconfig, pix, attrs );
 #ifdef CHECK_GL_ERROR
@@ -1420,18 +1439,7 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
     else
         filter = ImageFilterFast;
     if( filter == ImageFilterGood )
-        {
-        // avoid unneeded mipmap generation by only using trilinear
-        // filtering when it actually makes a difference, that is with
-        // minification or changed vertices
-        if( options->glSmoothScale == 2
-            && ( data.quads.smoothNeeded() || data.xScale < 1 || data.yScale < 1 ))
-            {
-            texture.setFilter( GL_LINEAR_MIPMAP_LINEAR );
-            }
-        else
-            texture.setFilter( GL_LINEAR );
-        }
+        texture.setFilter( GL_LINEAR );
     else
         texture.setFilter( GL_NEAREST );
     // do required transformations
@@ -1597,18 +1605,7 @@ void SceneOpenGL::Window::paintDecoration( const QPixmap* decoration, TextureTyp
     else
         return;
     if( filter == ImageFilterGood )
-        {
-        // avoid unneeded mipmap generation by only using trilinear
-        // filtering when it actually makes a difference, that is with
-        // minification or changed vertices
-        if( options->glSmoothScale == 2
-            && ( data.quads.smoothNeeded() || data.xScale < 1 || data.yScale < 1 ))
-            {
-            decorationTexture->setFilter( GL_LINEAR_MIPMAP_LINEAR );
-            }
-        else
-            decorationTexture->setFilter( GL_LINEAR );
-        }
+        decorationTexture->setFilter( GL_LINEAR );
     else
         decorationTexture->setFilter( GL_NEAREST );
     decorationTexture->setWrapMode( GL_CLAMP_TO_EDGE );
@@ -1696,13 +1693,18 @@ void SceneOpenGL::Window::prepareShaderRenderStates( TextureType type, double op
     // setup blending of transparent windows
     glPushAttrib( GL_ENABLE_BIT );
     bool opaque = isOpaque() && opacity == 1.0;
+    bool alpha = toplevel->hasAlpha() || type != Content;
     if( type != Content )
         opaque = false;
-    if( !opaque )
-        {
-        glEnable( GL_BLEND );
-        glBlendFunc( GL_ONE, GL_ONE_MINUS_SRC_ALPHA );
+    if (!opaque) {
+        glEnable(GL_BLEND);
+        if (alpha) {
+            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        } else {
+            glBlendColor((float)opacity, (float)opacity, (float)opacity, (float)opacity);
+            glBlendFunc(GL_ONE, GL_ONE_MINUS_CONSTANT_ALPHA);
         }
+    }
     shader->setUniform("opacity", (float)opacity);
     shader->setUniform("saturation", (float)saturation);
     shader->setUniform("brightness", (float)brightness);
@@ -1942,6 +1944,424 @@ void SceneOpenGL::Window::restoreRenderStates( TextureType type, double opacity,
     glPopAttrib();  // ENABLE_BIT
     }
 
+//****************************************
+// SceneOpenGL::EffectFrame
+//****************************************
+
+GLTexture* SceneOpenGL::EffectFrame::m_unstyledTexture = NULL;
+QPixmap* SceneOpenGL::EffectFrame::m_unstyledPixmap = NULL;
+
+SceneOpenGL::EffectFrame::EffectFrame( EffectFrameImpl* frame )
+    : Scene::EffectFrame( frame )
+    , m_texture( NULL )
+    , m_textTexture( NULL )
+    , m_textPixmap( NULL )
+    , m_oldTextTexture( NULL )
+    , m_iconTexture( NULL )
+    , m_oldIconTexture( NULL )
+    , m_selectionTexture( NULL )
+    , m_unstyledVBO( NULL )
+    {
+    if( m_effectFrame->style() == EffectFrameUnstyled && !m_unstyledTexture )
+        {
+        updateUnstyledTexture();
+        }
+    }
+
+SceneOpenGL::EffectFrame::~EffectFrame()
+    {
+    delete m_texture;
+    delete m_textTexture;
+    delete m_textPixmap;
+    delete m_oldTextTexture;
+    delete m_iconTexture;
+    delete m_oldIconTexture;
+    delete m_selectionTexture;
+    delete m_unstyledVBO;
+    }
+
+void SceneOpenGL::EffectFrame::free()
+    {
+    delete m_texture;
+    m_texture = NULL;
+    delete m_textTexture;
+    m_textTexture = NULL;
+    delete m_textPixmap;
+    m_textPixmap = NULL;
+    delete m_iconTexture;
+    m_iconTexture = NULL;
+    delete m_selectionTexture;
+    m_selectionTexture = NULL;
+    delete m_unstyledVBO;
+    m_unstyledVBO = NULL;
+    delete m_oldIconTexture;
+    m_oldIconTexture = NULL;
+    delete m_oldTextTexture;
+    m_oldTextTexture = NULL;
+    }
+
+void SceneOpenGL::EffectFrame::freeIconFrame()
+    {
+    delete m_iconTexture;
+    m_iconTexture = NULL;
+    }
+
+void SceneOpenGL::EffectFrame::freeTextFrame()
+    {
+    delete m_textTexture;
+    m_textTexture = NULL;
+    delete m_textPixmap;
+    m_textPixmap = NULL;
+    }
+
+void SceneOpenGL::EffectFrame::freeSelection()
+    {
+    delete m_selectionTexture;
+    m_selectionTexture = NULL;
+    }
+
+void SceneOpenGL::EffectFrame::crossFadeIcon()
+    {
+    delete m_oldIconTexture;
+    m_oldIconTexture = m_iconTexture;
+    m_iconTexture = NULL;
+    }
+
+void SceneOpenGL::EffectFrame::crossFadeText()
+    {
+    delete m_oldTextTexture;
+    m_oldTextTexture = m_textTexture;
+    m_textTexture = NULL;
+    }
+
+void SceneOpenGL::EffectFrame::render( QRegion region, double opacity, double frameOpacity )
+    {
+    if( m_effectFrame->geometry().isEmpty() )
+        return; // Nothing to display
+
+    region = infiniteRegion(); // TODO: Old region doesn't seem to work with OpenGL
+
+    GLShader* shader = m_effectFrame->shader();
+    if( shader )
+        {
+        shader->bind();
+        shader->setUniform("saturation", 1.0f);
+        shader->setUniform("brightness", 1.0f);
+
+        shader->setUniform("textureWidth", 1.0f);
+        shader->setUniform("textureHeight", 1.0f);
+        }
+
+    glPushAttrib( GL_CURRENT_BIT | GL_ENABLE_BIT | GL_TEXTURE_BIT );
+    glEnable( GL_BLEND );
+    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+    if( !shader )
+        glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
+
+    glPushMatrix();
+
+    // Render the actual frame
+    if( m_effectFrame->style() == EffectFrameUnstyled )
+        {
+        if( !m_unstyledVBO )
+            {
+            m_unstyledVBO = new GLVertexBuffer( GLVertexBuffer::Static );
+            QRect area = m_effectFrame->geometry();
+            area.moveTo(0,0);
+            area.adjust( -5, -5, 5, 5 );
+            
+            const int roundness = 5;
+            QVector<float> verts, texCoords;
+            verts.reserve( 84 );
+            texCoords.reserve( 84 );
+
+            // top left
+            verts << area.left() << area.top();
+            texCoords << 0.0f << 0.0f;
+            verts << area.left() << area.top() + roundness;
+            texCoords << 0.0f << 0.5f;
+            verts << area.left() + roundness << area.top();
+            texCoords << 0.5f << 0.0f;
+            verts << area.left() + roundness << area.top() + roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.left() << area.top() + roundness;
+            texCoords << 0.0f << 0.5f;
+            verts << area.left() + roundness << area.top();
+            texCoords << 0.5f << 0.0f;
+            // top
+            verts << area.left() + roundness << area.top();
+            texCoords << 0.5f << 0.0f;
+            verts << area.left() + roundness << area.top() + roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.right() - roundness << area.top();
+            texCoords << 0.5f << 0.0f;
+            verts << area.left() + roundness << area.top() + roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.right() - roundness << area.top() + roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.right() - roundness << area.top();
+            texCoords << 0.5f << 0.0f;
+            // top right
+            verts << area.right() - roundness << area.top();
+            texCoords << 0.5f << 0.0f;
+            verts << area.right() - roundness << area.top() + roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.right() << area.top();
+            texCoords << 1.0f << 0.0f;
+            verts << area.right() - roundness << area.top() + roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.right() << area.top() + roundness;
+            texCoords << 1.0f << 0.5f;
+            verts << area.right() << area.top();
+            texCoords << 1.0f << 0.0f;
+            // bottom left
+            verts << area.left() << area.bottom() - roundness;
+            texCoords << 0.0f << 0.5f;
+            verts << area.left() << area.bottom();
+            texCoords << 0.0f << 1.0f;
+            verts << area.left() + roundness << area.bottom() - roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.left() + roundness << area.bottom();
+            texCoords << 0.5f << 1.0f;
+            verts << area.left() << area.bottom();
+            texCoords << 0.0f << 1.0f;
+            verts << area.left() + roundness << area.bottom() - roundness;
+            texCoords << 0.5f << 0.5f;
+            // bottom
+            verts << area.left() + roundness << area.bottom() - roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.left() + roundness << area.bottom();
+            texCoords << 0.5f << 1.0f;
+            verts << area.right() - roundness << area.bottom() - roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.left() + roundness << area.bottom();
+            texCoords << 0.5f << 1.0f;
+            verts << area.right() - roundness << area.bottom();
+            texCoords << 0.5f << 1.0f;
+            verts << area.right() - roundness << area.bottom() - roundness;
+            texCoords << 0.5f << 0.5f;
+            // bottom right
+            verts << area.right() - roundness << area.bottom() - roundness;
+            texCoords << 0.5f << 0.5f;
+            verts << area.right() - roundness << area.bottom();
+            texCoords << 0.5f << 1.0f;
+            verts << area.right() << area.bottom() - roundness;
+            texCoords << 1.0f << 0.5f;
+            verts << area.right() - roundness << area.bottom();
+            texCoords << 0.5f << 1.0f;
+            verts << area.right() << area.bottom();
+            texCoords << 1.0f << 1.0f;
+            verts << area.right() << area.bottom() - roundness;
+            texCoords << 1.0f << 0.5f;
+            // center
+            verts << area.left() << area.top() + roundness;
+            texCoords << 0.0f << 0.5f;
+            verts << area.left() << area.bottom() - roundness;
+            texCoords << 0.0f << 0.5f;
+            verts << area.right() << area.top() + roundness;
+            texCoords << 1.0f << 0.5f;
+            verts << area.left() << area.bottom() - roundness;
+            texCoords << 0.0f << 0.5f;
+            verts << area.right() << area.bottom() - roundness;
+            texCoords << 1.0f << 0.5f;
+            verts << area.right() << area.top() + roundness;
+            texCoords << 1.0f << 0.5f;
+
+            m_unstyledVBO->setData( verts.count() / 2, 2, verts.data(), texCoords.data() );
+            }
+
+        if( shader )
+            shader->setUniform( "opacity", (float)(opacity * frameOpacity) );
+        else
+            glColor4f( 0.0, 0.0, 0.0, opacity * frameOpacity );
+
+        m_unstyledTexture->bind();
+        const QPoint pt = m_effectFrame->geometry().topLeft();
+        glTranslatef( pt.x(), pt.y(), 0.0f );
+        m_unstyledVBO->render( region, GL_TRIANGLES );
+        glTranslatef( -pt.x(), -pt.y(), 0.0f );
+        m_unstyledTexture->unbind();
+        }
+    else if( m_effectFrame->style() == EffectFrameStyled )
+        {
+        if( !m_texture ) // Lazy creation
+            updateTexture();
+
+        if( shader )
+            shader->setUniform( "opacity", (float)(opacity * frameOpacity) );
+        else
+            glColor4f( 1.0, 1.0, 1.0, opacity * frameOpacity );
+
+        m_texture->bind();
+        qreal left, top, right, bottom;
+        m_effectFrame->frame().getMargins( left, top, right, bottom ); // m_geometry is the inner geometry
+        m_texture->render( region, m_effectFrame->geometry().adjusted( -left, -top, right, bottom ));
+        m_texture->unbind();
+
+        if( !m_effectFrame->selection().isNull() )
+            {
+            if( !m_selectionTexture ) // Lazy creation
+                {
+                QPixmap pixmap = m_effectFrame->selectionFrame().framePixmap();
+                m_selectionTexture = new Texture( pixmap.handle(), pixmap.size(), pixmap.depth() );
+                }
+            glBlendFunc( GL_ONE, GL_ONE_MINUS_SRC_ALPHA );
+            m_selectionTexture->bind();
+            m_selectionTexture->render( region, m_effectFrame->selection() );
+            m_selectionTexture->unbind();
+            glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+            }
+        }
+
+    // Render icon
+    if( !m_effectFrame->icon().isNull() && !m_effectFrame->iconSize().isEmpty() )
+        {
+        QPoint topLeft( m_effectFrame->geometry().x(),
+                        m_effectFrame->geometry().center().y() - m_effectFrame->iconSize().height() / 2 );
+
+        if( m_effectFrame->isCrossFade() && m_oldIconTexture )
+            {
+            if( shader )
+                shader->setUniform( "opacity", (float)opacity * (1.0f - (float)m_effectFrame->crossFadeProgress()) );
+            else
+                glColor4f( 1.0, 1.0, 1.0, opacity * (1.0 - m_effectFrame->crossFadeProgress()) );
+
+            m_oldIconTexture->bind();
+            m_oldIconTexture->render( region, QRect( topLeft, m_effectFrame->iconSize() ));
+            m_oldIconTexture->unbind();
+            if( shader )
+                shader->setUniform( "opacity", (float)opacity * (float)m_effectFrame->crossFadeProgress() );
+            else
+                glColor4f( 1.0, 1.0, 1.0, opacity * m_effectFrame->crossFadeProgress() );
+            }
+        else
+            {
+            if( shader )
+                shader->setUniform( "opacity", (float)opacity );
+            else
+                glColor4f( 1.0, 1.0, 1.0, opacity );
+            }
+
+        if( !m_iconTexture ) // lazy creation
+            {
+            m_iconTexture = new Texture( m_effectFrame->icon().handle(),
+                                         m_effectFrame->icon().size(),
+                                         m_effectFrame->icon().depth() );
+            }
+        m_iconTexture->bind();
+        m_iconTexture->render( region, QRect( topLeft, m_effectFrame->iconSize() ));
+        m_iconTexture->unbind();
+        }
+
+    // Render text
+    if( !m_effectFrame->text().isEmpty() )
+        {
+        if( m_effectFrame->isCrossFade() && m_oldTextTexture )
+            {
+            if( shader )
+                shader->setUniform( "opacity", (float)opacity * (1.0f - (float)m_effectFrame->crossFadeProgress()) );
+            else
+                glColor4f( 1.0, 1.0, 1.0, opacity * (1.0 - m_effectFrame->crossFadeProgress()) );
+
+            m_oldTextTexture->bind();
+            m_oldTextTexture->render( region, m_effectFrame->geometry() );
+            m_oldTextTexture->unbind();
+            if( shader )
+                shader->setUniform( "opacity", (float)opacity * (float)m_effectFrame->crossFadeProgress() );
+            else
+                glColor4f( 1.0, 1.0, 1.0, opacity * m_effectFrame->crossFadeProgress() );
+            }
+        else
+            {
+            if( shader )
+                shader->setUniform( "opacity", (float)opacity );
+            else
+                glColor4f( 1.0, 1.0, 1.0, opacity );
+            }
+        if( !m_textTexture ) // Lazy creation
+            updateTextTexture();
+        m_textTexture->bind();
+        m_textTexture->render( region, m_effectFrame->geometry() );
+        m_textTexture->unbind();
+        }
+
+    if( shader )
+        shader->unbind();
+
+    glPopMatrix();
+    glPopAttrib();
+    }
+
+void SceneOpenGL::EffectFrame::updateTexture()
+    {
+    delete m_texture;
+    if( m_effectFrame->style() == EffectFrameStyled )
+        {
+        QPixmap pixmap = m_effectFrame->frame().framePixmap();
+        m_texture = new Texture( pixmap.handle(), pixmap.size(), pixmap.depth() );
+        }
+    }
+
+void SceneOpenGL::EffectFrame::updateTextTexture()
+    {
+    delete m_textTexture;
+    delete m_textPixmap;
+
+    if( m_effectFrame->text().isEmpty() )
+        return;
+
+    // Determine position on texture to paint text
+    QRect rect( QPoint( 0, 0 ), m_effectFrame->geometry().size() );
+    if( !m_effectFrame->icon().isNull() && !m_effectFrame->iconSize().isEmpty() )
+        rect.setLeft( m_effectFrame->iconSize().width() );
+
+    // If static size elide text as required
+    QString text = m_effectFrame->text();
+    if( m_effectFrame->isStatic() )
+        {
+        QFontMetrics metrics( m_effectFrame->font() );
+        text = metrics.elidedText( text, Qt::ElideRight, rect.width() );
+        }
+
+    m_textPixmap = new QPixmap( m_effectFrame->geometry().size() );
+    m_textPixmap->fill( Qt::transparent );
+    QPainter p( m_textPixmap );
+    p.setFont( m_effectFrame->font() );
+    if( m_effectFrame->style() == EffectFrameStyled )
+        p.setPen( m_effectFrame->styledTextColor() );
+    else // TODO: What about no frame? Custom color setting required
+        p.setPen( Qt::white );
+    p.drawText( rect, m_effectFrame->alignment(), text );
+    p.end();
+    m_textTexture = new Texture( m_textPixmap->handle(), m_textPixmap->size(), m_textPixmap->depth() );
+    }
+
+void SceneOpenGL::EffectFrame::updateUnstyledTexture()
+    {
+    delete m_unstyledTexture;
+    delete m_unstyledPixmap;
+    // Based off circle() from kwinxrenderutils.cpp
+#define CS 8
+    m_unstyledPixmap = new QPixmap( 2 * CS, 2 * CS );
+    m_unstyledPixmap->fill( Qt::transparent );
+    QPainter p( m_unstyledPixmap );
+    p.setRenderHint( QPainter::Antialiasing );
+    p.setPen( Qt::NoPen );
+    p.setBrush( Qt::black );
+    p.drawEllipse( m_unstyledPixmap->rect() );
+    p.end();
+#undef CS
+    m_unstyledTexture = new Texture( m_unstyledPixmap->handle(), m_unstyledPixmap->size(), m_unstyledPixmap->depth() );
+    }
+
+void SceneOpenGL::EffectFrame::cleanup()
+    {
+    delete m_unstyledTexture;
+    m_unstyledTexture = NULL;
+    delete m_unstyledPixmap;
+    m_unstyledPixmap = NULL;
+    }
+
 } // namespace
 
 #endif
