@@ -12,25 +12,29 @@
 #include "backgrounddialog.h"
 #include "mouseplugins.h"
 
-#include <QPainter>
-#include <QFile>
 #include <QAbstractItemView>
+#include <QDesktopWidget>
+#include <QFile>
+#include <QPainter>
 #include <QStandardItemModel>
 
-#include <KStandardDirs>
-#include <KDesktopFile>
 #include <KColorScheme>
+#include <KDebug>
+#include <KDesktopFile>
+#include <KStandardDirs>
 
 #include <Plasma/Containment>
+#include <Plasma/Context>
+#include <Plasma/Corona>
 #include <Plasma/FrameSvg>
 #include <Plasma/Wallpaper>
 #include <Plasma/View>
-#include <Plasma/Corona>
 
 #include "kworkspace/screenpreviewwidget.h"
+#include "kworkspace/kactivityinfo.h"
+#include "kworkspace/kactivitycontroller.h"
 
 #include "ui_BackgroundDialog.h"
-#include "ui_ActivityConfiguration.h"
 
 typedef QPair<QString, QString> WallpaperInfo;
 Q_DECLARE_METATYPE(WallpaperInfo)
@@ -185,13 +189,12 @@ public:
     {
     }
 
-    Ui::ActivityConfiguration activityUi;
     Ui::BackgroundDialog backgroundDialogUi;
 
     QStandardItemModel* containmentModel;
     Plasma::Wallpaper* wallpaper;
     Plasma::View* view;
-    Plasma::Containment* containment;
+    QWeakPointer<Plasma::Containment> containment;
     ScreenPreviewWidget* preview;
     KPageWidgetItem *activityItem;
     KPageWidgetItem *appearanceItem;
@@ -206,26 +209,21 @@ BackgroundDialog::BackgroundDialog(const QSize& res, Plasma::Containment *c, Pla
 {
     setWindowIcon(KIcon("preferences-desktop-wallpaper"));
     setCaption(i18n("Desktop Settings"));
-    showButtonSeparator(true);
     setButtons(Ok | Cancel | Apply);
 
     QWidget *main= new QWidget(this);
     d->backgroundDialogUi.setupUi(main);
-    d->appearanceItem = addPage(main, i18n("Wallpaper"), "preferences-desktop-wallpaper");
-    
-    QWidget *activity = new QWidget(this);
-    d->activityUi.setupUi(activity);
-    d->activityItem = addPage(activity, i18n("Activity"), "plasma");
-    
+    d->appearanceItem = addPage(main, i18n("View"), "preferences-desktop-wallpaper");
+
     qreal previewRatio = (qreal)res.width() / (qreal)res.height();
     QSize monitorSize(200, int(200 * previewRatio));
 
-    d->backgroundDialogUi.m_monitor->setFixedSize(200, 200);
-    d->backgroundDialogUi.m_monitor->setText(QString());
-    d->backgroundDialogUi.m_monitor->setWhatsThis(i18n(
+    d->backgroundDialogUi.monitor->setFixedSize(200, 200);
+    d->backgroundDialogUi.monitor->setText(QString());
+    d->backgroundDialogUi.monitor->setWhatsThis(i18n(
         "This picture of a monitor contains a preview of "
         "what the current settings will look like on your desktop."));
-    d->preview = new ScreenPreviewWidget(d->backgroundDialogUi.m_monitor);
+    d->preview = new ScreenPreviewWidget(d->backgroundDialogUi.monitor);
     d->preview->setRatio(previewRatio);
     d->preview->resize(200, 200);
 
@@ -234,35 +232,45 @@ BackgroundDialog::BackgroundDialog(const QSize& res, Plasma::Containment *c, Pla
     connect(this, SIGNAL(applyClicked()), this, SLOT(saveConfig()));
 
     if (d->containment) {
-        connect(d->containment, SIGNAL(destroyed()), this, SLOT(close()));
+        connect(d->containment.data(), SIGNAL(destroyed()), this, SLOT(close()));
+        connect(d->containment.data(), SIGNAL(immutabilityChanged(Plasma::ImmutabilityType)),
+                this, SLOT(containmentImmutabilityChanged(Plasma::ImmutabilityType)));
     }
 
     d->containmentModel = new QStandardItemModel(this);
-    d->activityUi.m_containmentComboBox->setModel(d->containmentModel);
-    d->activityUi.m_containmentComboBox->setItemDelegate(new AppletDelegate());
+    d->backgroundDialogUi.containmentComboBox->setModel(d->containmentModel);
+    d->backgroundDialogUi.containmentComboBox->setItemDelegate(new AppletDelegate());
 
-    MousePlugins *m = new MousePlugins(d->containment, this);
+    MousePlugins *m = new MousePlugins(d->containment.data(), this);
     connect(m, SIGNAL(modified(bool)), this, SLOT(settingsModified(bool)));
     d->mouseItem = addPage(m, i18n("Mouse Actions"), "input-mouse");
 
-    if (d->containment && d->containment->hasConfigurationInterface()) {
-        d->containment->createConfigurationInterface(this);
+    if (d->containment && d->containment.data()->hasConfigurationInterface()) {
+        d->containment.data()->createConfigurationInterface(this);
     }
 
-    reloadConfig();
-    adjustSize();
+    QSize dialogSize = QSize(650, 720).expandedTo(sizeHint());
+    if (d->containment) {
+        const int screen = d->containment.data()->screen();
+        dialogSize = dialogSize.boundedTo(qApp->desktop()->availableGeometry(screen).size());
+    }
+    setInitialSize(dialogSize);
 
-    connect(d->activityUi.m_containmentComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsModified()));
-    connect(d->activityUi.m_activityName, SIGNAL(textChanged(const QString&)), this, SLOT(settingsModified()));
-    connect(d->activityUi.m_activityName, SIGNAL(editingFinished()), this, SLOT(checkActivityName()));
-    
-    connect(d->backgroundDialogUi.m_wallpaperMode, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsModified()));
+    KConfigGroup cg(KGlobal::config(), "BackgroundConfigDialog");
+    restoreDialogSize(cg);
+
+    reloadConfig();
+
+    connect(d->backgroundDialogUi.containmentComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsModified()));
+    connect(d->backgroundDialogUi.wallpaperMode, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsModified()));
 
     settingsModified(false);
 }
 
 BackgroundDialog::~BackgroundDialog()
 {
+    KConfigGroup cg(KGlobal::config(), "BackgroundConfigDialog");
+    saveDialogSize(cg);
     cleanup();
     delete d;
 }
@@ -273,16 +281,29 @@ void BackgroundDialog::cleanup()
     d->wallpaper = 0;
 }
 
-void BackgroundDialog::checkActivityName()
+void BackgroundDialog::containmentImmutabilityChanged(Plasma::ImmutabilityType type)
 {
-    if (d->containment && d->activityUi.m_activityName->text().isEmpty()) {
-        d->activityUi.m_activityName->setText(d->containment->activity());
+    bool showWidgetsUnlocking = false;
+
+    if (d->containment && d->containment.data()->corona()) {
+        QAction *unlockAction = d->containment.data()->corona()->action("lock widgets");
+        if (unlockAction) {
+            connect(d->backgroundDialogUi.unlockButton, SIGNAL(clicked()), unlockAction, SLOT(trigger()), Qt::UniqueConnection);
+            showWidgetsUnlocking = (type == Plasma::UserImmutable);
+            kDebug() << showWidgetsUnlocking << type;
+        }
     }
+
+    d->backgroundDialogUi.widgetLocking->setVisible(showWidgetsUnlocking);
+
+    const bool enabled = type == Plasma::Mutable;
+    d->backgroundDialogUi.containmentLabel->setEnabled(enabled);
+    d->backgroundDialogUi.containmentComboBox->setEnabled(enabled);
 }
 
 void BackgroundDialog::reloadConfig()
 {
-    disconnect(d->backgroundDialogUi.m_wallpaperMode, SIGNAL(currentIndexChanged(int)), this, SLOT(changeBackgroundMode(int)));
+    disconnect(d->backgroundDialogUi.wallpaperMode, SIGNAL(currentIndexChanged(int)), this, SLOT(changeBackgroundMode(int)));
     int containmentIndex = 0;
     int wallpaperIndex = 0;
 
@@ -291,39 +312,48 @@ void BackgroundDialog::reloadConfig()
     d->containmentModel->clear();
     int i = 0;
     foreach (const KPluginInfo& info, plugins) {
+        if (info.property("NoDisplay").toBool()) {
+            continue;
+        }
+
         QStandardItem* item = new QStandardItem(KIcon(info.icon()), info.name());
         item->setData(info.comment(), AppletDelegate::DescriptionRole);
         item->setData(info.pluginName(), AppletDelegate::PluginNameRole);
         d->containmentModel->appendRow(item);
 
-        if (d->containment && info.pluginName() == d->containment->pluginName()) {
+        if (d->containment && info.pluginName() == d->containment.data()->pluginName()) {
             containmentIndex = i;
         }
 
         ++i;
     }
 
-    d->activityUi.m_containmentComboBox->setCurrentIndex(containmentIndex);
-    
+    d->backgroundDialogUi.containmentComboBox->setCurrentIndex(containmentIndex);
+
     if (d->containment) {
-        d->activityUi.m_activityName->setText(d->containment->activity());
+        containmentImmutabilityChanged(d->containment.data()->immutability());
+    } else {
+        d->backgroundDialogUi.containmentLabel->setEnabled(false);
+        d->backgroundDialogUi.containmentComboBox->setEnabled(false);
     }
 
     // Wallpaper
-    bool doWallpaper = !d->containment || d->containment->drawWallpaper();
+    bool doWallpaper = !d->containment || d->containment.data()->drawWallpaper();
     #if 0
     d->wallpaperLabel->setVisible(doWallpaper);
     #endif
-    d->backgroundDialogUi.m_wallpaperGroup->setVisible(doWallpaper);
-    d->backgroundDialogUi.m_monitor->setVisible(doWallpaper);
+
+    d->backgroundDialogUi.wallpaperGroup->setVisible(doWallpaper);
+    d->backgroundDialogUi.monitor->setVisible(doWallpaper);
     d->preview->setVisible(doWallpaper);
+
     //kDebug() << "do wallpapers?!" << doWallpaper;
     if (doWallpaper) {
         // Load wallpaper plugins
         QString currentPlugin;
         QString currentMode;
 
-        Plasma::Wallpaper *currentWallpaper = d->containment ? d->containment->wallpaper() : 0;
+        Plasma::Wallpaper *currentWallpaper = d->containment ? d->containment.data()->wallpaper() : 0;
         if (currentWallpaper) {
             currentPlugin = currentWallpaper->pluginName();
             currentMode = currentWallpaper->renderingMode().name();
@@ -332,7 +362,7 @@ void BackgroundDialog::reloadConfig()
         }
 
         plugins = Plasma::Wallpaper::listWallpaperInfo();
-        d->backgroundDialogUi.m_wallpaperMode->clear();
+        d->backgroundDialogUi.wallpaperMode->clear();
         int i = 0;
         foreach (const KPluginInfo& info, plugins) {
             //kDebug() << "doing wallpaper" << info.pluginName() << currentPlugin;
@@ -345,7 +375,7 @@ void BackgroundDialog::reloadConfig()
                 }
 
                 foreach (const KServiceAction& mode, modes) {
-                    d->backgroundDialogUi.m_wallpaperMode->addItem(KIcon(mode.icon()), mode.text(),
+                    d->backgroundDialogUi.wallpaperMode->addItem(KIcon(mode.icon()), mode.text(),
                                     QVariant::fromValue(WallpaperInfo(info.pluginName(), mode.name())));
                     //kDebug() << matches << mode.name() << currentMode;
                     if (matches && mode.name() == currentMode) {
@@ -355,7 +385,7 @@ void BackgroundDialog::reloadConfig()
                     ++i;
                 }
             } else {
-                d->backgroundDialogUi.m_wallpaperMode->addItem(KIcon(info.icon()), info.name(),
+                d->backgroundDialogUi.wallpaperMode->addItem(KIcon(info.icon()), info.name(),
                                 QVariant::fromValue(WallpaperInfo(info.pluginName(), QString())));
                 if (matches) {
                     wallpaperIndex = i;
@@ -366,12 +396,13 @@ void BackgroundDialog::reloadConfig()
             }
         }
 
-        //kDebug() << "match is said to be" << wallpaperIndex << "out of" << d->backgroundDialogUi.m_wallpaperMode->count();
-        d->backgroundDialogUi.m_wallpaperMode->setCurrentIndex(wallpaperIndex);
+        //kDebug() << "match is said to be" << wallpaperIndex << "out of" << d->backgroundDialogUi.wallpaperMode->count();
+        d->backgroundDialogUi.wallpaperMode->setCurrentIndex(wallpaperIndex);
         changeBackgroundMode(wallpaperIndex);
     }
 
-    connect(d->backgroundDialogUi.m_wallpaperMode, SIGNAL(currentIndexChanged(int)), this, SLOT(changeBackgroundMode(int)));
+
+    connect(d->backgroundDialogUi.wallpaperMode, SIGNAL(currentIndexChanged(int)), this, SLOT(changeBackgroundMode(int)));
     settingsModified(false);
 }
 
@@ -379,10 +410,10 @@ void BackgroundDialog::changeBackgroundMode(int mode)
 {
     kDebug();
     QWidget* w = 0;
-    WallpaperInfo wallpaperInfo = d->backgroundDialogUi.m_wallpaperMode->itemData(mode).value<WallpaperInfo>();
+    WallpaperInfo wallpaperInfo = d->backgroundDialogUi.wallpaperMode->itemData(mode).value<WallpaperInfo>();
 
-    if (d->backgroundDialogUi.m_wallpaperGroup->layout()->count() > 1) {
-        QLayoutItem *item = d->backgroundDialogUi.m_wallpaperGroup->layout()->takeAt(1);
+    if (d->backgroundDialogUi.wallpaperGroup->layout()->count() > 1) {
+        QLayoutItem *item = d->backgroundDialogUi.wallpaperGroup->layout()->takeAt(1);
         QWidget *widget = item->widget();
         delete item;
         delete widget;
@@ -398,59 +429,64 @@ void BackgroundDialog::changeBackgroundMode(int mode)
     }
 
     if (d->wallpaper) {
+        d->wallpaper->setPreviewing(true);
         d->preview->setPreview(d->wallpaper);
         d->wallpaper->setRenderingMode(wallpaperInfo.second);
         KConfigGroup cfg = wallpaperConfig(wallpaperInfo.first);
         //kDebug() << "making a" << wallpaperInfo.first << "in mode" << wallpaperInfo.second;
         if (d->containment) {
-            d->wallpaper->setTargetSizeHint(d->containment->size());
+            d->wallpaper->setTargetSizeHint(d->containment.data()->size());
         }
         d->wallpaper->restore(cfg);
 
-        WallpaperWidget *wallpaperWidget = new WallpaperWidget(d->backgroundDialogUi.m_wallpaperGroup);
+        WallpaperWidget *wallpaperWidget = new WallpaperWidget(d->backgroundDialogUi.wallpaperGroup);
         w = d->wallpaper->createConfigurationInterface(wallpaperWidget);
         connect(wallpaperWidget, SIGNAL(modified(bool)), this, SLOT(settingsModified(bool)));
+
+        const bool needsPreviewDuringConfiguration = d->wallpaper->needsPreviewDuringConfiguration();
+        d->backgroundDialogUi.monitorWidget->setVisible(needsPreviewDuringConfiguration);
+        d->preview->setVisible(needsPreviewDuringConfiguration);
     }
 
     if (!w) {
-        w = new QWidget(d->backgroundDialogUi.m_wallpaperGroup);
+        w = new QWidget(d->backgroundDialogUi.wallpaperGroup);
     } else if (w->layout()) {
         QGridLayout *gridLayout = dynamic_cast<QGridLayout *>(w->layout());
         if (gridLayout) {
-            gridLayout->setColumnMinimumWidth(0, d->backgroundDialogUi.m_wallpaperTypeLabel->geometry().right());
+            gridLayout->setColumnMinimumWidth(0, d->backgroundDialogUi.wallpaperTypeLabel->geometry().right());
             gridLayout->setColumnStretch(0, 0);
             gridLayout->setColumnStretch(1, 10);
             gridLayout->setContentsMargins(0, 0, 0, 0);
         }
     }
 
-    d->backgroundDialogUi.m_wallpaperGroup->layout()->addWidget(w);
+    d->backgroundDialogUi.wallpaperGroup->layout()->addWidget(w);
     settingsModified(true);
 }
 
 KConfigGroup BackgroundDialog::wallpaperConfig(const QString &plugin)
 {
     //FIXME: we have details about the structure of the containment config duplicated here!
-    KConfigGroup cfg = d->containment ? d->containment->config() : KConfigGroup(KGlobal::config(), "Wallpaper");
+    KConfigGroup cfg = d->containment ? d->containment.data()->config() : KConfigGroup(KGlobal::config(), "Wallpaper");
     cfg = KConfigGroup(&cfg, "Wallpaper");
     return KConfigGroup(&cfg, plugin);
 }
 
 void BackgroundDialog::saveConfig()
 {
-    QString wallpaperPlugin = d->backgroundDialogUi.m_wallpaperMode->itemData(d->backgroundDialogUi.m_wallpaperMode->currentIndex()).value<WallpaperInfo>().first;
-    QString wallpaperMode = d->backgroundDialogUi.m_wallpaperMode->itemData(d->backgroundDialogUi.m_wallpaperMode->currentIndex()).value<WallpaperInfo>().second;
-    QString containment = d->activityUi.m_containmentComboBox->itemData(d->activityUi.m_containmentComboBox->currentIndex(),
+    const QString wallpaperPlugin = d->backgroundDialogUi.wallpaperMode->itemData(d->backgroundDialogUi.wallpaperMode->currentIndex()).value<WallpaperInfo>().first;
+    const QString wallpaperMode = d->backgroundDialogUi.wallpaperMode->itemData(d->backgroundDialogUi.wallpaperMode->currentIndex()).value<WallpaperInfo>().second;
+    const QString containment = d->backgroundDialogUi.containmentComboBox->itemData(d->backgroundDialogUi.containmentComboBox->currentIndex(),
                                                           AppletDelegate::PluginNameRole).toString();
 
     // Containment
     if (d->containment) {
-        if (d->containment->pluginName() != containment) {
-            disconnect(d->containment, SIGNAL(destroyed()), this, SLOT(close()));
-            disconnect(this, 0, d->containment, 0);
+        if (d->containment.data()->pluginName() != containment) {
+            disconnect(d->containment.data(), SIGNAL(destroyed()), this, SLOT(close()));
+            disconnect(this, 0, d->containment.data(), 0);
 
-            d->containment = d->view->swapContainment(d->containment, containment);
-            emit containmentPluginChanged(d->containment);
+            d->containment = d->view->swapContainment(d->containment.data(), containment);
+            emit containmentPluginChanged(d->containment.data());
 
             //remove all pages but our own
             KPageWidgetModel *m = qobject_cast<KPageWidgetModel *>(pageWidget()->model());
@@ -466,8 +502,8 @@ void BackgroundDialog::saveConfig()
 
                     KPageWidgetItem *item = m->item(idx);
 
-                    if (item && item != d->appearanceItem && item != d->mouseItem
-                      && item != d->activityItem) {
+                    if (item && item != d->appearanceItem &&
+                        item != d->mouseItem && item != d->activityItem) {
                         itemsToRemove.append(item);
                     }
                 }
@@ -478,16 +514,14 @@ void BackgroundDialog::saveConfig()
             }
 
             //add the new containment's config
-            if (d->containment->hasConfigurationInterface()) {
-                d->containment->createConfigurationInterface(this);
+            if (d->containment.data()->hasConfigurationInterface()) {
+                d->containment.data()->createConfigurationInterface(this);
             }
-            connect(d->containment, SIGNAL(destroyed()), this, SLOT(close()));
+            connect(d->containment.data(), SIGNAL(destroyed()), this, SLOT(close()));
         }
 
-        d->containment->setActivity(d->activityUi.m_activityName->text());
-
         // Wallpaper
-        Plasma::Wallpaper *currentWallpaper = d->containment->wallpaper();
+        Plasma::Wallpaper *currentWallpaper = d->containment.data()->wallpaper();
         if (currentWallpaper) {
             KConfigGroup cfg = wallpaperConfig(currentWallpaper->pluginName());
             currentWallpaper->save(cfg);
@@ -500,7 +534,7 @@ void BackgroundDialog::saveConfig()
     }
 
     if (d->containment) {
-        d->containment->setWallpaper(wallpaperPlugin, wallpaperMode);
+        d->containment.data()->setWallpaper(wallpaperPlugin, wallpaperMode);
     }
 
     settingsModified(false);
