@@ -23,6 +23,7 @@
 #include <QFileInfo>
 #include <QScriptValueIterator>
 
+#include <KDebug>
 #include <kdeversion.h>
 #include <KServiceTypeTrader>
 #include <KShell>
@@ -48,7 +49,7 @@ ScriptEngine::ScriptEngine(Plasma::Corona *corona, QObject *parent)
       m_corona(corona)
 {
     Q_ASSERT(m_corona);
-    AppInterface *interface = new AppInterface(corona, this);
+    AppInterface *interface = new AppInterface(this);
     connect(interface, SIGNAL(print(QString)), this, SIGNAL(print(QString)));
     m_scriptSelf = newQObject(interface, QScriptEngine::QtOwnership,
                               QScriptEngine::ExcludeSuperClassProperties | QScriptEngine::ExcludeSuperClassMethods);
@@ -193,25 +194,6 @@ QScriptValue ScriptEngine::panelById(QScriptContext *context, QScriptEngine *eng
     return engine->undefinedValue();
 }
 
-QScriptValue ScriptEngine::activities(QScriptContext *context, QScriptEngine *engine)
-{
-    Q_UNUSED(context)
-
-    QScriptValue panels = engine->newArray();
-    ScriptEngine *env = envFor(engine);
-    int count = 0;
-
-    foreach (Plasma::Containment *c, env->m_corona->containments()) {
-        if (!isPanel(c)) {
-            panels.setProperty(count, env->wrap(c));
-            ++count;
-        }
-    }
-
-    panels.setProperty("length", count);
-    return panels;
-}
-
 QScriptValue ScriptEngine::panels(QScriptContext *context, QScriptEngine *engine)
 {
     Q_UNUSED(context)
@@ -301,8 +283,15 @@ QScriptValue ScriptEngine::loadTemplate(QScriptContext *context, QScriptEngine *
     ScriptEngine *env = envFor(engine);
     env->globalObject().setProperty("templateName", env->newVariant(info.name()), QScriptValue::ReadOnly | QScriptValue::Undeletable);
     env->globalObject().setProperty("templateComment", env->newVariant(info.comment()), QScriptValue::ReadOnly | QScriptValue::Undeletable);
+
+    QScriptValue rv = env->newObject();
+    QScriptContext *ctx = env->pushContext();
+    ctx->setThisObject(rv);
+
     env->evaluateScript(script, path);
-    return true;
+
+    env->popContext();
+    return rv;
 }
 
 void ScriptEngine::setupEngine()
@@ -327,9 +316,6 @@ void ScriptEngine::setupEngine()
     m_scriptSelf.setProperty("panels", newFunction(ScriptEngine::panels));
     m_scriptSelf.setProperty("fileExists", newFunction(ScriptEngine::fileExists));
     m_scriptSelf.setProperty("loadTemplate", newFunction(ScriptEngine::loadTemplate));
-    m_scriptSelf.setProperty("applicationVersion", KGlobal::mainComponent().aboutData()->version(), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-    m_scriptSelf.setProperty("scriptingVersion", newVariant(2), QScriptValue::ReadOnly | QScriptValue::Undeletable);
-    m_scriptSelf.setProperty("platformVersion", KDE::versionString(), QScriptValue::ReadOnly | QScriptValue::Undeletable);
 
     setGlobalObject(m_scriptSelf);
 }
@@ -344,6 +330,30 @@ bool ScriptEngine::isPanel(const Plasma::Containment *c)
            c->containmentType() == Plasma::Containment::CustomPanelContainment;
 }
 
+QScriptValue ScriptEngine::activities(QScriptContext *context, QScriptEngine *engine)
+{
+    Q_UNUSED(context)
+
+    QScriptValue containments = engine->newArray();
+    ScriptEngine *env = envFor(engine);
+    int count = 0;
+
+    foreach (Plasma::Containment *c, env->corona()->containments()) {
+        if (!isPanel(c)) {
+            containments.setProperty(count, env->wrap(c));
+            ++count;
+        }
+    }
+
+    containments.setProperty("length", count);
+    return containments;
+}
+
+Plasma::Corona *ScriptEngine::corona() const
+{
+    return m_corona;
+}
+
 bool ScriptEngine::evaluateScript(const QString &script, const QString &path)
 {
     //kDebug() << "evaluating" << m_editor->toPlainText();
