@@ -52,6 +52,13 @@ namespace Ifaces
          * @returns the Unique Network Identifier of the current network
          */
         virtual QString uni() const = 0;
+        /**
+         * Retrieves the Unique Device Identifier (UDI).
+         * This is needed for ModemManager support.
+         *
+         * @returns the Unique Device Identifier of the current device
+         */
+        virtual QString udi() const = 0;
         /**
          * The system name for the network interface
          */
