@@ -43,7 +43,12 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <QX11Info>
 #endif
 
+#ifdef Q_WS_WIN
+#include <windows.h>
+#endif
+
 #include <kephal/screens.h>
+#include <kactivityconsumer.h>
 
 namespace TaskManager
 {
@@ -64,23 +69,45 @@ TaskManager* TaskManager::self()
 class TaskManager::Private
 {
 public:
-    Private()
-        : active(0),
-          startupInfo(0)
+    Private(TaskManager *manager)
+        : q(manager),
+          active(0),
+          startupInfo(0),
+          watcher(0)
     {
     }
 
+    void onAppExitCleanup()
+    {
+        q->disconnect(KWindowSystem::self(), 0, q, 0);
+        delete watcher;
+        watcher = 0;
+        delete startupInfo;
+        startupInfo = 0;
+
+        foreach (TaskPtr task, tasksByWId) {
+            task->clearPixmapData();
+        }
+
+        foreach (StartupPtr startup, startups) {
+            startup->clearPixmapData();
+        }
+    }
+
+    TaskManager *q;
     TaskPtr active;
     KStartupInfo* startupInfo;
+    KDirWatch *watcher;
     TaskDict tasksByWId;
     StartupList startups;
     WindowList skiptaskbarWindows;
     QSet<QUuid> trackGeometryTokens;
+    KActivityConsumer activityConsumer;
 };
 
 TaskManager::TaskManager()
     : QObject(),
-      d(new Private)
+      d(new Private(this))
 {
     KGlobal::locale()->insertCatalog("libtaskmanager");
     connect(KWindowSystem::self(), SIGNAL(windowAdded(WId)),
@@ -91,8 +118,15 @@ TaskManager::TaskManager()
             this,       SLOT(activeWindowChanged(WId)));
     connect(KWindowSystem::self(), SIGNAL(currentDesktopChanged(int)),
             this,       SLOT(currentDesktopChanged(int)));
-    connect(KWindowSystem::self(), SIGNAL(windowChanged(WId,unsigned int)),
-            this,       SLOT(windowChanged(WId,unsigned int)));
+    connect(KWindowSystem::self(), SIGNAL(windowChanged(WId,const unsigned long*)),
+            this,       SLOT(windowChanged(WId,const unsigned long*)));
+    connect(&d->activityConsumer, SIGNAL(currentActivityChanged(QString)),
+            this,       SLOT(currentActivityChanged(QString)));
+    if (QCoreApplication::instance()) {
+        connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), this, SLOT(onAppExitCleanup()));
+    }
+
+    currentActivityChanged(d->activityConsumer.currentActivity());
 
     // register existing windows
     const QList<WId> windows = KWindowSystem::windows();
@@ -106,11 +140,11 @@ TaskManager::TaskManager()
     WId win = KWindowSystem::activeWindow();
     activeWindowChanged(win);
 
-    KDirWatch *watcher = new KDirWatch(this);
-    watcher->addFile(KGlobal::dirs()->locateLocal("config", "klaunchrc"));
-    connect(watcher, SIGNAL(dirty(const QString&)), this, SLOT(configureStartup()));
-    connect(watcher, SIGNAL(created(const QString&)), this, SLOT(configureStartup()));
-    connect(watcher, SIGNAL(deleted(const QString&)), this, SLOT(configureStartup()));
+    d->watcher = new KDirWatch(this);
+    d->watcher->addFile(KGlobal::dirs()->locateLocal("config", "klaunchrc"));
+    connect(d->watcher, SIGNAL(dirty(const QString&)), this, SLOT(configureStartup()));
+    connect(d->watcher, SIGNAL(created(const QString&)), this, SLOT(configureStartup()));
+    connect(d->watcher, SIGNAL(deleted(const QString&)), this, SLOT(configureStartup()));
 
     configureStartup();
 }
@@ -176,6 +210,7 @@ TaskPtr TaskManager::findTask(int desktop, const QPoint& p)
         {
             continue;
         }
+        //FIXME activities?
 
         if (t->isIconified() || t->isShaded())
         {
@@ -198,6 +233,7 @@ TaskPtr TaskManager::findTask(int desktop, const QPoint& p)
 
 void TaskManager::windowAdded(WId w )
 {
+#ifdef Q_WS_X11
     NETWinInfo info(QX11Info::display(), w, QX11Info::appRootWindow(),
                     NET::WMWindowType | NET::WMPid | NET::WMState);
 
@@ -240,6 +276,7 @@ void TaskManager::windowAdded(WId w )
             }
         }
     }
+#endif
 
     TaskPtr t(new Task(w, 0));
     d->tasksByWId[w] = t;
@@ -293,9 +330,10 @@ void TaskManager::windowRemoved(WId w)
     }
 }
 
-void TaskManager::windowChanged(WId w, unsigned int dirty)
+void TaskManager::windowChanged(WId w, const unsigned long *dirty)
 {
-    if (dirty & NET::WMState) {
+#ifdef Q_WS_X11
+    if (dirty[NETWinInfo::PROTOCOLS] & NET::WMState) {
         NETWinInfo info (QX11Info::display(), w, QX11Info::appRootWindow(),
                          NET::WMState | NET::XAWMState);
 
@@ -314,10 +352,11 @@ void TaskManager::windowChanged(WId w, unsigned int dirty)
     }
 
     // check if any state we are interested in is marked dirty
-    if (!(dirty & (NET::WMVisibleName | NET::WMName |
-                   NET::WMState | NET::WMIcon |
-                   NET::XAWMState | NET::WMDesktop) ||
-          (trackGeometry() && dirty & NET::WMGeometry))) {
+    if (!(dirty[NETWinInfo::PROTOCOLS] & (NET::WMVisibleName | NET::WMName |
+                                          NET::WMState | NET::WMIcon |
+                                          NET::XAWMState | NET::WMDesktop) ||
+          (trackGeometry() && dirty[NETWinInfo::PROTOCOLS] & NET::WMGeometry) ||
+         (dirty[NETWinInfo::PROTOCOLS2] & NET::WM2Activities))) {
         return;
     }
 
@@ -327,17 +366,19 @@ void TaskManager::windowChanged(WId w, unsigned int dirty)
         return;
     }
 
-    //kDebug() << "TaskManager::windowChanged " << w << " " << dirty;
+    //kDebug() << "TaskManager::windowChanged " << w << " " << dirty[NETWinInfo::PROTOCOLS] << dirty[NETWinInfo::PROTOCOLS2];
 
-    if (dirty & NET::WMState) {
-        t->updateDemandsAttentionState(w);
+    unsigned long propagatedChanges = 0;
+    if ((dirty[NETWinInfo::PROTOCOLS] & NET::WMState) && t->updateDemandsAttentionState(w)) {
+        propagatedChanges = NET::WMState;
     }
 
-    //kDebug() << "got changes, but will we refresh?" << dirty;
-    if (dirty) {
+    //kDebug() << "got changes, but will we refresh?" << dirty[NETWinInfo::PROTOCOLS] << dirty[NETWinInfo::PROTOCOLS2];
+    if (dirty[NETWinInfo::PROTOCOLS] || dirty[NETWinInfo::PROTOCOLS2]) {
         // only refresh this stuff if we have other changes besides icons
-        t->refresh(dirty);
+        t->refresh(Task::WindowProperties(dirty[NETWinInfo::PROTOCOLS] | propagatedChanges, dirty[NETWinInfo::PROTOCOLS2]));
     }
+#endif
 }
 
 void TaskManager::taskChanged(::TaskManager::TaskChanges changes)
@@ -385,6 +426,11 @@ void TaskManager::currentDesktopChanged(int desktop)
     emit desktopChanged(desktop);
 }
 
+void TaskManager::currentActivityChanged(const QString &activity)
+{
+    emit activityChanged(activity);
+}
+
 void TaskManager::gotNewStartup( const KStartupInfoId& id, const KStartupInfoData& data )
 {
     StartupPtr s( new Startup( id, data, 0 ) );
@@ -484,10 +530,11 @@ bool TaskManager::isOnTop(const Task* task) const
             if (t == task) {
                 return true;
             }
-
+#ifndef Q_WS_WIN
             if (!t->isIconified() && (t->isAlwaysOnTop() == task->isAlwaysOnTop())) {
                 return false;
             }
+#endif
         }
     } while (it-- != begin);
 
@@ -531,6 +578,11 @@ int TaskManager::currentDesktop() const
     return KWindowSystem::currentDesktop();
 }
 
+QString TaskManager::currentActivity() const
+{
+    return d->activityConsumer.currentActivity(); //TODO cache
+}
+
 } // TaskManager namespace
 
 
