@@ -1,5 +1,3 @@
-#ifndef ABSTRACTTASKITEM_CPP
-#define ABSTRACTTASKITEM_CPP
 /***************************************************************************
  *   Copyright (C) 2007 by Robert Knight <robertknight@gmail.com>          *
  *   Copyright (C) 2008 by Alexis Ménard <darktears31@gmail.com>           *
@@ -26,6 +24,7 @@
 
 // Qt
 #include <QApplication>
+#include <QFileInfo>
 #include <QGraphicsLinearLayout>
 #include <QGraphicsSceneContextMenuEvent>
 #include <QGraphicsView>
@@ -40,12 +39,12 @@
 
 // KDE
 #include <KAuthorized>
+#include <KColorUtils>
 #include <KDebug>
+#include <KGlobalSettings>
 #include <KIcon>
 #include <KIconEffect>
-#include <KGlobalSettings>
 #include <KIconLoader>
-#include <KColorUtils>
 
 #include <NETWinInfo>
 
@@ -95,9 +94,8 @@ AbstractTaskItem::AbstractTaskItem(QGraphicsWidget *parent, Tasks *applet)
     setFocusPolicy(Qt::StrongFocus);
     setFlag(QGraphicsItem::ItemIsFocusable);
 
-    Plasma::ToolTipManager::self()->registerWidget(this);
-    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(syncActiveRect()));
-    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(queueUpdate()));
+    checkSettings();
+    connect(applet->itemBackground(), SIGNAL(repaintNeeded()), this, SLOT(syncActiveRect()));
     connect(applet, SIGNAL(settingsChanged()), this, SLOT(checkSettings()));
 }
 
@@ -151,6 +149,7 @@ void AbstractTaskItem::setPreferredOnscreenSize()
 
 AbstractTaskItem::~AbstractTaskItem()
 {
+    stopWindowHoverEffect();
     emit destroyed(this);
     Plasma::ToolTipManager::self()->unregisterWidget(this);
 }
@@ -200,16 +199,19 @@ void AbstractTaskItem::setTaskFlags(const TaskFlags flags)
 {
     if (((m_flags & TaskWantsAttention) != 0) != ((flags & TaskWantsAttention) != 0)) {
         //kDebug() << "task attention state changed" << m_attentionTimerId;
+        m_flags = flags;
         if (flags & TaskWantsAttention) {
-            m_applet->needsVisualFocus();
+            m_applet->needsVisualFocus(true);
             // start attention getting
             if (!m_attentionTimerId) {
                 m_attentionTimerId = startTimer(500);
             }
-        } else if (m_attentionTimerId) {
-            killTimer(m_attentionTimerId);
-            m_attentionTimerId = 0;
-            // stop attention getting
+        } else {
+            m_applet->needsVisualFocus(false);
+            if (m_attentionTimerId) {
+                killTimer(m_attentionTimerId);
+                m_attentionTimerId = 0;
+            }
         }
     }
 
@@ -256,7 +258,6 @@ void AbstractTaskItem::fadeBackground(const QString &newBackground, int duration
     }
 }
 
-
 AbstractTaskItem::TaskFlags AbstractTaskItem::taskFlags() const
 {
     return m_flags;
@@ -300,8 +301,8 @@ void AbstractTaskItem::queueUpdate()
         return;
     }
 
-    if (m_lastUpdate.elapsed() < 200) {
-        m_updateTimerId = startTimer(200);
+    if (m_lastUpdate.elapsed() < 100) {
+        m_updateTimerId = startTimer(100);
         return;
     }
 
@@ -339,6 +340,7 @@ void AbstractTaskItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
     if (w && this != m_applet->rootGroupItem()) {
         if (m_hoverEffectTimerId) {
             killTimer(m_hoverEffectTimerId);
+            m_hoverEffectTimerId = 0;
         }
 
         m_hoverEffectTimerId = startTimer(HOVER_EFFECT_TIMEOUT);
@@ -367,10 +369,6 @@ void AbstractTaskItem::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
 
 void AbstractTaskItem::stopWindowHoverEffect()
 {
-    if (!parentGroup()) {
-        return;
-    }
-
     if (m_hoverEffectTimerId) {
         killTimer(m_hoverEffectTimerId);
         m_hoverEffectTimerId = 0;
@@ -383,7 +381,7 @@ void AbstractTaskItem::stopWindowHoverEffect()
 
 void AbstractTaskItem::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
-    if (event->button() == Qt::LeftButton) {
+    if (event->button() == Qt::LeftButton && boundingRect().contains(event->pos())) {
         activate();
     }
 }
@@ -464,7 +462,7 @@ void AbstractTaskItem::timerEvent(QTimerEvent *event)
 #ifdef Q_WS_X11
         QList<WId> windows;
 
-        if (m_abstractItem && m_abstractItem->isGroupItem()) {
+        if (m_abstractItem && m_abstractItem->itemType() == TaskManager::GroupItemType) {
             TaskManager::TaskGroup *group = qobject_cast<TaskManager::TaskGroup *>(m_abstractItem);
 
             if (group) {
@@ -475,14 +473,12 @@ void AbstractTaskItem::timerEvent(QTimerEvent *event)
                 }
 
                 foreach (AbstractGroupableItem *item, group->members()) {
-                    if (item->isGroupItem()) {
-                        //TODO: recurse through sub-groups?
-                    } else {
+                    if (item->itemType() == TaskManager::TaskItemType) {
                         TaskManager::TaskItem *taskItem = qobject_cast<TaskManager::TaskItem *>(item);
                         if (taskItem && taskItem->task()) {
                             windows.append(taskItem->task()->window());
                         }
-                    }
+                    } //TODO: if taskgroup, recurse through sub-groups?
                 }
             }
         } else {
@@ -519,8 +515,10 @@ void AbstractTaskItem::paint(QPainter *painter,
     //kDebug() << "painting" << (QObject*)this << text();
     painter->setRenderHint(QPainter::Antialiasing);
 
-    // draw background
-    drawBackground(painter, option, widget);
+    if (m_abstractItem->itemType() != TaskManager::LauncherItemType) { //Launchers have no frame
+        // draw background
+        drawBackground(painter, option, widget);
+    }
 
     // draw icon and text
     drawTask(painter, option, widget);
@@ -548,7 +546,8 @@ void AbstractTaskItem::syncActiveRect()
     // check to see if there is enough room!
     QFontMetrics fm(font());
     const int minimumWidth = left + 8 + IconTextSpacing + right;
-    m_showText = (size().width() >= fm.width("mmm") + minimumWidth);
+    m_showText = (size().width() >= fm.width("M") * 6 + minimumWidth);
+    queueUpdate();
 }
 
 void AbstractTaskItem::resizeEvent(QGraphicsSceneResizeEvent *event)
@@ -589,14 +588,14 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     */
     Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
 
-    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
-    if (size().toSize() != itemBackground->frameSize()) {
-        resizeBackground(size().toSize());
-    }
-
     if (~option->state & QStyle::State_Sunken && 
         (!m_backgroundFadeAnim || m_backgroundFadeAnim->state() != QAbstractAnimation::Running)) {
         itemBackground->setElementPrefix(m_backgroundPrefix);
+        //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
+        if (size().toSize() != itemBackground->frameSize()) {
+            resizeBackground(size().toSize());
+        }
+
         if (itemBackground->frameSize() == m_activeRect.size().toSize()) {
             itemBackground->paintFrame(painter, m_activeRect.topLeft());
         } else {
@@ -607,6 +606,11 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     }
 
     itemBackground->setElementPrefix(m_oldBackgroundPrefix);
+    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
+    if (size().toSize() != itemBackground->frameSize()) {
+        resizeBackground(size().toSize());
+    }
+
     QPixmap oldBackground;
 
     if (option->state & QStyle::State_Sunken) {
@@ -618,6 +622,11 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     }
 
     itemBackground->setElementPrefix(m_backgroundPrefix);
+    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
+    if (size().toSize() != itemBackground->frameSize()) {
+        resizeBackground(size().toSize());
+    }
+
     QPixmap result = Plasma::PaintUtils::transition(oldBackground, itemBackground->framePixmap(), m_alpha);
 
     if (result.size() == m_activeRect.size().toSize()) {
@@ -625,14 +634,19 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     } else {
         painter->drawPixmap(QPoint(0,0), result);
     }
-
 }
 
 void AbstractTaskItem::drawTask(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
 {
     Q_UNUSED(option)
 
-    QRectF bounds = boundingRect().adjusted(m_applet->itemLeftMargin(), m_applet->itemTopMargin(), -m_applet->itemRightMargin(), -m_applet->itemBottomMargin());
+    QRectF bounds = boundingRect();
+
+    if (m_abstractItem->itemType() != TaskManager::LauncherItemType) {
+        bounds = bounds.adjusted(m_applet->itemLeftMargin(), m_applet->itemTopMargin(), -m_applet->itemRightMargin(), -m_applet->itemBottomMargin());
+    } else {
+        bounds = bounds.adjusted(5,5,-5,-5);
+    }
 
     WindowTaskItem *window = qobject_cast<WindowTaskItem *>(this);
     QGraphicsWidget *busyWidget;
@@ -649,7 +663,10 @@ void AbstractTaskItem::drawTask(QPainter *painter, const QStyleOptionGraphicsIte
         const bool fadingBg = m_backgroundFadeAnim && m_backgroundFadeAnim->state() == QAbstractAnimation::Running;
         if ((!fadingBg && !(option->state & QStyle::State_MouseOver)) ||
             (m_oldBackgroundPrefix != "hover" && m_backgroundPrefix != "hover")) {
-            icon().paint(painter, iconR.toRect());
+            // QIcon::paint does some alignment work and can lead to funny
+            // things when icon().size() != iconR.toRect().size()
+            QPixmap result = icon().pixmap(iconR.toRect().size());
+            painter->drawPixmap(iconR.topLeft(), result);
         } else {
             KIconEffect *effect = KIconLoader::global()->iconEffect();
             QPixmap result = icon().pixmap(iconR.toRect().size());
@@ -670,38 +687,40 @@ void AbstractTaskItem::drawTask(QPainter *painter, const QStyleOptionGraphicsIte
 
     painter->setPen(QPen(textColor(), 1.0));
 
-    if (m_showText) {
-        QRect rect = textRect(bounds).toRect();
-        if (rect.height() > 20) {
-            rect.adjust(2, 2, -2, -2); // Create a text margin
+    if (m_abstractItem->itemType() != TaskManager::LauncherItemType) {
+        if (m_showText) {
+            QRect rect = textRect(bounds).toRect();
+            if (rect.height() > 20) {
+                rect.adjust(2, 2, -2, -2); // Create a text margin
+            }
+            QTextLayout layout;
+            layout.setFont(KGlobalSettings::taskbarFont());
+            layout.setTextOption(textOption());
+
+            layoutText(layout, text(), rect.size());
+            drawTextLayout(painter, layout, rect);
         }
-        QTextLayout layout;
-        layout.setFont(KGlobalSettings::taskbarFont());
-        layout.setTextOption(textOption());
 
-        layoutText(layout, text(), rect.size());
-        drawTextLayout(painter, layout, rect);
-    }
+        TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(this);
+        if (groupItem) {
+            QFont font(KGlobalSettings::smallestReadableFont());
+            QFontMetrics fm(font);
+            QRectF rect(expanderRect(bounds));
 
-    TaskGroupItem *groupItem = qobject_cast<TaskGroupItem *>(this);
-    if (groupItem) {
-        QFont font(KGlobalSettings::smallestReadableFont());
-        QFontMetrics fm(font);
-        QRectF rect(expanderRect(bounds));
+            Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
 
-        Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
+            if (itemBackground->hasElement(expanderElement())) {
+                QSizeF arrowSize(itemBackground->elementSize(expanderElement()));
+                QRectF arrowRect(rect.center()-QPointF(arrowSize.width()/2, arrowSize.height()+fm.xHeight()/2), arrowSize);
+                itemBackground->paint(painter, arrowRect, expanderElement());
 
-        if (itemBackground->hasElement(expanderElement())) {
-            QSizeF arrowSize(itemBackground->elementSize(expanderElement()));
-            QRectF arrowRect(rect.center()-QPointF(arrowSize.width()/2, arrowSize.height()+fm.xHeight()/2), arrowSize);
-            itemBackground->paint(painter, arrowRect, expanderElement());
-
-            painter->setFont(font);
-            rect.setTop(arrowRect.bottom());
-            painter->drawText(rect, Qt::AlignHCenter|Qt::AlignTop, QString::number(groupItem->count()));
-        } else {
-            painter->setFont(font);
-            painter->drawText(rect, Qt::AlignCenter, QString::number(groupItem->count()));
+                painter->setFont(font);
+                rect.setTop(arrowRect.bottom());
+                painter->drawText(rect, Qt::AlignHCenter|Qt::AlignTop, QString::number(groupItem->count()));
+            } else {
+                painter->setFont(font);
+                painter->drawText(rect, Qt::AlignCenter, QString::number(groupItem->count()));
+            }
         }
     }
 }
@@ -851,11 +870,41 @@ void AbstractTaskItem::setBackgroundFadeAlpha(qreal progress)
     update();
 }
 
+bool AbstractTaskItem::shouldIgnoreDragEvent(QGraphicsSceneDragDropEvent *event)
+{
+   if (event->mimeData()->hasFormat(TaskManager::Task::mimetype()) ||
+       event->mimeData()->hasFormat(TaskManager::Task::groupMimetype())) {
+        return true;
+    }
+
+    if (event->mimeData()->hasFormat("text/uri-list")) {
+        // we want to check if we have executables; if so, then we treat it as a possible
+        // drop for a launcher
+        const KUrl::List uris = KUrl::List::fromMimeData(event->mimeData());
+        if (!uris.isEmpty()) {
+            foreach (const QUrl &uri, uris) {
+                KUrl url(uri);
+                if (url.isLocalFile()) {
+                    const QString path = url.toLocalFile();
+                    QFileInfo info(path);
+                    if (info.isDir() || !info.isExecutable()) {
+                        return false;
+                        break;
+                    }
+                }
+            }
+
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void AbstractTaskItem::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 {
-    if (event->mimeData()->hasFormat(TaskManager::Task::mimetype()) ||
-        event->mimeData()->hasFormat(TaskManager::Task::groupMimetype())) {
-        event->ignore(); //ignore it so the taskbar gets the event
+    if (shouldIgnoreDragEvent(event)) {
+        event->ignore();
         return;
     }
 
@@ -884,6 +933,7 @@ void AbstractTaskItem::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
 
     if (m_activateTimerId) {
         killTimer(m_activateTimerId);
+        m_activateTimerId = 0;
     }
 }
 
@@ -953,6 +1003,7 @@ void AbstractTaskItem::setGeometry(const QRectF& geometry)
     if (m_lastGeometryUpdate.elapsed() < 500) {
         if (m_updateGeometryTimerId) {
             killTimer(m_updateGeometryTimerId);
+            m_updateGeometryTimerId = 0;
         }
 
         m_updateGeometryTimerId = startTimer(500 - m_lastGeometryUpdate.elapsed());
@@ -961,20 +1012,23 @@ void AbstractTaskItem::setGeometry(const QRectF& geometry)
         m_lastGeometryUpdate.restart();
     }
 
-    QGraphicsWidget::setGeometry(geometry);
     //TODO:remove when we will have proper animated layouts
     if (m_firstGeometryUpdate && !m_layoutAnimationLock) {
+        QRectF animStartGeom(oldPos, geometry.size());
+        QGraphicsWidget::setGeometry(animStartGeom);
+
         if (m_layoutAnimation->state() == QAbstractAnimation::Running) {
             m_layoutAnimation->stop();
         }
 
-        setPos(oldPos);
         m_layoutAnimation->setEndValue(geometry.topLeft());
         m_layoutAnimation->start();
+    } else {
+        QGraphicsWidget::setGeometry(geometry);
     }
 }
 
-QRectF AbstractTaskItem::iconRect(const QRectF &b) const
+QRectF AbstractTaskItem::iconRect(const QRectF &b)
 {
     QRectF bounds(b);
     const int right = bounds.right();
@@ -1001,12 +1055,16 @@ QRectF AbstractTaskItem::iconRect(const QRectF &b) const
         }
     }
 
+    if (iconSize != m_lastIconSize) {
+        m_cachedShadow = QPixmap();
+    }
+    m_lastIconSize = iconSize;
     return QStyle::alignedRect(QApplication::layoutDirection(),
                                (m_showText ? Qt::AlignLeft : Qt::AlignCenter) | Qt::AlignVCenter,
                                iconSize, bounds.toRect());
 }
 
-QRectF AbstractTaskItem::expanderRect(const QRectF &bounds) const
+QRectF AbstractTaskItem::expanderRect(const QRectF &bounds)
 {
     const TaskGroupItem *groupItem = qobject_cast<const TaskGroupItem *>(this);
     if (!groupItem) {
@@ -1024,7 +1082,7 @@ QRectF AbstractTaskItem::expanderRect(const QRectF &bounds) const
                                expanderSize, bounds.toRect());
 }
 
-QRectF AbstractTaskItem::textRect(const QRectF &bounds) const
+QRectF AbstractTaskItem::textRect(const QRectF &bounds)
 {
     QSize size(bounds.size().toSize());
     QRectF effectiveBounds(bounds);
@@ -1144,4 +1202,3 @@ TaskManager::AbstractGroupableItem * AbstractTaskItem::abstractItem()
 }
 
 #include "abstracttaskitem.moc"
-#endif // ABSTRACTTASKITEM_CPP
