@@ -40,7 +40,7 @@
 #include <unistd.h>
 
 static const char description[] = I18N_NOOP( "The KDE desktop, panels and widgets workspace application." );
-static const char version[] = "0.3";
+static const char version[] = "0.4";
 
 extern "C"
 KDE_EXPORT int kdemain(int argc, char **argv)
@@ -96,10 +96,10 @@ KDE_EXPORT int kdemain(int argc, char **argv)
 
     QByteArray appName = "plasma-desktop";
     if (associatedScreen > 0) {
-        appName.append("-screen-").append(associatedScreen);
+        appName.append("-screen-").append(QByteArray::number(associatedScreen));
     }
 
-    KAboutData aboutData(appName, 0, ki18n("Plasma Workspace"),
+    KAboutData aboutData(appName, 0, ki18n("Plasma Desktop Shell"),
                          version, ki18n(description), KAboutData::License_GPL,
                          ki18n("Copyright 2006-2009, The KDE Team"));
     aboutData.addAuthor(ki18n("Aaron J. Seigo"),
