@@ -20,7 +20,6 @@
 #include "panelview.h"
 
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QGraphicsLinearLayout>
 #include <QPropertyAnimation>
 #include <QTimer>
@@ -30,8 +29,9 @@
 #include <X11/extensions/shape.h>
 #endif
 
-#include <KWindowSystem>
 #include <KDebug>
+#include <KIdleTime>
+#include <KWindowSystem>
 
 #include <Plasma/Containment>
 #include <Plasma/Corona>
@@ -309,6 +309,7 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
       m_glowBar(0),
       m_mousePollTimer(0),
       m_strutsTimer(new QTimer(this)),
+      m_rehideAfterAutounhideTimer(new QTimer(this)),
       m_spacer(0),
       m_spacerIndex(-1),
       m_shadowWindow(0),
@@ -318,7 +319,8 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
       m_visibilityMode(NormalPanel),
       m_lastHorizontal(true),
       m_editing(false),
-      m_triggerEntered(false)
+      m_triggerEntered(false),
+      m_respectStatus(true)
 {
     // KWin setup
     KWindowSystem::setOnAllDesktops(winId(), true);
@@ -328,11 +330,13 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
     m_strutsTimer->setSingleShot(true);
     connect(m_strutsTimer, SIGNAL(timeout()), this, SLOT(updateStruts()));
 
+    // this timer controls checks to re-hide a panel after it's been unhidden
+    // for the user because, e.g., something is demanding attention
+    m_rehideAfterAutounhideTimer->setSingleShot(true);
+    connect(m_rehideAfterAutounhideTimer, SIGNAL(timeout()), this, SLOT(checkAutounhide()));
+
     // Graphics view setup
     setFrameStyle(QFrame::NoFrame);
-    //setAutoFillBackground(true);
-    //setDragMode(QGraphicsView::RubberBandDrag);
-    //setCacheMode(QGraphicsView::CacheBackground);
     setInteractive(true);
     setAcceptDrops(true);
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
@@ -352,10 +356,20 @@ PanelView::PanelView(Plasma::Containment *panel, int id, QWidget *parent)
 
     const bool onScreen = panel->screen() < PlasmaApp::self()->corona()->numScreens();
     const QRect screenRect = onScreen ?  PlasmaApp::self()->corona()->screenGeometry(panel->screen()) : QRect();
-    if (!onScreen) {
+    const int sw = screenRect.width();
+    const int sh = screenRect.height();
+    m_lastSeenSize = sizes.readEntry("lastsize", m_lastHorizontal ? sw : sh);
+
+    if (onScreen) {
+        const QString last = m_lastHorizontal ? "Horizontal" + QString::number(sw) :
+                                                "Vertical" + QString::number(sh);
+        if (sizes.hasGroup(last)) {
+            KConfigGroup thisSize(&sizes, last);
+            resize(thisSize.readEntry("size", m_lastHorizontal ? QSize(sw, 27) : QSize(27, sh)));
+        }
+    } else {
         resize(panel->size().toSize());
     }
-    m_lastSeenSize = sizes.readEntry("lastsize", m_lastHorizontal ? screenRect.width() : screenRect.height());
 
     m_alignment = alignmentFilter((Qt::Alignment)viewConfig.readEntry("Alignment", (int)Qt::AlignLeft));
     KConfigGroup sizeConfig(&viewConfig, (m_lastHorizontal ? "Horizontal" : "Vertical") +
@@ -399,7 +413,7 @@ void PanelView::setContainment(Plasma::Containment *containment)
         disconnect(oldContainment);
     }
 
-    connect(containment, SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(setStatus(Plasma::ItemStatus)));
+    connect(containment, SIGNAL(newStatus(Plasma::ItemStatus)), this, SLOT(statusUpdated(Plasma::ItemStatus)));
     connect(containment, SIGNAL(destroyed(QObject*)), this, SLOT(panelDeleted()));
     connect(containment, SIGNAL(toolBoxToggled()), this, SLOT(togglePanelController()));
     connect(containment, SIGNAL(appletAdded(Plasma::Applet *, const QPointF &)), this, SLOT(appletAdded(Plasma::Applet *)));
@@ -422,10 +436,6 @@ void PanelView::setContainment(Plasma::Containment *containment)
 
     // ensure we aren't overlapping other panels
     const QRect screenRect = PlasmaApp::self()->corona()->screenGeometry(containment->screen());
-    const QRegion availGeom = PlasmaApp::self()->corona()->availableScreenRegion(containment->screen());
-    const int w = containment->size().width();
-    const int h = containment->size().height();
-    const int length = containment->formFactor() == Plasma::Horizontal ? w : h;
 
     View::setContainment(containment);
 
@@ -438,6 +448,11 @@ void PanelView::setContainment(Plasma::Containment *containment)
 
     updateStruts();
     checkShadow();
+
+    // if we are an autohiding panel, then see if the status mandates we do something about it
+    if (m_visibilityMode != NormalPanel && m_visibilityMode != WindowsGoBelow) {
+        checkUnhide(containment->status());
+    }
 }
 
 void PanelView::themeChanged()
@@ -448,6 +463,7 @@ void PanelView::themeChanged()
 
 void PanelView::checkShadow()
 {
+#ifndef Q_WS_WIN
     if (KWindowSystem::compositingActive() && containment()->property("shadowPath").isValid()) {
         if (!m_shadowWindow) {
             m_shadowWindow = new ShadowWindow(this);
@@ -462,12 +478,11 @@ void PanelView::checkShadow()
         if (m_shadowWindow->isValid() && isVisible()) {
             m_shadowWindow->show();
         }
-    } else {
-        if (m_shadowWindow) {
-            m_shadowWindow->deleteLater();
-            m_shadowWindow = 0;
-        }
+    } else if (m_shadowWindow) {
+        m_shadowWindow->deleteLater();
+        m_shadowWindow = 0;
     }
+#endif
 }
 
 void PanelView::setPanelDragPosition(const QPoint &point)
@@ -694,8 +709,6 @@ void PanelView::updatePanelGeometry()
             geom.moveTopLeft(QPoint(screenGeom.center().x() - geom.width()/2 + 1 - geom.width()%2 + m_offset, screenGeom.top()));
         }
 
-        //enable borders if needed
-        //c->setGeometry(QRect(geom.left(), c->geometry().top(), geom.width(), geom.height()));
         break;
 
     case Plasma::LeftEdge:
@@ -707,8 +720,6 @@ void PanelView::updatePanelGeometry()
             geom.moveTopLeft(QPoint(screenGeom.left(), screenGeom.center().y() - geom.height()/2 + 1 - geom.height()%2 + m_offset));
         }
 
-        //enable borders if needed
-        //c->setGeometry(QRect(c->geometry().left(), geom.top(), geom.width(), geom.height()));
         break;
 
     case Plasma::RightEdge:
@@ -720,8 +731,6 @@ void PanelView::updatePanelGeometry()
             geom.moveTopRight(QPoint(screenGeom.right(), screenGeom.center().y() - geom.height()/2 + 1 - geom.height()%2 + m_offset));
         }
 
-        //enable borders if needed
-        //c->setGeometry(QRect(c->geometry().left(), geom.top(), geom.width(), geom.height()));
         break;
 
     case Plasma::BottomEdge:
@@ -734,8 +743,6 @@ void PanelView::updatePanelGeometry()
             geom.moveBottomLeft(QPoint(screenGeom.center().x() - geom.width()/2 + 1 - geom.width()%2 + m_offset, screenGeom.bottom()));
         }
 
-        //enable borders if needed
-        //c->setGeometry(QRect(geom.left(), c->geometry().top(), geom.width(), geom.height()));
         break;
     }
 
@@ -811,8 +818,7 @@ void PanelView::pinchContainment(const QRect &screenGeom)
     KConfigGroup sizes = config();
     sizes = KConfigGroup(&sizes, "Sizes");
 
-    if (m_lastHorizontal != horizontal ||
-        m_lastSeenSize != (horizontal ? sw : sh)) {
+    if (m_lastHorizontal != horizontal || m_lastSeenSize != (horizontal ? sw : sh)) {
         // we're adjusting size. store the current size now
         KConfigGroup lastSize(&sizes, (m_lastHorizontal ? "Horizontal" : "Vertical") +
                                       QString::number(m_lastSeenSize));
@@ -822,8 +828,8 @@ void PanelView::pinchContainment(const QRect &screenGeom)
         lastSize.writeEntry("max", m_lastMax);
         configNeedsSaving();
 
-        QString last = (horizontal ? "Horizontal" : "Vertical") +
-                       QString::number(horizontal ? sw : sh);
+        const QString last = horizontal ? "Horizontal" + QString::number(sw) :
+                                          "Vertical" + QString::number(sh);
         if (sizes.hasGroup(last)) {
             KConfigGroup thisSize(&sizes, last);
 
@@ -844,6 +850,7 @@ void PanelView::pinchContainment(const QRect &screenGeom)
                    (horizontal ? c->geometry().width() :
                                  c->geometry().height()) >= m_lastSeenSize) {
             // we are moving from a smaller space where we are 100% to a larger one
+            kDebug() << "we are moving from a smaller space where we are 100% to a larger one";
             c->setMinimumSize(0, 0);
             c->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
 
@@ -860,6 +867,35 @@ void PanelView::pinchContainment(const QRect &screenGeom)
                     c->setMinimumSize(min.width(), sh);
                 }
             }
+        } else if (m_lastSeenSize > (horizontal ? sw : sh) &&
+                    (m_offset + (horizontal ? c->geometry().width() :
+                                 c->geometry().height())) > (horizontal ? sw : sh)) {
+            kDebug() << "we are moving from a bigger space to a smaller one where the panel won't fit!!";
+            if ((horizontal ? c->geometry().width() :
+                                 c->geometry().height()) > (horizontal ? sw : sh)) {
+                kDebug() << "panel is larger than screen, adjusting panel size";
+                setOffset(0);
+                c->setMinimumSize(0, 0);
+                c->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+
+                if (horizontal) {
+                    c->setMaximumSize(sw, max.height());
+                    c->resize(sw, c->geometry().height());
+                    if (min.width() == max.width()) {
+                        c->setMinimumSize(sw, min.height());
+                    }
+                } else {
+                    c->setMaximumSize(max.width(), sh);
+                    c->resize(c->geometry().width(), sh);
+                    if (min.height() == max.height()) {
+                        c->setMinimumSize(min.width(), sh);
+                    }
+                }
+            } else {
+                kDebug() << "reducing offset so the panel fits in screen";
+                setOffset((horizontal ? sw : sh) -
+                          (horizontal ? c->geometry().width() : c->geometry().height()));
+            }
         }
     }
 
@@ -996,17 +1032,23 @@ void PanelView::togglePanelController()
         }
     }
 
-    if (!m_panelController->isVisible()) {
+    if (m_panelController->isVisible()) {
+        if (m_panelController->showingWidgetExplorer() ||
+            m_panelController->showingActivityManager()) {
+            m_panelController->switchToController();
+            m_panelController->move(m_panelController->positionForPanelGeometry(geometry()));
+        } else {
+            Plasma::WindowEffects::slideWindow(m_panelController, location());
+            m_panelController->close();
+            updateStruts();
+        }
+    } else {
         m_editing = true;
         m_panelController->resize(m_panelController->sizeHint());
         m_panelController->move(m_panelController->positionForPanelGeometry(geometry()));
         Plasma::WindowEffects::slideWindow(m_panelController, location());
         kDebug() << "showing panel controller!" << m_panelController->geometry();
         m_panelController->show();
-    } else {
-        Plasma::WindowEffects::slideWindow(m_panelController, location());
-        m_panelController->close();
-        updateStruts();
     }
 }
 
@@ -1396,7 +1438,7 @@ void PanelView::unhide(bool destroyTrigger)
     }
 }
 
-void PanelView::setStatus(Plasma::ItemStatus newStatus)
+void PanelView::statusUpdated(Plasma::ItemStatus newStatus)
 {
     if (newStatus == Plasma::AcceptingInputStatus) {
         KWindowSystem::forceActiveWindow(winId());
@@ -1405,14 +1447,50 @@ void PanelView::setStatus(Plasma::ItemStatus newStatus)
 
 void PanelView::checkUnhide(Plasma::ItemStatus newStatus)
 {
-    //kDebug() << "================= got a new status: " << newStatus << Plasma::ActiveStatus;
+    //kDebug() << "================= got a status: " << newStatus << Plasma::ActiveStatus;
+    m_respectStatus = true;
+
     if (newStatus > Plasma::ActiveStatus) {
         unhide();
+        if (newStatus == Plasma::NeedsAttentionStatus) {
+            //kDebug() << "starting the timer!";
+            // start our rehide timer, so that the panel doesn't stay up and stuck forever and a day
+            m_rehideAfterAutounhideTimer->start(AUTOUNHIDE_CHECK_DELAY);
+        }
     } else {
+        //kDebug() << "new status, just autohiding";
         startAutoHide();
     }
 }
 
+void PanelView::checkAutounhide()
+{
+    //kDebug() << "***************************" << KIdleTime::instance()->idleTime();
+    if (KIdleTime::instance()->idleTime() >= AUTOUNHIDE_CHECK_DELAY) {
+        // the user is idle .. let's not hige the panel on them quite yet, but rather given them a
+        // chance to see this thing!
+        connect(KIdleTime::instance(), SIGNAL(resumingFromIdle()), this, SLOT(checkAutounhide()),
+                Qt::UniqueConnection);
+        KIdleTime::instance()->catchNextResumeEvent();
+        //kDebug() << "exit 1 ***************************";
+        return;
+    }
+
+    m_respectStatus = false;
+    //kDebug() << "in to check ... who's resonsible?" << sender() << KIdleTime::instance();
+    if (sender() == KIdleTime::instance()) {
+        //kDebug() << "doing a 2s wait";
+        QTimer::singleShot(2000, this, SLOT(startAutoHide()));
+    } else {
+        //kDebug() << "just starting autohide!";
+        startAutoHide();
+    }
+
+    // this line must come after the check on sender() as it *clears* that value!
+    disconnect(KIdleTime::instance(), SIGNAL(resumingFromIdle()), this, SLOT(checkAutounhide()));
+    //kDebug() << "exit 0 ***************************";
+}
+
 void PanelView::unhide()
 {
     unhide(true);
@@ -1425,10 +1503,31 @@ void PanelView::resetTriggerEnteredSuppression()
 
 void PanelView::startAutoHide()
 {
-    //TODO: is 5s too long? not long enough?
-    if (m_editing || (containment() && containment()->status() > Plasma::ActiveStatus) ||
-        geometry().adjusted(-10, -10, 10, 10).contains(QCursor::pos()) ||
-        hasPopup()) {
+    /*
+    kDebug() << m_editing << (containment() ? containment()->status() : 0) << Plasma::ActiveStatus
+             << geometry().adjusted(-10, -10, 10, 10).contains(QCursor::pos()) << hasPopup();
+    if (containment() && containment()->status() > Plasma::ActiveStatus) {
+        foreach (Plasma::Applet *applet, containment()->applets()) {
+            kDebug() << "     " << applet->name() << applet->status();
+        }
+    }
+    */
+
+
+    if (m_editing || (m_respectStatus && (containment() && containment()->status() > Plasma::ActiveStatus))) {
+        if (m_mousePollTimer) {
+            m_mousePollTimer->stop();
+            disconnect(m_mousePollTimer, SIGNAL(timeout()), this, SLOT(startAutoHide()));
+        }
+
+        return;
+    }
+
+    // since we've gotten this far, we don't need to worry about rehiding-after-auto-unhide, so just
+    // stop the timer
+    m_rehideAfterAutounhideTimer->stop();
+
+    if (geometry().adjusted(-10, -10, 10, 10).contains(QCursor::pos()) || hasPopup()) {
         if (!m_mousePollTimer) {
             leaveEvent(0);
         }
