@@ -605,7 +605,7 @@ void Battery::initExtenderItem(Plasma::ExtenderItem *item)
         action->setIcon(KIcon("configure"));
         action->setText(i18n("Power save settings"));
         connect(action, SIGNAL(triggered(bool)), this, SLOT(openConfig()));
-        connect(configButton, SIGNAL(activated()), this, SLOT(openConfig()));
+        connect(configButton, SIGNAL(clicked()), this, SLOT(openConfig()));
         addAction("configure_powersave", action);
         configButton->setIcon(action->icon());
         configButton->setToolTip(action->toolTip());
@@ -765,8 +765,10 @@ void Battery::openConfig()
 {
     //kDebug() << "opening powermanagement configuration dialog";
     QStringList args;
-    args << "powerdevilglobalconfig";
-    args << "powerdevilprofilesconfig";
+    args << QLatin1String("--icon")
+        << QLatin1String("preferences-system-power-management")
+        << QLatin1String("powerdevilglobalconfig")
+        << QLatin1String("powerdevilprofilesconfig");
     KToolInvocation::kdeinitExec("kcmshell4", args);
 }
 
