@@ -85,6 +85,7 @@ Applet::Applet(QObject *parent, const QVariantList &arguments)
     : Plasma::PopupApplet(parent, arguments),
       m_taskArea(0),
       m_background(0),
+      m_separator(0),
       m_firstRun(true)
 {
     if (!s_manager) {
@@ -98,6 +99,9 @@ Applet::Applet(QObject *parent, const QVariantList &arguments)
     m_background = new Plasma::FrameSvg(this);
     m_background->setImagePath("widgets/systemtray");
     m_background->setCacheAllRenderedFrames(true);
+    m_separator = new Plasma::FrameSvg(this);
+    m_separator->setImagePath("widgets/line");
+    m_separator->setCacheAllRenderedFrames(true);
     m_taskArea = new TaskArea(this);
     lay->addItem(m_taskArea);
     connect(m_taskArea, SIGNAL(toggleHiddenItems()), this, SLOT(togglePopup()));
@@ -107,6 +111,7 @@ Applet::Applet(QObject *parent, const QVariantList &arguments)
 
     setPopupIcon(QIcon());
     setPassivePopup(false);
+    setPopupAlignment(Qt::AlignRight);
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
     setHasConfigurationInterface(true);
 }
@@ -151,7 +156,7 @@ void Applet::init()
             this, SLOT(propogateSizeHintChange(Qt::SizeHint)));
 
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
-            this, SLOT(checkSizes()));
+            this, SLOT(themeChanged()));
 
     QTimer::singleShot(0, this, SLOT(checkDefaultApplets()));
     configChanged();
@@ -257,6 +262,12 @@ QSet<Task::Category> Applet::shownCategories() const
     return m_shownCategories;
 }
 
+void Applet::themeChanged()
+{
+    checkSizes();
+    update();
+}
+
 void Applet::checkSizes()
 {
     Plasma::FormFactor f = formFactor();
@@ -266,23 +277,13 @@ void Applet::checkSizes()
     m_background->getMargins(leftMargin, topMargin, rightMargin, bottomMargin);
 
     QSizeF minSize = m_taskArea->effectiveSizeHint(Qt::MinimumSize);
-    if (f == Plasma::Horizontal && minSize.height() >= size().height() - topMargin - bottomMargin) {
-        m_background->setElementPrefix(QString());
-        m_background->setEnabledBorders(Plasma::FrameSvg::LeftBorder | Plasma::FrameSvg::RightBorder);
-        m_background->setElementPrefix("lastelements");
+    if (f == Plasma::Horizontal && minSize.height() > size().height() - topMargin - bottomMargin) {
         m_background->setEnabledBorders(Plasma::FrameSvg::LeftBorder | Plasma::FrameSvg::RightBorder);
         layout()->setContentsMargins(leftMargin, 0, rightMargin, 0);
-    } else if (f == Plasma::Vertical && minSize.width() >= size().width() - leftMargin - rightMargin) {
-        m_background->setElementPrefix(QString());
-        m_background->setEnabledBorders(Plasma::FrameSvg::TopBorder | Plasma::FrameSvg::BottomBorder);
-        m_background->setElementPrefix("lastelements");
+    } else if (f == Plasma::Vertical && minSize.width() > size().width() - leftMargin - rightMargin) {
         m_background->setEnabledBorders(Plasma::FrameSvg::TopBorder | Plasma::FrameSvg::BottomBorder);
         layout()->setContentsMargins(0, topMargin, 0, bottomMargin);
     } else {
-        m_background->setElementPrefix(QString());
-        m_background->setEnabledBorders(Plasma::FrameSvg::AllBorders);
-        m_background->setElementPrefix("lastelements");
-        m_background->setEnabledBorders(Plasma::FrameSvg::AllBorders);
         layout()->setContentsMargins(leftMargin, topMargin, rightMargin, bottomMargin);
     }
 
@@ -386,19 +387,19 @@ void Applet::paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *o
 
     if (leftEasement > 0) {
         if (formFactor() == Plasma::Vertical) {
-            if (m_background->hasElement("horizontal-separator")) {
-                QSize s = m_background->elementRect("horizontal-separator").size().toSize();
-                m_background->paint(painter, QRect(normalRect.topLeft() - QPoint(0, s.height() / 2),
-                                    QSize(normalRect.width(), s.height())), "horizontal-separator");
+            if (m_separator->hasElement("horizontal-line")) {
+                QSize s = m_separator->elementRect("horizontal-line").size().toSize();
+                m_separator->paint(painter, QRect(normalRect.topLeft() - QPoint(0, s.height() / 2),
+                                    QSize(normalRect.width(), s.height())), "horizontal-line");
             }
-        } else if (m_background->hasElement("vertical-separator")) {
-            QSize s = m_background->elementRect("vertical-separator").size().toSize();
+        } else if (m_separator->hasElement("vertical-line")) {
+            QSize s = m_separator->elementRect("vertical-line").size().toSize();
             if (QApplication::layoutDirection() == Qt::RightToLeft) {
-                m_background->paint(painter, QRect(normalRect.topRight() - QPoint(s.width() / 2, 0),
-                                    QSize(s.width(), normalRect.height())), "vertical-separator");
+                m_separator->paint(painter, QRect(normalRect.topRight() - QPoint(s.width() / 2, 0),
+                                    QSize(s.width(), normalRect.height())), "vertical-line");
             } else {
-                m_background->paint(painter, QRect(normalRect.topLeft() - QPoint(s.width() / 2, 0),
-                                    QSize(s.width(), normalRect.height())), "vertical-separator");
+                m_separator->paint(painter, QRect(normalRect.topLeft() - QPoint(s.width() / 2, 0),
+                                    QSize(s.width(), normalRect.height())), "vertical-line");
             }
         }
     }
@@ -426,8 +427,11 @@ void Applet::createConfigurationInterface(KConfigDialog *parent)
             unlockAction = containment()->corona()->action("lock widgets");
         }
         if (unlockAction) {
+            disconnect(m_visibleItemsUi.unlockButton, SIGNAL(clicked()), this, SLOT(unlockContainment()));
+            connect(m_visibleItemsUi.unlockButton, SIGNAL(clicked()), unlockAction, SLOT(trigger()), Qt::UniqueConnection);
+        } else {
             disconnect(m_visibleItemsUi.unlockButton, SIGNAL(clicked()), unlockAction, SLOT(trigger()));
-            connect(m_visibleItemsUi.unlockButton, SIGNAL(clicked()), unlockAction, SLOT(trigger()));
+            connect(m_visibleItemsUi.unlockButton, SIGNAL(clicked()), this, SLOT(unlockContainment()), Qt::UniqueConnection);
         }
 
 
@@ -604,6 +608,14 @@ void Applet::createConfigurationInterface(KConfigDialog *parent)
     }
 }
 
+//not always the corona lock action is available: netbook locks per-containment
+void Applet::unlockContainment()
+{
+    if (containment() && containment()->immutability() == Plasma::UserImmutable) {
+        containment()->setImmutability(Plasma::Mutable);
+    }
+}
+
 void Applet::configAccepted()
 {
     KConfigGroup cg = config();
