@@ -31,7 +31,7 @@
 MprisFactory::MprisFactory(QObject* parent)
     : DBusPlayerFactory(parent)
 {
-    setObjectName("MprisFactory");
+    setObjectName( QLatin1String("MprisFactory" ));
     qDBusRegisterMetaType<MprisDBusVersion>();
     qDBusRegisterMetaType<MprisDBusStatus>();
 }
@@ -82,7 +82,8 @@ Mpris::~Mpris()
     delete m_player;
 }
 
-void Mpris::setup() {
+void Mpris::setup()
+{
     delete m_player;
     m_player = new MprisPlayer(
             m_playerName,
@@ -189,6 +190,14 @@ QString Mpris::genre()
     return QString();
 }
 
+QString Mpris::lyrics()
+{
+    if (m_metadata.contains("lyrics")) {
+        return m_metadata["lyrics"].toString();
+    }
+    return QString();
+}
+
 int Mpris::length()
 {
     if (m_metadata.contains("time")) {
