@@ -22,6 +22,7 @@
 
 #include <KIcon>
 #include <QSqlDatabase>
+#include <QMimeData>
 #include <Plasma/AbstractRunner>
 
 
@@ -29,7 +30,7 @@ class KBookmark;
 class KBookmarkManager;
 class KJob;
 
-/** This runner searchs for bookmarks in browsers like Konqueror and Firefox */
+/** This runner searchs for bookmarks in browsers like Konqueror, Firefox and Opera */
 class BookmarksRunner : public Plasma::AbstractRunner
 {
     Q_OBJECT
@@ -51,9 +52,10 @@ class BookmarksRunner : public Plasma::AbstractRunner
           * -adding a statement to whichBrowser()
           * @see whichBrowser()
           */
-        enum Browser { Konqueror, ///< the browser is Konqueror
+        enum Browser {
                        Firefox, ///< the browser is Firefox
-                       Default = Konqueror ///< Konqueror is default
+                       Opera, ///< the browser is Opera
+                       Default ///< KDE bookmarks
                      };
 
         /** @returns the favicon for the url */
@@ -66,6 +68,10 @@ class BookmarksRunner : public Plasma::AbstractRunner
         void matchKonquerorBookmarks(Plasma::RunnerContext& context, bool allBookmarks,
                                                         const QString& term);
 
+        /** Get the bookmarks from Opera */
+        void matchOperaBookmarks(Plasma::RunnerContext& context, bool allBookmarks,
+                                                        const QString& term);
+
         /** @returns the browser to get the bookmarks from
           * @see Browser
           */
@@ -79,6 +85,10 @@ class BookmarksRunner : public Plasma::AbstractRunner
         QString m_dbCacheFile;
         QSqlDatabase m_db;
         KBookmarkManager *m_bookmarkManager;
+        QStringList m_operaBookmarkEntries;
+
+    protected Q_SLOTS:
+        QMimeData * mimeDataForMatch(const Plasma::QueryMatch *match);
 
     private Q_SLOTS:
         void prep();
