@@ -1,2 +1,5 @@
 #! /usr/bin/env bash
+$EXTRACTRC *.ui >> rc.cpp
 $XGETTEXT *.cpp -o $podir/kcm_solid.pot
+rm -f rc.cpp
+
