@@ -30,7 +30,7 @@ ActionEditor::ActionEditor(QWidget *parent) : KDialog(parent)
     rootModel = new PredicateModel( topItem, this );
     // Prepare the dialog
     setInitialSize( QSize(600, 600) ); // Set a decent initial size
-    setModal( true );
+    // setModal( true );
     // Set up the interface
     ui.setupUi( mainWidget() );
     ui.TvPredicateTree->setHeaderHidden( true );
@@ -65,7 +65,7 @@ void ActionEditor::setActionToEdit( ActionItem * item )
     // Set all the text appropriately
     ui.IbActionIcon->setIcon( item->icon() );
     ui.LeActionFriendlyName->setText( item->name() );
-    ui.LeActionCommand->setPath( item->exec() );
+    ui.LeActionCommand->setUrl( KUrl(item->exec()) );
 
     setPredicate( item->predicate() );
     setCaption( i18n("Editing Action %1", item->name()) ); // Set a friendly i18n caption
